package com.uoc.tfc.Base.common;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.CipherOutputStream;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;

import org.apache.xml.security.utils.Base64;


/**
 * Encriptaci basada en una paraula clau (PBE, PKCS#5). Algorisme DES, disponible en totes les JVM.
 * L'encriptaci d'strings es fa: encoding=utf-8 i finalment es codifica en Base64.
 *
 * @author ojaen
 *
 */
public class Encriptacio {

	private static final String ALGORISME_ENCRIPTACIO = "PBEWithMD5AndDES"; //$NON-NLS-1$
	private static final String ENCODING = "UTF-8"; //$NON-NLS-1$
	private static final int MIDA_BUFFER_STREAMS=1024;

	/** Parametre SALT de PBE(PKCS#5)*/
	private static final byte[] SALT = {
		(byte)0xA9, (byte)0x9B, (byte)0xC8, (byte)0x32, (byte)0x56, (byte)0x35, (byte)0xE3, (byte)0x03};

	/** Parametre ITERACIONS de PBE(PKCS#5)*/
	private static final int ITERACIONS = 19;

	/** Parametre PASSWORD de PBE(PKCS#5)*/
	private static final String PASSWORD = "UOCTFC"; //$NON-NLS-1$

	private static Encriptacio instance;

	private Cipher encriptador = null;
	private Cipher desencriptador = null;

	/**
	 * Obte l'objecte que pot d'encriptar i desencriptar.
	 * @return Encriptacio
	 * @throws EncriptacioExcepcio
	 */
	public static Encriptacio getInstance() throws Throwable{
		if (instance==null){
			instance=new Encriptacio(PASSWORD,SALT,ITERACIONS);
		}
		return instance;
	}


	/**
	 * Inicialitza l'encriptacio.
	 */
	private Encriptacio(String p_clau,byte[] p_salt,int p_iteracions) throws Throwable {
		if(p_clau == null)
			throw new IllegalArgumentException("Falta indicar la clau.");

		// generem la clau secreta a partir de la paraula
		PBEParameterSpec algorithm = new PBEParameterSpec(SALT, ITERACIONS);
		PBEKeySpec pbeKeySpec = new PBEKeySpec(p_clau.toCharArray());
		SecretKeyFactory keyBuilder = SecretKeyFactory.getInstance(ALGORISME_ENCRIPTACIO);
		SecretKey key = keyBuilder.generateSecret(pbeKeySpec);

		// instanciem el encriptador i el desencriptador
		this.encriptador = javax.crypto.Cipher.getInstance(ALGORISME_ENCRIPTACIO);
		this.desencriptador = javax.crypto.Cipher.getInstance(ALGORISME_ENCRIPTACIO);

		// inicialitzem encriptador i desencriptador
		this.encriptador.init(Cipher.ENCRYPT_MODE, key, algorithm);
		this.desencriptador.init(Cipher.DECRYPT_MODE, key, algorithm);

	}

	/**
	 * Encripta un string
	 * @param p_entrada
	 * @return String
	 * @throws EncriptacioExcepcio
	 */
	public String encriptar(String p_entrada) throws Throwable {
		byte[] bytesEncriptats = this.encriptador.doFinal(p_entrada.getBytes(ENCODING));
		return Base64.encode(bytesEncriptats);
	}

	/**
	 * Encripta el missatge contingut a "p_entrada" sobre p_sortida.
	 * @param p_entrada
	 * @param p_sortida
	 * @throws EncriptacioExcepcio
	 */
	public void encriptar(InputStream p_entrada, OutputStream p_sortida) throws Throwable {

		p_sortida = new CipherOutputStream (p_sortida, encriptador);
		volcarContingut(p_entrada, p_sortida);
	}

	/**
	 * Desencripta el contingut del parametre "p_entrada" i retorna el resultat.
	 * @param p_entrada
	 * @return String
	 * @throws EncriptacioExcepcio
	 */
	public String desencriptar(String p_entrada) throws Throwable {

		byte[] bytesEncriptats = Base64.decode(p_entrada);
		byte[] uft8Desencriptat = desencriptador.doFinal(bytesEncriptats);
		return new String(uft8Desencriptat,ENCODING);
	}

	/**
	 * Desencripta el missatge contigut a p_entrada sobre p_sortida. 
	 * @param p_entrada
	 * @param p_sortida
	 * @throws EncriptacioExcepcio
	 */
	public void desencriptar(InputStream p_entrada, OutputStream p_sortida) throws Throwable {

		p_entrada = new CipherInputStream(p_entrada, desencriptador);
		volcarContingut(p_entrada, p_sortida);
	}

	/**
	 * Volca el contingut de l'entrada sobre la sortida.
	 * Tanca els 2 streams.
	 * @param p_entrada
	 * @param p_sortida
	 * @throws IOException
	 */
	public static void volcarContingut(InputStream p_entrada, OutputStream p_sortida)
	throws IOException
	{
		try{
			byte[] buffer = new byte[MIDA_BUFFER_STREAMS];
			int llegit=p_entrada.read(buffer);
			while(llegit>=0){
				p_sortida.write(buffer,0,llegit);
				llegit=p_entrada.read(buffer);
			}
		} finally {
			if (p_sortida!=null) p_sortida.close();
			if (p_entrada!=null) p_entrada.close();
		}
	}


}