package com.uoc.tfc.Base.common;

import java.util.Collection;
import java.util.Date;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * Classe que conte ajudes per la programaci
 * @author ojaen
 */
public class Helper{


	/**
	 * 
	 * Mtode d'ajuda per verificar un array.
	 * 
	 * @param p_array
	 * @return boolean
	 * 
	 */
	public static boolean validarArray(Object[] p_array) {

		if (p_array == null)
			return false;
		if (p_array.length == 0)
			return false;
		return true;
	}

	/**
	 * Mtode per a validar un array i la seva longitud mnima
	 * 
	 * @param p_array
	 * @param longMin
	 * @return boolean
	 */
	public static boolean validarArray(Object[] p_array, int longMin) {

		if (!validarArray(p_array))
			return false;
		if (p_array.length < longMin)
			return false;
		return true;
	}

	/**
	 * 
	 * Mtode per validar una colecci
	 * 
	 * @param p_coleccio
	 * @return boolean
	 * 
	 */
	public static boolean validarColleccio(Collection p_coleccio) {

		if (p_coleccio == null)
			return false;
		if (p_coleccio.size() == 0)
			return false;
		return true;
	}

	/**
	 * Mtode per a validar una colecci i la seva longitud mnima
	 * 
	 * @param p_coleccio
	 * @param longMin :
	 *            longitud mnima que ha de tenir la colecci per retornar true
	 * @return boolean
	 */
	public static boolean validarColleccio(Collection p_coleccio, int longMin) {

		if (!validarColleccio(p_coleccio))
			return false;
		if (p_coleccio.size() < longMin)
			return false;
		return true;
	}

	/**
	 * 
	 * Mtode per validar un map
	 * 
	 * @param p_map
	 * @return boolean
	 * 
	 */
	public static boolean validarMapa(Map p_map) {

		if (p_map == null)
			return false;
		if (p_map.size() == 0)
			return false;
		return true;
	}

	/**
	 * 
	 * Mtode per validar si una cadena es valida per comparar.
	 * 
	 * @param p_cadena
	 * @return boolean
	 * 
	 */
	public static boolean validarCadena(String p_cadena) {

		if (p_cadena == null)
			return false;
		if (p_cadena.equals("")) //$NON-NLS-1$
			return false;
		return true;
	}

	/**
	 * 
	 * Mtode que serveix per determinar si un integer es true o false. Si s null o diferent a 1, sera false.
	 * 
	 * @param p_integer
	 * @return boolean
	 * 
	 */
	public static boolean esIntegerTrue(Integer p_integer) {

		if (p_integer == null)
			return false;
		if (p_integer.intValue() != 1)
			return false;
		return true;
	}

	/**
	 * 
	 * Mtode per comprovar que un mapa cont una clau i el seu valor no es null
	 * 
	 * @param p_map
	 * @param p_clau
	 * @return boolean
	 * 
	 */
	public static boolean validarContingutMapa(Map p_map, Object p_clau) {

		if (!validarMapa(p_map))
			return false;
		if (p_clau == null)
			return false;
		if (p_map.get(p_clau) == null)
			return false;
		if (p_map.get(p_clau) instanceof String){
			if (!validarCadena((String) p_map.get(p_clau))){
				return false;
			}
		}

		return true;
	}


	/**
	 * 
	 * Determina si un valor est dins d'un interval definit.
	 * Si noms l'inici esta definit, el complir si es major que l'inici
	 * Si nomes el fi esta definit, el complir si es menor que el fi.
	 * Si cap dels dos esta definit, retornara false.
	 *@param p_valorRefencia
	 *@param p_valorInicial
	 *@param p_valorFinal
	 *@return boolean
	 *
	 */
	public static boolean validarInterval(Date p_valorRefencia, Date p_valorInicial, Date p_valorFinal) {

		if (p_valorRefencia != null) {
			if (p_valorInicial != null && p_valorFinal != null) {
				if (p_valorRefencia.after(p_valorInicial)
						&& p_valorRefencia.before(p_valorFinal) ) {
					return true;
				}
			} else if (p_valorInicial != null) {
				if (p_valorRefencia.after(p_valorInicial)) {
					return true;
				}
			} else if (p_valorFinal != null) {
				if (p_valorRefencia.before(p_valorFinal)) {
					return true;
				}
			}
		}
		return false;
	}




	/**
	 * 
	 * Retorna true si p_string esta contingut a p_list, false en altre cas.
	 *@param p_string
	 *@param p_array
	 *@return boolean
	 *
	 */
	public static boolean isStringInArray(String p_string, Object[] p_array) {
		if(!Helper.validarCadena(p_string))
			return false;
		if(!Helper.validarArray(p_array))
			return false;
		for(int i=0; i<p_array.length; i++) {
			if(p_string.equals(p_array[i]))
				return true;
		}
		return false;
	}

	/**
	 * 
	 * Retorna true si p_string esta contingut a p_list, false en altre cas.
	 *@param p_string
	 *@param p_coleccio
	 *@return boolean
	 *
	 */
	public static boolean isStringInCollection(String p_string, Collection p_coleccio) {
		return isStringInArray(p_string, p_coleccio.toArray());
	}

	/**
	 * 
	 * Mtode que compara si dos Strings sn iguals. Pero considera la possiblitat de que un o els dos objectes
	 * siguin null
	 * 
	 * @param p_str1
	 * @param p_str2
	 * @return boolean
	 * 
	 */
	public static boolean compararStrings(String p_str1, String p_str2, boolean p_nullIsBuit) {
		if(p_nullIsBuit) {
			if(p_str1==null) {
				p_str1 = ""; //$NON-NLS-1$
			}
			if(p_str2==null) {
				p_str2 = ""; //$NON-NLS-1$
			}
		}

		if (p_str1 != null) {
			if (p_str2 != null) {
				return (p_str1.equals(p_str2));
			} else {
				return false;
			}
		} else {
			if (p_str2 != null) {
				return false;
			}
		}
		return true;
	}


	/**
	 * 
	 * Mtode que compara si dos Strings son iguals. Pero considera la possiblitat de que un o els dos objectes
	 * siguin null
	 * 
	 * @param p_str1
	 * @param p_str2
	 * @return boolean
	 * 
	 */
	public static boolean compararStrings(String p_str1, String p_str2) {
		return compararStrings(p_str1, p_str2, false);
	}

	/**
	 * 
	 * Mtode que compara si dos Dates son iguals. Pero considera la possiblitat de que un o els dos objectes
	 * siguin null
	 * 
	 * @param p_data1
	 * @param p_data2
	 * @return boolean
	 * 
	 */
	public static boolean compararDates(Date p_data1, Date p_data2) {
		if (p_data1 != null) {
			if (p_data2 != null) {
				return (p_data1.equals(p_data2));
			} else {
				return false;
			}
		} else {
			if (p_data2 != null) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Verificar si la sessi ha expirat
	 * @param request
	 * @return boolean
	 */
	public static boolean isSessioExpirada(HttpServletRequest request) {
		HttpSession session = request.getSession(false);
		if(ConfiguracioUsuari.getInstance() != null && 
		 ConfiguracioUsuari.getInstance().getUsuari() != null &&
			(session==null || session.getAttribute("initialized") == null) ) 
			return true;
		return false;
	}


}
