package com.uoc.tfc.Base.common;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.StringTokenizer;

/**
 * Classe per configurar els formats de les dates, hores, numeros
 * @author ojaen
 *
 */
public final class Internacionalitzacio {
	private static DecimalFormat formatSencers = null;
	private static DecimalFormat formatNumerics = null;
	private static DateFormat formatDates;
	private static DateFormat formatDatesCurtes;
	private static DateFormat formatHores;
	private static String formatData;
	private static String formatNum;



	/**
	 * Parsejar un string en una data forma llarg HH:mm:ss
	 * @param p_hora
	 * @return Date
	 * @throws ParseException
	 */
	public static Date parsejaHora(String p_hora) throws ParseException {
		return formatHores.parse(p_hora);
	}



	/**
	 * Parsejar un string en una data format curt HH:mm
	 * @param p_data
	 * @return Date
	 * @throws ParseException
	 */
	public static Date parsejaDataCurta(String p_data) throws ParseException {
		return formatDatesCurtes.parse(p_data);
	}


	/**
	 * Parsetjar un string en una data
	 * @param p_data
	 * @return Date
	 * @throws ParseException
	 */
	public static Date parsejaData(String p_data) throws ParseException {
		return formatDates.parse(p_data);
	}

	/**
	 * Parsejar un string a un sencer
	 * @param p_numero
	 * @return int
	 * @throws ParseException
	 */
	public static int parsejaSencer(String p_numero) throws ParseException {
		return formatSencers.parse(p_numero).intValue();
	}

	/**
	 * Informem el format Numeric i de dates. 
	 **/
	public static void Configurar() {
		String fd = null;

		char puntDecimal = ','; 
		char separadorMilers = '.'; 


		formatNumerics = (DecimalFormat) DecimalFormat.getInstance();

		formatSencers = (DecimalFormat) formatNumerics.clone();
		formatSencers.setMaximumFractionDigits(0);
		formatSencers.setMinimumFractionDigits(0);

		formatData = "dd/MM/yyyy HH:mm:ss"; //$NON-NLS-1$
		formatDates = new SimpleDateFormat(formatData);

		fd = "dd/MM/yyyy"; //$NON-NLS-1$
		formatDatesCurtes = new SimpleDateFormat(fd);

		fd = "HH:mm:ss"; //$NON-NLS-1$
		formatHores = new SimpleDateFormat(fd);

		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator(puntDecimal);
		dfs.setGroupingSeparator(separadorMilers);
		formatNumerics.setDecimalFormatSymbols(dfs);
		formatSencers.setDecimalFormatSymbols(dfs);

	}

	/**
	 * Getter del format de la data
	 * @return
	 */
	public static String getFormatData() {
		return formatData;
	}
	
	/**
	 * Getter del format de la data curta
	 * @return DateFormat
	 */
	public static DateFormat getFormatDatesCurtes() {
		return formatDatesCurtes;
	}
	
	/**
	 * Getter format de les hores
	 * @return DateFormat
	 */
	public static DateFormat getFormatHores() {
		return formatHores;
	}
	
	/**
	 * Getter del format numric
	 * @return String
	 */
	public static String getFormatNum() {
		return formatNum;
	}


	private Internacionalitzacio(){}


	/**
	 * Constructor estatic. En un principi el fem moure amb un constructor estatic
	 * 
	 * */
	static {
		formatNumerics = (DecimalFormat) DecimalFormat.getInstance();
		formatSencers = (DecimalFormat) DecimalFormat.getInstance();
		formatSencers.setMinimumFractionDigits(0);
		formatSencers.setMaximumFractionDigits(0);
		formatDates = DateFormat.getDateInstance();
		formatDatesCurtes = DateFormat.getDateInstance();
		formatHores = DateFormat.getDateInstance();
		formatData = "dd/MM/yyyy HH:mm:ss"; //$NON-NLS-1$
		formatNum = "###,###,###,##0.00"; //$NON-NLS-1$

	}
	/**
	 * 
	 * @return Retorna el Format per defecte dels numeros
	 */
	public static DateFormat getDataFormat(){
		return formatDates;
	}

	/**
	 * Indicar el format de la data que volem utilitzar
	 * @param p_format
	 */
	public static void setDataFormat(String p_format) {

		String fd = null;
		String fh = null;

		StringTokenizer st = new StringTokenizer(p_format," ");				 //$NON-NLS-1$
		fd = st.nextToken();					
		fh = st.nextToken();

		formatDates = new SimpleDateFormat(p_format);
		formatDatesCurtes = new SimpleDateFormat(fd);
		formatHores = new SimpleDateFormat(fh);
	}

	/**
	 * Ens retornar la data formatejada
	 * @param p_data
	 * @return String
	 */
	public static String FormataData(Date p_data) {
		return formatDates.format(p_data);
	}

	/**
	 * Ens retorna l'hora formatejada
	 * @param p_hora
	 * @return String
	 */
	public static String FormataHora(Date p_hora) {
		return formatHores.format(p_hora);
	}

	/**
	 * Ens retorna la data en format curt
	 * @param p_dataCurta
	 * @return String
	 */
	public static String FormataDataCurta(Date p_dataCurta) {
		return formatDatesCurtes.format(p_dataCurta);
	}

	/**
	 * Ens retorna un numero formatejat
	 * @return NumberFormat
	 */
	public static NumberFormat getNumericFormat(){
		return formatNumerics;
	}

	/**
	 * Indiquem el format que volem utilitzar en els formats numerics i sencers
	 * @param p_format
	 */
	public static void setNumericFormat(String p_format) {
		formatNumerics = new DecimalFormat(p_format);
		formatSencers = new DecimalFormat(p_format);
		formatSencers.setMaximumFractionDigits(0);
		formatSencers.setMinimumFractionDigits(0);

	}

	/**
	 * Indiquem el format que volem utilitzar en els formats numerics (sense decimals)
	 * @param p_numero
	 * @return String
	 */
	public static String FormataNumeric(double p_numero){
		return FormataNumeric(p_numero, 0, true);
	}

	/**
	 * Ens retorna el numero formatejat indicant el numero de decimals 
	 * @param p_numero
	 * @param p_decimals
	 * @return String
	 */
	public static String FormataNumeric(double p_numero, int p_decimals) {
		return FormataNumeric(p_numero, p_decimals , true);
	}



	/**
	 * Formatejar un numero indicant 
	 * @param p_numero
	 * @param p_decimals
	 * @param p_mostrarseparadors
	 * @return String
	 */
	public static String FormataNumeric(double p_numero, int p_decimals,
			boolean p_mostrarseparadors) {
		boolean milers = formatNumerics.isGroupingUsed();
		formatNumerics.setGroupingUsed(p_mostrarseparadors);
		String p ; 
		if (p_decimals ==0)
		{
			milers = formatSencers.isGroupingUsed();
			formatSencers.setGroupingUsed(p_mostrarseparadors);
			p = formatSencers.format(p_numero);
			formatSencers.setGroupingUsed(milers); //restauramos los millares
		}
		else
		{
			int cuantos = formatNumerics.getMinimumFractionDigits(); 
			formatNumerics.setMinimumFractionDigits(p_decimals);
			formatNumerics.setMaximumFractionDigits(p_decimals);
			p = formatNumerics.format(p_numero);
			formatNumerics.setMinimumFractionDigits(cuantos);
			formatNumerics.setMaximumFractionDigits(cuantos);
		}
		formatNumerics.setGroupingUsed(milers); //restauramos los millares
		return p;
	}


	/**
	 * Formatejar un Double
	 * @param p_numero
	 * @return String
	 */
	public static String FormataNumeric(Double p_numero){
		return FormataNumeric(p_numero.doubleValue());
	}

	/**
	 * Formatejar un double indicant el numero de decimals
	 * @param p_numero
	 * @param p_decimals
	 * @return String
	 */
	public static String FormataNumeric(Double p_numero, int p_decimals) {
		return FormataNumeric(p_numero.doubleValue(), p_decimals);
	}

	/**
	 * Formatejar un double indicant el numero de decimals indicant si volem mostrar separadors de milers
	 * @param p_numero
	 * @param p_decimals
	 * @param p_mostrarseparadors
	 * @return String
	 */
	public static String FormataNumeric(Double p_numero, int p_decimals,
			boolean p_mostrarseparadors) {
		return FormataNumeric(p_numero.doubleValue(), p_decimals, p_mostrarseparadors);
	}


	/**
	 * Passant un String retornar un double
	 * @param p_numero
	 * @return double
	 * @throws ParseException
	 */
	public static double ParsejaNumeric(String p_numero) throws ParseException {
		return formatNumerics.parse(p_numero).doubleValue();
	}


	/**
	 * Indicar el format que volvem utilitzar pels numeros
	 * @param formatNum
	 */
	public static void setFormatNum(String formatNum) {
		Internacionalitzacio.formatNum = formatNum;
	}

	/**
	 * Ens retorna el format dels numeros
	 * @return DecimalFormat
	 */
	public static DecimalFormat getFormatNumerics() {
		return formatNumerics;
	}

	/**
	 * Ens retorna el format dels sencers
	 * @return DecimalFormat
	 */
	public static DecimalFormat getFormatSencers() {
		return formatSencers;
	}

	/**
	 * Elimina la part horaria de la data indicada.
	 * Deixa l'objecte a les 00:00:00.
	 * Es util per fer comparacions de dades (after,before).
	 * @param p_data la data a tractar.
	 * @return la data sense hores.
	 */
	public static Date nomesData(Date p_data){
		Calendar c = Calendar.getInstance();
		c.setTime(p_data);
		c.set(Calendar.HOUR_OF_DAY, 0);
		c.set(Calendar.MINUTE, 0);
		c.set(Calendar.SECOND, 0);
		c.set(Calendar.MILLISECOND, 0);
		return c.getTime();
	}

}
