package com.uoc.tfc.Base.manteniment;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import com.uoc.tfc.Base.mestres.Estat;

/**
 * Bean de les aules
 * @author ojaen
 *
 */

public class Aula implements Serializable {


	/**
	 *
	 */
	private static final long serialVersionUID = 4787154412398712750L;
	public static final String CRITERI_AULA_ID_AULA = "idAula";
	public static final String CRITERI_AULA_CODI = "codiAula";
	public static final String CRITERI_AULA_DESCRIPCIO = "descripcioAula";
	public static final String CRITERI_AULA_ID_CURS = "idCursAula";
	public static final String CRITERI_AULA_ID_ESTAT = "idEstatAula";
	public static final String CRITERI_AULA_NOT_EQUAL_ID = "notEqualIdAula";
	public static final String CRITERI_AULA_ORDER = "orderAula";

	private Integer idAula;
	private String codiAula;
	private String descripcio;
	private Integer aforament;
	private Curs idCurs;
	private Estat idEstat;
	private Integer versio;
	private Set<AssignaturaAulaProfessor> setOfAssignaturaAulaProfessor = new HashSet<AssignaturaAulaProfessor>();



	/**
	 * Simple constructor d'Aula.
	 */
	public Aula()
	{
	}

	/**
	 * Getter idAula
	 * @return Integer
	 */
	public Integer getIdAula() {
		return idAula;
	}

	/**
	 * Getter idAula en String
	 * @return String
	 */
	public String getIdAulaString() {
		return idAula.toString();
	}

	/**
	 * Setter idAula
	 * @param idAula
	 */
	public void setIdAula(Integer idAula) {
		this.idAula = idAula;
	}

	/**
	 * Getter idAula
	 * @return Integer
	 */
	public Integer getId() {
		return getIdAula();
	}

	/**
	 * Setter idAula
	 * @param id
	 */
	public void setId(Integer id) {
		setIdAula(id);
	}

	/**
	 * Getter codiAula
	 * @return String
	 */
	public String getCodiAula() {
		return codiAula;
	}

	/**
	 * Setter codiAula
	 * @param codi
	 */
	public void setCodiAula(String codi) {
		this.codiAula = codi;
	}

	/**
	 * Getter descripcio
	 * @return String
	 */
	public String getDescripcio() {
		return descripcio;
	}

	/**
	 * Getter descripcio complerta: aula (curs - tipustitulacio)
	 * @return String
	 */
	public String getDescripcioComplert() {
		String desc=descripcio;
		if (getIdCurs() != null)
			desc += "(" + getIdCurs().getDescripcio() + " - " + getIdCurs().getDescTipusTitulacio() + ")";
		return desc;
	}

	/**
	 * Getter descripcio
	 * @param descripcio
	 */
	public void setDescripcio(String descripcio) {
		this.descripcio = descripcio;
	}


	/**
	 * Setter aformament
	 * @param aforament
	 */
	public void setAforament(Integer aforament) {
		this.aforament = aforament;
	}

	/**
	 * Getter aformament
	 * @return Integer aforament
	 */
	public Integer getAforament() {
		return aforament;
	}

	/**
	 * Getter idEstat
	 * @return Estat
	 */
	public Estat getIdEstat() {
		return idEstat;
	}

	/**
	 * Setter idEstat
	 * @param idEstat
	 */
	public void setIdEstat(Estat idEstat) {
		this.idEstat = idEstat;
	}

	/**
	 * Getter descripcio estat
	 * @return String
	 */
	public String getDescEstat() {
		if (getIdEstat() != null)
			return getIdEstat().getDescripcio();
		else
			return "";
	}

	/**
	 * Setter idCurs
	 * @param idCurs
	 */
	public void setIdCurs(Curs idCurs) {
		this.idCurs = idCurs;
	}

	/**
	 * Getter idCurs
	 * @return Curs
	 */
	public Curs getIdCurs() {
		return idCurs;
	}

	/**
	 * Getter descripcio curs
	 * @return String
	 */
	public String getDescCurs() {
		if (getIdCurs() != null)
			return getIdCurs().getDescripcio();
		else
			return "";
	}

	/**
	 * Getter versio
	 * @return Integer
	 */
	public Integer getVersio() {
		return versio;
	}

	/**
	 * Setter versio
	 * @param versio
	 */
	public void setVersio(Integer versio) {
		this.versio = versio;
	}

	/**
	 * Getter SetOfAssignaturaAulaProfessor
	 * @return Set<AssignaturaAulaProfessor>
	 */
	public Set<AssignaturaAulaProfessor> getSetOfAssignaturaAulaProfessor() {
		return setOfAssignaturaAulaProfessor;
	}

	/**
	 * Setter Set<AssignaturaAulaProfessor>
	 * @param setOfAssignaturaAulaProfessor
	 */
	public void setSetOfAssignaturaAulaProfessor(Set<AssignaturaAulaProfessor> setOfAssignaturaAulaProfessor) {
		this.setOfAssignaturaAulaProfessor = setOfAssignaturaAulaProfessor;
	}

	/**
	 * Afegir nova relaci assignatura-aula-professor
	 * @param p_AssignaturaAulaProfessor
	 */
	public void addAssignaturaAulaProfessor(AssignaturaAulaProfessor p_AssignaturaAulaProfessor) {
		if (p_AssignaturaAulaProfessor != null) {
			getSetOfAssignaturaAulaProfessor().remove(p_AssignaturaAulaProfessor);
			p_AssignaturaAulaProfessor.setIdAula(this);
			getSetOfAssignaturaAulaProfessor().add(p_AssignaturaAulaProfessor);
		}
	}

	/**
	 * Eliminar relaci assignatura-aula-professor
	 * @param p_AssignaturaAulaProfessor
	 * @return Collection
	 */
	public Collection removeAssignaturaAulaProfessor(AssignaturaAulaProfessor p_AssignaturaAulaProfessor) {
		Collection resultat = null;
		getSetOfAssignaturaAulaProfessor().remove(p_AssignaturaAulaProfessor);
		resultat = Collections.unmodifiableCollection(getSetOfAssignaturaAulaProfessor());
		return resultat;
	}


}
