package com.uoc.tfc.Base.manteniment;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import com.uoc.tfc.Base.mestres.Estat;

/**
 * Bean dels Cursos
 * @author ojaen
 *
 */
public class Curs implements Serializable {


	/**
	 *
	 */
	private static final long serialVersionUID = 8814239125175421628L;

	public static final String CRITERI_CURS_ID_CURS = "idCurs";
	public static final String CRITERI_CURS_CODI = "codiCurs";
	public static final String CRITERI_CURS_DESCRIPCIO = "descripcioCurs";
	public static final String CRITERI_CURS_ID_TIPUS_TITULACIO = "idTipusTitulacioCurs";
	public static final String CRITERI_CURS_ID_ESTAT = "idEstatCurs";
	public static final String CRITERI_CURS_ID_CURS_PREVI = "idCursPrevi";
	public static final String CRITERI_CURS_INIT = "CursInit";
	public static final String CRITERI_CURS_NOT_EQUAL_ID = "notEqualIdCurs";
	public static final String CRITERI_CURS_ORDER = "orderCurs";

	private Integer idCurs;
	private String codiCurs;
	private String descripcio;
	private TipusTitulacio idTipusTitulacio;
	private Set<CursAssignatura> setOfCursAssignatura = new HashSet<CursAssignatura>();
	private Estat idEstat;
	private Curs idCursPrevi;
	private Integer versio;



	/**
	 * Simple constructor de Curs.
	 */
	public Curs()
	{
	}

	/**
	 * Getter idCurs
	 * @return Integer
	 */
	public Integer getIdCurs() {
		return idCurs;
	}

	/**
	 * Getter idCurs en String
	 * @return String
	 */
	public String getIdCursString() {
		return idCurs.toString();
	}

	/**
	 * Setter idCurs
	 * @param idCurs
	 */
	public void setIdCurs(Integer idCurs) {
		this.idCurs = idCurs;
	}

	/**
	 * Getter idCurs
	 * @return Integer
	 */
	public Integer getId() {
		return getIdCurs();
	}

	/**
	 * Setter idCurs
	 * @param id
	 */
	public void setId(Integer id) {
		setIdCurs(id);
	}

	/**
	 * Getter codiCurs
	 * @return String
	 */
	public String getCodiCurs() {
		return codiCurs;
	}

	/**
	 * Setter codiCurs
	 * @param codi
	 */
	public void setCodiCurs(String codi) {
		this.codiCurs = codi;
	}

	/**
	 * Getter descripcio
	 * @return String
	 */
	public String getDescripcio() {
		return descripcio;
	}

	/**
	 * Setter descripcio
	 * @param descripcio
	 */
	public void setDescripcio(String descripcio) {
		this.descripcio = descripcio;
	}


	/**
	 * Setter de setOfCursAssignatura
	 * @return Set<CursAssignatura>
	 */
	public Set<CursAssignatura> getSetOfCursAssignatura() {
		return setOfCursAssignatura;
	}

	/**
	 * Setter de setOfCursAssignatura
	 * @param setOfCursAssignatura
	 */
	public void setSetOfCursAssignatura(Set<CursAssignatura> setOfCursAssignatura) {
		this.setOfCursAssignatura = setOfCursAssignatura;
	}

	/**
	 * Getter idEstat
	 * @return Estat
	 */
	public Estat getIdEstat() {
		return idEstat;
	}

	/**
	 * Setter idEstat
	 * @param idEstat
	 */
	public void setIdEstat(Estat idEstat) {
		this.idEstat = idEstat;
	}

	/**
	 * Getter descripcio estat
	 * @return String
	 */
	public String getDescEstat() {
		if (getIdEstat() != null)
			return getIdEstat().getDescripcio();
		else
			return "";
	}

	/**
	 * Setter idTipusTitulacio
	 * @param idTipusTitulacio
	 */
	public void setIdTipusTitulacio(TipusTitulacio idTipusTitulacio) {
		this.idTipusTitulacio = idTipusTitulacio;
	}

	/**
	 * Getter idTipusTitulacio
	 * @return TipusTitulacio
	 */
	public TipusTitulacio getIdTipusTitulacio() {
		return idTipusTitulacio;
	}

	/**
	 * Getter descripcio tipusTitulacio
	 * @return String
	 */
	public String getDescTipusTitulacio() {
		if (getIdTipusTitulacio() != null)
			return getIdTipusTitulacio().getDescripcio();
		else
			return "";
	}

	/**
	 * Getter idCursPrevi
	 * @return Curs
	 */
	public Curs getIdCursPrevi() {
		return idCursPrevi;
	}

	/**
	 * Setter idCursPrevi
	 * @param idCursPrevi
	 */
	public void setIdCursPrevi(Curs idCursPrevi) {
		this.idCursPrevi = idCursPrevi;
	}

	/**
	 * Getter versio
	 * @return Integer
	 */
	public Integer getVersio() {
		return versio;
	}

	/**
	 * Setter versio
	 * @param versio
	 */
	public void setVersio(Integer versio) {
		this.versio = versio;
	}

	/**
	 * Getter descripcio complerta: curs (tipus titulaci)
	 * @return String
	 */
	public String getDescripcioComplert() {
		return descripcio + "(" +
		getIdTipusTitulacio().getDescripcio() + ")";
	}

	/**
	 * Afegir nova assignatura al curs
	 * @param p_CursAssignatura
	 */
	public void addCursAssignatura(CursAssignatura p_CursAssignatura) {
		if (p_CursAssignatura != null) {
			getSetOfCursAssignatura().remove(p_CursAssignatura);
			p_CursAssignatura.setIdCurs(this);
			getSetOfCursAssignatura().add(p_CursAssignatura);
		}
	}

	/**
	 * Eliminar assignatura del curs
	 * @param p_CursAssignatura
	 * @return Collection
	 */
	public Collection removeCursAssignatura(CursAssignatura p_CursAssignatura) {
		Collection resultat = null;
		getSetOfCursAssignatura().remove(p_CursAssignatura);
		resultat = Collections.unmodifiableCollection(getSetOfCursAssignatura());
		return resultat;
	}


}
