package com.uoc.tfc.Base.matricula;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.common.Internacionalitzacio;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.LN.dao.manteniment.AnyAcademicDAO;

/**
 * Bean de matrcula
 * @author ojaen
 *
 */
public class Matricula implements Serializable {

	/**
	 *
	 */
	private static final long serialVersionUID = 6024747307742052176L;

	public static final String CRITERI_MATRICULA_ID_MATRICULA = "idMatricula";
	public static final String CRITERI_MATRICULA_ID_USUARI = "idUsuariMatricula";
	public static final String CRITERI_MATRICULA_NOM_USUARI = "nomUsuariMatricula";
	public static final String CRITERI_MATRICULA_COGNOM1_USUARI = "cognom1UsuariMatricula";
	public static final String CRITERI_MATRICULA_COGNOM2_USUARI = "cognom2UsuariMatricula";
	public static final String CRITERI_MATRICULA_NUM_DOCUMENT_USUARI = "numDocumentUsuariMatricula";
	public static final String CRITERI_MATRICULA_ID_ESTAT_USUARI = "idEstatUsuariMatricula";
	public static final String CRITERI_MATRICULA_ID_TIPUS_DOCUMENT_USUARI = "idTipusDocumentUsuariMatricula";
	public static final String CRITERI_MATRICULA_ID_ANY_ACADEMIC_USUARI = "idAnyAcademicMatricula";
	public static final String CRITERI_MATRICULA_ID_AULA = "idAulaMatricula";
	public static final String CRITERI_MATRICULA_SENSE_AULA = "matriculaSenseAula";
	public static final String CRITERI_MATRICULA_ID_ESTAT = "idEstatMatricula";
	public static final String CRITERI_MATRICULA_ORDER = "orderMatricula";

	public static final Integer NUM_ASSIGNATURES_SUSPESES_REPETEIX = 2;
	private Integer idMatricula;
	private AnyAcademic idAnyAcademic;
	private UsuariAlumne idUsuari;
	private Aula idAula;
	private Curs idCurs;
	private Estat idEstat;
	private Integer versio;
	private Date dataMatricula;


	/**
	 * Simple constructor de Matricula.
	 */
	public Matricula()
	{
		setDataMatricula(Internacionalitzacio.nomesData(new Date()));
	}

	/**
	 * Getter idMatricula
	 * @return Integer
	 */
	public Integer getIdMatricula() {
		return idMatricula;
	}

	/**
	 * Getter idMatricula en String
	 * @return String
	 */
	public String getIdMatriculaString() {
		return idMatricula.toString();
	}

	/**
	 * Setter idMatricula
	 * @param idMatricula
	 */
	public void setIdMatricula(Integer idMatricula) {
		if (idMatricula != null && idMatricula.intValue() == 0)
			this.idMatricula = null;
		else
			this.idMatricula = idMatricula;
	}

	/**
	 * Getter idMatricula
	 * @return Integer
	 */
	public Integer getId() {
		return getIdMatricula();
	}

	/**
	 * Setter idMatricula
	 * @param id
	 */
	public void setId(Integer id) {
		setIdMatricula(id);
	}

	/**
	 * Getter idUsuari
	 * @return Usuari
	 */
	public UsuariAlumne getIdUsuari() {
		return idUsuari;
	}

	/**
	 * Setter idUsuari
	 * @param idUsuari
	 */
	public void setIdUsuari(UsuariAlumne idUsuari) {
		this.idUsuari = idUsuari;
	}

	/**
	 * Getter descripcio usuari
	 * @return String
	 */
	public String getDescUsuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getNomComplert();
		else
			return "";
	}

	/**
	 * Setter idAula
	 * @param idAula
	 */
	public void setIdAula(Aula idAula) {
		this.idAula = idAula;
	}

	/**
	 * Getter idAula
	 * @return Aula
	 */
	public Aula getIdAula() {
		return idAula;
	}

	/**
	 * Getter descripcio aula
	 * @return String
	 */
	public String getDescAula() {
		if (getIdAula() != null)
			return getIdAula().getDescripcio();
		else
			return "";
	}

	/**
	 * Setter idCurs
	 * @param idCurs
	 */
	public void setIdCurs(Curs idCurs) {
		this.idCurs = idCurs;
	}

	/**
	 * Getter idCurs
	 * @return Curs
	 */
	public Curs getIdCurs() {
		return idCurs;
	}

	/**
	 * Getter descripcio curs
	 * @return String
	 */
	public String getDescCurs() {
		if (getIdCurs() != null)
			return getIdCurs().getDescripcio();
		else
			return "";
	}

	/**
	 * Setter idAnyAcademic
	 * @param idAnyAcademic
	 */
	public void setIdAnyAcademic(AnyAcademic idAnyAcademic) {
		this.idAnyAcademic = idAnyAcademic;
	}

	/**
	 * Getter idAnyAcademic
	 * @return AnyAcademic
	 */
	public AnyAcademic getIdAnyAcademic() {
		return idAnyAcademic;
	}

	/**
	 * Getter descripcio any acadmic
	 * @return String
	 */
	public String getDescAnyAcademic() {
		if (getIdAnyAcademic() != null)
			return getIdAnyAcademic().getDescripcio();
		else
			return "";
	}

	/**
	 * Getter idEstat
	 * @return Estat
	 */
	public Estat getIdEstat() {
		return idEstat;
	}

	/**
	 * Setter idEstat
	 * @param idEstat
	 */
	public void setIdEstat(Estat idEstat) {
		this.idEstat = idEstat;
	}

	/**
	 * Setter dataMatricula
	 * @param dataMatricula
	 */
	public void setDataMatricula(Date dataMatricula) {
		this.dataMatricula = dataMatricula;
	}

	/**
	 * Getter dataMatricula
	 * @return Date
	 */
	public Date getDataMatricula() {
		return dataMatricula;
	}

	/**
	 * Getter descripcio estat
	 * @return String
	 */
	public String getDescEstat() {
		if (getIdEstat() != null)
			return getIdEstat().getDescripcio();
		else
			return "";
	}

	/**
	 * Getter versio
	 * @return Integer
	 */
	public Integer getVersio() {
		return versio;
	}

	/**
	 * Setter versio
	 * @param versio
	 */
	public void setVersio(Integer versio) {
		this.versio = versio;
	}

	/**
	 * Getter descripcio _ usuari - curs - aula
	 * @return String
	 */
	public String getDescripcio() {
		if (getIdUsuari() != null)
			return getDescUsuari() + " - " +
			getDescCurs() + " - " + getDescAula();
		else
			return "";
	}

	/**
	 * Getter Codi usuari
	 * @return Integer
	 */
	public Integer getCodiUsuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getId();
		return null;
	}

	/**
	 * Getter nom usuari
	 * @return String
	 */
	public String getNomUsuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getNom();
		return "";
	}

	/**
	 * Getter primer cognom usuari
	 * @return String
	 */
	public String getCognom1Usuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getCognom1();
		return "";
	}

	/**
	 * Getter segon cognom usuari
	 * @return String
	 */
	public String getCognom2Usuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getCognom2();
		return "";
	}

	/**
	 * Getter descripcio estat usuari
	 * @return String
	 */
	public String getDescEstatUsuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getDescEstat();
		return "";
	}

	/**
	 * Getter numero document usuari
	 * @return String
	 */
	public String getNumDocumentUsuari() {
		if (getIdUsuari() != null)
			return getIdUsuari().getNumDocument();
		return "";
	}

	/**
	 * Getter Matricular
	 * @return String: Si/No
	 */
	public String getMatriculat() {
		if (getIsMatriculat())
			return "Si";
		return "No";
	}

	/**
	 * Getter isMatriculat
	 * @return boolean
	 */
	public boolean getIsMatriculat() {
		try {
			AnyAcademic[] anysAcademics = cercaAnyAcademicAMatricularPerDataMatricula(); 
			if (Helper.validarArray(anysAcademics))
				return isMatriculat(anysAcademics);			
			
			anysAcademics = cercaAnyAcademicAMatricularPerDataAnyAcademic();
			if (Helper.validarArray(anysAcademics))
				return isMatriculat(anysAcademics);
			return false;
				
		} catch (Throwable e) {
			return false;
		}		
	}
	
	/**
	 * Verifiquem si ja est matriculat
	 * @param anysAcademics
	 * @return boolean
	 */
	private boolean isMatriculat(AnyAcademic[] anysAcademics) {
		if (Helper.validarColleccio(getIdUsuari().getSetOfMatricula())) {
			Iterator<Matricula> iter= getIdUsuari().getSetOfMatricula().iterator();
			while (iter.hasNext()) {
				Matricula matricula =  iter.next();
				
				for (int i =0;i<anysAcademics.length;i++)
					if (anysAcademics[i].getIdAnyAcademic().equals(matricula.getIdAnyAcademic().getIdAnyAcademic()))
						return true;				
			}
		}
		
		return false;
	}

	/**
	 * Cerca any acadmic a matricular
	 * @return AnyAcademic[]
	 * @throws Throwable
	 */
	private AnyAcademic[] cercaAnyAcademicAMatricularPerDataMatricula() throws Throwable {
		Date dataActual = Internacionalitzacio.nomesData(new Date());

		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI_MATRICULA, dataActual);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI_MATRICULA, dataActual);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

		AnyAcademic[] anysAcademics = recuperaAnysAcademics(parametres);
		return anysAcademics;
	}
	
	/**
	 * Cerca any acadmic a matricular
	 * @return AnyAcademic[]
	 * @throws Throwable
	 */
	private AnyAcademic[] cercaAnyAcademicAMatricularPerDataAnyAcademic() throws Throwable {
		Date dataActual = Internacionalitzacio.nomesData(new Date());

		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI, dataActual);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI, dataActual);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

		AnyAcademic[] anysAcademics = recuperaAnysAcademics(parametres);
		return anysAcademics;
	}
	
	/**
	 * Recuperem els anys academics de la base de dades
	 * @return AnyAcademic[]: Array d'anys acadmics
	 */
	private AnyAcademic[] recuperaAnysAcademics(HashMap p_parametres) throws Throwable{

		AnyAcademicDAO aulaDAO = new AnyAcademicDAO();
		AnyAcademic[] o = (AnyAcademic[]) aulaDAO.findAnyAcademic(p_parametres);
		return o;
	}




}
