package com.uoc.tfc.LN.dao;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * Classe on hereden totes les classes DAO
 * @author ojaen
 *
 */
public class BaseDAO implements IBaseDAO{
	/**
	 * Mtode que realitza el insert del bean a la taula correponent
	 * @param        oBean Object Objecte correponen al Bean que volem insertar a BBDD.
	 *
	 **/
	public void Insert(Object oBean)  {

		HibernateUtil hu = HibernateUtil.getInstance();
		Session s = hu.getSession();
		Transaction tx = s.beginTransaction();
		try {
			Insert(s, oBean);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
		} finally {
			s.close();
		}
	}

	/**
	 * Mtode per insertar un nou registre
	 * @param s session
	 * @param oBean Object Objecte correponen al Bean que volem insertar a BBDD.
	 * @throws Exception
	 */
	public void Insert(Session s, Object oBean) throws Exception {
		s.saveOrUpdate(oBean);
	}



	/**
	 * Mtode que realitza el insert del bean a la taula correponent
	 * @param        oBean Object Objecte correponen al Bean que volem insertar a BBDD.
	 * @throws Exception indica que ha hagut un error en Hibernate.
	 *
	 **/
	public void Update(Session s, Object oBean) throws Exception
	{
		s.saveOrUpdate(oBean);
	}

	/**
	 *  Mtode que realitza el insert del bean a la taula correponent, sense sessi
	 * @param oBean
	 */
	public void Update(Object oBean) {

		HibernateUtil hu = HibernateUtil.getInstance();
		Session s = hu.getSession();
		Transaction tx = s.beginTransaction();
		try {
			Update(s,oBean);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
		} finally {
			s.close();
		}
	}

	/**
	 * Mtode que realitza el insert del bean a la taula correponent
	 * @param  oBean Object Objecte correponen al Bean que volem insertar a BBDD.
	 * @throws Exception indica que ha hagut un error en Hibernate.
	 *
	 **/
	public void Delete(Object oBean)  {

		HibernateUtil hu = HibernateUtil.getInstance();
		Session s = hu.getSession();
		Transaction tx = s.beginTransaction();
		try {

			Delete(s, oBean);
			tx.commit();

		} catch (Exception ex) {
			tx.rollback();
		} finally {
			s.close();
		}
	}

	/**
	 *
	 * @param s Session
	 * @param oBean Object Objecte correponen al Bean que volem insertar a BBDD.
	 * @throws Exception
	 */
	public void Delete(Session s, Object oBean) throws Exception
	{
		s.delete(oBean);
	}

	/**
	 * Executar la consulta
	 * @param query
	 * @return Iterator<Object>
	 */
	public Iterator<Object> executaConsulta(Query query) {
		List resultat = executaConsultaList(query);
		return resultat.iterator();
	}

	/**
	 * Executar la consulta
	 * @param query
	 * @return List
	 */
	public List executaConsultaList(Query query) {
		List resultat = query.list();
		return resultat;
	}

	/**
	 * Executar la consulta
	 * @param query
	 * @param numRegistres
	 * @return Iterator
	 */
	public Iterator executaConsulta(Query query, int numRegistres) {
		query.setMaxResults(numRegistres);
		List resultat = query.list();
		return resultat.iterator();
	}


	/**
	 * Mtode que inicialitza una col.leccio i els elements que hi conte
	 * @param col Col.leccio a inicialitzar.
	 * @return
	 */
	protected void inicialitzaColeccio(Collection _col) {
		if (_col == null) return;

		Hibernate.initialize(_col);

		Iterator it = _col.iterator();
		while (it.hasNext()){
			Object bean = it.next();
			if (bean != null)
				Hibernate.initialize(bean);
		}
	}


}
