package com.uoc.tfc.LN.dao.manteniment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean AnyAcademic
 * 
 */

public class AnyAcademicDAO extends BaseDAO implements IAnyAcademicDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";


	/**
	 * Cerca dels anys academics passant-li parametres de criteri de cerca
	 * @param p_parametres
	 * @return AnyAcademic[]
	 * @throws Throwable
	 */
	public AnyAcademic[] findAnyAcademic(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		AnyAcademic[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca dels anys academics passant-li un criteri de cerca en una sessio oberta
	 * @param s
	 * @param p_parametres
	 * @return AnyAcademic[]
	 * @throws Throwable
	 */
	public AnyAcademic[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from AnyAcademic as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC) && 
				HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_NOT_EQUAL_ID)) {
			select += sand + " f.idAnyAcademic <> :" + AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC)) {
			select += sand + " f.idAnyAcademic = :" + AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_CODI)) {
			select += sand + " f.codi like :" + AnyAcademic.CRITERI_ANY_ACADEMIC_CODI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_DESCRIPCIO)) {
			select += sand + " f.descripcio like :" + AnyAcademic.CRITERI_ANY_ACADEMIC_DESCRIPCIO;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI)){
			select += sand + " ( f.dataInici <= :" + AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI + " or f.dataInici is null ) ";
			sand = AND;
		}
		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI)){
			select += sand + " ( f.dataFi >= :" + AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI + " or f.dataFi is null )";
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI_MATRICULA)){
			select += sand + " ( f.dataIniciMatricula <= :" + AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI_MATRICULA + " or f.dataIniciMatricula is null ) ";
			sand = AND;
		}
		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI_MATRICULA)){
			select += sand + " ( f.dataFiMatricula >= :" + AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI_MATRICULA + " or f.dataFiMatricula is null )";
			sand = AND;
		}





		if (HelperDB.validarContingutMapa(p_parametres, AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT;
			sand = AND;
		}

		select += " order by f.dataInici desc ";



		Query query = s.createQuery(select);

		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			AnyAcademic anyAcademic = (AnyAcademic) it.next();
			al.add(anyAcademic);
		}
		AnyAcademic[] array = new AnyAcademic[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (AnyAcademic) al.get(i);
		return array;

	}

	/**
	 * Gravar un anyAcademic
	 * @param p_anyAcademic
	 * @return AnyAcademic
	 * @throws Throwable
	 */
	public AnyAcademic gravar(AnyAcademic p_anyAcademic) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_anyAcademic);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return p_anyAcademic;
	}

	/**
	 * Esborrar un anyAcademic
	 * @param p_anyAcademic
	 * @throws Throwable
	 */
	public void esborrar(AnyAcademic p_anyAcademic) throws Throwable {

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_anyAcademic);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}


}