package com.uoc.tfc.LN.dao.manteniment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.manteniment.Assignatura;
import com.uoc.tfc.Base.manteniment.AssignaturaAulaProfessor;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean Assignatura
 * 
 */

public class AssignaturaDAO extends BaseDAO implements IAssignaturaDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";


	/**
	 * Cerca de les assignatures
	 * @param p_parametres
	 * @return Assignatura[]
	 * @throws Throwable
	 */
	public Assignatura[] findAssignatura(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Assignatura[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca de les assignatures
	 * @param session
	 * @param p_parametres
	 * @return Assignatura[]
	 * @throws Exception
	 */
	private Assignatura[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from Assignatura as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA) && 
				HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASSIGNATURA_NOT_EQUAL_ID)) {
			select += sand + " f.idAssignatura <> :" + Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA)) {
			select += sand + " f.idAssignatura = :" + Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASSIGNATURA_CODI)) {
			select += sand + " f.codiAssignatura like :" + Assignatura.CRITERI_ASSIGNATURA_CODI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASSIGNATURA_DESCRIPCIO)) {
			select += sand + " f.descripcio like :" + Assignatura.CRITERI_ASSIGNATURA_DESCRIPCIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASIGNATURA_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + Assignatura.CRITERI_ASIGNATURA_ID_ESTAT;
			sand = AND;
		}
		
		select += " order by f.descripcio ";


		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Assignatura assignatura = (Assignatura) it.next();
			al.add(assignatura);
		}
		Assignatura[] array = new Assignatura[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Assignatura) al.get(i);
		return array;

	}

	/**
	 * Gravar assignatura
	 * @param p_assignatura
	 * @return Assignatura
	 * @throws Throwable
	 */
	public Assignatura gravar(Assignatura p_assignatura) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_assignatura);
			tx.commit();
		} catch (Exception ex) {			
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}


		return p_assignatura;
	}

	/**
	 * Esborrar assignatura
	 * @param p_assignatura
	 * @throws Throwable
	 */
	public void esborrar(Assignatura p_assignatura) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_assignatura);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}
	
	/**
	 * Cerca de les relacions de les assignatures aules i professors
	 * @param p_parametres
	 * @return Assignatura[]
	 * @throws Throwable
	 */
	public AssignaturaAulaProfessor[] findAssignaturaAulaProfessor(HashMap p_parametres) throws Throwable {

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		AssignaturaAulaProfessor[] array = null;
		try {
			array = findAssignaturaAulaProfessor(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}	
	
	/**
	 * Cerca de les relacions de les assignatures aules i professors
	 * @param session
	 * @param p_parametres
	 * @return Assignatura[]
	 * @throws Exception
	 */
	private AssignaturaAulaProfessor[] findAssignaturaAulaProfessor(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from AssignaturaAulaProfessor as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA)) {
			select += sand + " f.idAssignatura.idAssignatura = :" + Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA;
			sand = AND;
		}

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			AssignaturaAulaProfessor assignaturaAulaProfessor = (AssignaturaAulaProfessor) it.next();
			al.add(assignaturaAulaProfessor);
		}
		AssignaturaAulaProfessor[] array = new AssignaturaAulaProfessor[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (AssignaturaAulaProfessor) al.get(i);
		return array;

	}	


}