package com.uoc.tfc.LN.dao.manteniment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.common.ConfiguracioUsuari;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.usuari.UsuariProfessor;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean Aula
 * 
 */

public class AulaDAO extends BaseDAO implements IAulaDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";

	/**
	 * Cerca de les aules passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return Aula[]
	 * @throws Throwable
	 */

	public Aula[] findAula(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Aula[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca de les aules passant-li parmetres de criteri de cerca en una sessio
	 * @param s
	 * @param p_parametres
	 * @return Aula[]
	 * @throws Throwable
	 */
	private Aula[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from Aula as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_ID_AULA) && 
				HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_NOT_EQUAL_ID)) {
			select += sand + " f.idAula <> :" + Aula.CRITERI_AULA_ID_AULA;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_ID_AULA)) {
			select += sand + " f.idAula = :" + Aula.CRITERI_AULA_ID_AULA;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_CODI)) {
			select += sand + " f.codiAula like :" + Aula.CRITERI_AULA_CODI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_DESCRIPCIO)) {
			select += sand + " f.descripcio like :" + Aula.CRITERI_AULA_DESCRIPCIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_ID_CURS)) {
			select += sand + " f.idCurs.idCurs = :" + Aula.CRITERI_AULA_ID_CURS;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Aula.CRITERI_AULA_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + Aula.CRITERI_AULA_ID_ESTAT;
			sand = AND;
		}

		if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariProfessor) {
			select += sand + filtreProfessor();
			sand = AND;
		}
		
		select += " order by f.codiAula ";


		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Aula aula = (Aula) it.next();
			al.add(aula);
		}
		Aula[] array = new Aula[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Aula) al.get(i);
		return array;

	}

	/**
	 * Gravar aula
	 * @param p_aula
	 * @return Aula
	 * @throws Throwable
	 */
	public Aula gravar(Aula p_aula) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_aula);
			tx.commit();
		} catch (Exception ex) {			
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}


		return p_aula;
	}

	/**
	 * Esborrar Aula
	 * @param p_aula
	 * @throws Throwable
	 */
	public void esborrar(Aula p_aula) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_aula);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}

	/**
	 * Filtrar aules per professor
	 * @return String
	 */
	private String filtreProfessor() {
		String select = " f.idAula in (";
		select += " select aap.idAsigAulaProf.idAula.idAula " +
		" from AssignaturaAulaProfessor aap " +
		" where aap.idAsigAulaProf.idProfessor.idUsuari = '" +   
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "')";
		return select;
	}


}