package com.uoc.tfc.LN.dao.manteniment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean Curs
 * 
 */

public class CursDAO extends BaseDAO implements ICursDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";

	/**
	 * Cerca dels cursos passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return Curs[]
	 * @throws Throwable
	 */
	public Curs[] findCurs(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Curs[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca dels cursos passant-li parmetres de criteri de cerca en una sessio
	 * @param s
	 * @param p_parametres
	 * 	 * @return Curs[]
	 * @throws Throwable
	 */
	private Curs[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from Curs as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_ID_CURS) && 
				HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_NOT_EQUAL_ID)) {
			select += sand + " f.idCurs <> :" + Curs.CRITERI_CURS_ID_CURS;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_ID_CURS)) {
			select += sand + " f.idCurs = :" + Curs.CRITERI_CURS_ID_CURS;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_CODI)) {
			select += sand + " f.codiCurs like :" + Curs.CRITERI_CURS_CODI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_DESCRIPCIO)) {
			select += sand + " f.descripcio like :" + Curs.CRITERI_CURS_DESCRIPCIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_ID_TIPUS_TITULACIO)) {
			select += sand + " f.idTipusTitulacio.idTipusTitulacio = :" + Curs.CRITERI_CURS_ID_TIPUS_TITULACIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + Curs.CRITERI_CURS_ID_ESTAT;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Curs.CRITERI_CURS_ID_CURS_PREVI)) {
			select += sand + " f.idCursPrevi.idCurs = :" + Curs.CRITERI_CURS_ID_CURS_PREVI;
			sand = AND;
		}

		select += " order by f.codiCurs ";

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Curs curs = (Curs) it.next();
			al.add(curs);
		}
		Curs[] array = new Curs[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Curs) al.get(i);
		return array;

	}

	/**
	 * Gravar curs
	 * @param p_curs
	 * @return Curs
	 * @throws Throwable
	 */
	public Curs gravar(Curs p_curs) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {

			Update(s, p_curs);
			tx.commit();
		} catch (Exception ex) {			
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}


		return p_curs;
	}

	/**
	 * Esborrar curs
	 * @param p_curs
	 * @throws Throwable
	 */
	public void esborrar(Curs p_curs) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_curs);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}


}