package com.uoc.tfc.LN.dao.manteniment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.manteniment.TipusQualificacio;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO TipusQualificacio
 * 
 */

public class TipusQualificacioDAO extends BaseDAO implements ITipusQualificacioDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";

	/**
	 * Cerca dels tipus de qualificaci passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return TipusQualificacio[]
	 * @throws Throwable
	 */
	public TipusQualificacio[] findTipusQualificacio(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();


		TipusQualificacio[] array = null;

		try{
			array = find(s, p_parametres);
		} catch (Exception ex) {
			throw new Throwable(ex);
		} finally {		
			s.close();
		}

		return array;


	}

	/**
	 * Cerca dels tipus de qualificaci passant-li un criteri de cerca en una sessio oberta
	 * @param s
	 * @param p_parametres
	 * @return TipusQualificacio[]
	 * @throws Throwable
	 */
	private TipusQualificacio[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from TipusQualificacio as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO) && 
				HelperDB.validarContingutMapa(p_parametres, TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_NOT_EQUAL_ID)) {
			select += sand + " f.idTipusQualificacio <> :" + TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO)) {
			select += sand + " f.idTipusQualificacio = :" + TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_CODI)) {
			select += sand + " f.codiTipusQualificacio like :" + TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_CODI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_DESCRIPCIO)) {
			select += sand + " f.descripcio like :" + TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_DESCRIPCIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_CODI_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_CODI_ESTAT;
			sand = AND;
		}

		select += " order by f.idTipusQualificacio ";

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			TipusQualificacio tipusQualificacio = (TipusQualificacio) it.next();
			al.add(tipusQualificacio);
		}
		TipusQualificacio[] array = new TipusQualificacio[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (TipusQualificacio) al.get(i);
		return array;

	}

	/**
	 * Gravar tipus de qualificaci
	 * @param p_tipusQualificacio
	 * @return TipusQualificacio
	 * @throws Throwable
	 */
	public TipusQualificacio gravar(TipusQualificacio p_tipusQualificacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_tipusQualificacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}


		return p_tipusQualificacio;
	}

	/**
	 * Esborrar un tipus de qualificaci
	 * @param p_tipusQualificacio
	 * @throws Throwable
	 */
	public void esborrar(TipusQualificacio p_tipusQualificacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_tipusQualificacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}


}