package com.uoc.tfc.LN.dao.manteniment;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.manteniment.TipusTitulacio;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO TipusTitulacio
 * 
 */

public class TipusTitulacioDAO extends BaseDAO implements ITipusTitulacioDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";


	/**
	 * Cerca dels tipus de titulaci passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return TipusTitulacio[]
	 * @throws Throwable
	 */	
	public TipusTitulacio[] findTipusTitulacio(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		TipusTitulacio[] array = null; 

		try {
			array= find(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		} 

		return array;


	}

	/**
	 * Cerca dels tipus de titulaci passant-li un criteri de cerca en una sessio oberta
	 * @param s
	 * @param p_parametres
	 * @return TipusTitulacio[]
	 * @throws Throwable
	 */
	private TipusTitulacio[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from TipusTitulacio as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO) && 
				HelperDB.validarContingutMapa(p_parametres, TipusTitulacio.CRITERI_TIPUSTITULACIO_NOT_EQUAL_ID)) {
			select += sand + " f.idTipusTitulacio <> :" + TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO)) {
			select += sand + " f.idTipusTitulacio = :" + TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, TipusTitulacio.CRITERI_TIPUSTITULACIO_CODI)) {
			select += sand + " f.codiTipusTitulacio like :" + TipusTitulacio.CRITERI_TIPUSTITULACIO_CODI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, TipusTitulacio.CRITERI_TIPUSTITULACIO_DESCRIPCIO)) {
			select += sand + " f.descripcio like :" + TipusTitulacio.CRITERI_TIPUSTITULACIO_DESCRIPCIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, TipusTitulacio.CRITERI_TIPUSTITULACIO_CODI_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + TipusTitulacio.CRITERI_TIPUSTITULACIO_CODI_ESTAT;
			sand = AND;
		}
		
		select += " order by f.idTipusTitulacio ";


		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			TipusTitulacio tipusTitulacio = (TipusTitulacio) it.next();
			al.add(tipusTitulacio);
		}
		TipusTitulacio[] array = new TipusTitulacio[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (TipusTitulacio) al.get(i);
		return array;

	}

	/**
	 * Gravar tipus de titulaci
	 * @param p_tipusTitulacio
	 * @return TipusTitulacio
	 * @throws Throwable
	 */
	public TipusTitulacio gravar(TipusTitulacio p_tipusTitulacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_tipusTitulacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return p_tipusTitulacio;
	}

	/**
	 * Eliminar tipusTitulaci
	 * @param p_tipusTitulacio
	 * @throws Throwable
	 */
	public void esborrar(TipusTitulacio p_tipusTitulacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_tipusTitulacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}


}