
package com.uoc.tfc.LN.dao.matricula;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.ConfiguracioUsuari;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.manteniment.AssignaturaAulaProfessor;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.manteniment.Missatge;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.qualificacio.Qualificacio;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.Base.usuari.UsuariPare;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.dao.manteniment.AulaDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean Matricula
 *
 */
public class MatriculaDAO extends BaseDAO implements IMatriculaDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";


	/**
	 * Gravar matrcula
	 * @param p_matricula
	 * @return MAtricula
	 * @throws Throwable
	 */
	public Matricula gravar(Matricula p_matricula) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {

			if (p_matricula.getIdAula() != null && p_matricula.getIdAula().getIdAula().intValue() > 0) {
				crearQualificacions(s, p_matricula.getIdAnyAcademic(),
						p_matricula.getIdUsuari(),
						p_matricula.getIdCurs(),
						p_matricula.getIdAula());
			}
			else
				p_matricula.setIdAula(null);
			Update(s, p_matricula);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return p_matricula;
	}

	/**
	 * Si la matrcula ja te aula ja podem crear la relacio de qualificacions de l'alumne
	 * @param s
	 * @param idAnyAcademic
	 * @param p_usuari
	 * @param curs
	 * @param p_aula
	 * @throws Exception
	 */
	private void crearQualificacions(Session s, AnyAcademic idAnyAcademic, UsuariAlumne p_usuari, Curs curs,
			Aula p_aula) throws Exception {
		boolean bExisteixQualificions = false;
		for (Iterator<Qualificacio> iter=p_usuari.getSetOfQualificacio().iterator();iter.hasNext();) {
			Qualificacio qualif = iter.next();
			if (qualif.getIdAnyAcademic().getIdAnyAcademic().equals(idAnyAcademic.getIdAnyAcademic()))
				bExisteixQualificions = true;
		}

		if (!bExisteixQualificions) {

			for (Iterator<AssignaturaAulaProfessor> iter=p_aula.getSetOfAssignaturaAulaProfessor().iterator();iter.hasNext();) {
				AssignaturaAulaProfessor aap = iter.next();

				if (aap.getIdAssignatura().getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA) &&
					aap.getIdAula().getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					Qualificacio qualificacio = new Qualificacio(idAnyAcademic, p_usuari, aap);
					p_usuari.addQualificacio(qualificacio);
					Update(s, qualificacio);
				}
			}

		}

	}

	/**
	 * Cerca de les matrcules passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return Matricula[]
	 * @throws Throwable
	 */
	public Matricula[] findMatricula(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Matricula[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca de les matrcules passant-li parmetres de criteri de cerca en una sessio
	 * @param s
	 * @param p_parametres
	 * @return Matricula[]
	 * @throws Throwable
	 */
	private Matricula[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from Matricula as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_MATRICULA)) {
			select += sand + " f.idMatricula = :" + Matricula.CRITERI_MATRICULA_ID_MATRICULA;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_USUARI)) {
			select += sand + " f.idUsuari.idUsuari = :" + Matricula.CRITERI_MATRICULA_ID_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_NOM_USUARI)) {
			select += sand + " f.idUsuari.nom like :" + Matricula.CRITERI_MATRICULA_NOM_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_COGNOM1_USUARI)) {
			select += sand + " f.idUsuari.cognom1 like :" + Matricula.CRITERI_MATRICULA_COGNOM1_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_COGNOM2_USUARI)) {
			select += sand + " f.idUsuari.cognom2 like :" + Matricula.CRITERI_MATRICULA_COGNOM2_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_NUM_DOCUMENT_USUARI)) {
			select += sand + " f.idUsuari.numDocument = :" + Matricula.CRITERI_MATRICULA_NUM_DOCUMENT_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_TIPUS_DOCUMENT_USUARI) &&
				HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_NUM_DOCUMENT_USUARI)) {
			select += sand + " f.idUsuari.idTipusDocument.idTipusDocument = :" + Matricula.CRITERI_MATRICULA_ID_TIPUS_DOCUMENT_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_ESTAT_USUARI)) {
			select += sand + " f.idUsuari.idEstat.idEstat = :" + Matricula.CRITERI_MATRICULA_ID_ESTAT_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + Matricula.CRITERI_MATRICULA_ID_ESTAT;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_ANY_ACADEMIC_USUARI)) {
			select += sand + " f.idAnyAcademic.idAnyAcademic = :" + Matricula.CRITERI_MATRICULA_ID_ANY_ACADEMIC_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_ID_AULA)) {
			select += sand + " f.idAula.idAula = :" + Matricula.CRITERI_MATRICULA_ID_AULA;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Matricula.CRITERI_MATRICULA_SENSE_AULA)) {
			select += sand + " (f.idAula is null or f.idAula.idAula = 0) ";
			sand = AND;
		}


		if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariAlumne) {
			select += sand + filtreAlumne();
			sand = AND;
		}
		else if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariPare) {
			select += sand + filtrePare();
			sand = AND;
		}

		select += " order by f.idUsuari.cognom1, f.idUsuari.cognom2, f.idUsuari.nom ";

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Matricula usuari = (Matricula) it.next();
			al.add(usuari);
		}
		Matricula[] array = new Matricula[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Matricula) al.get(i);
		return array;

	}

	/**
	 * Filtre matrcules per alumne
	 * @return
	 */
	private String filtreAlumne() {
		String select = " f.idUsuari.idUsuari = '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "'";
		return select;
	}

	/**
	 * Filtre matrcules per pare
	 * @return
	 */
	private String filtrePare() {
		String select = " f.idUsuari.idUsuari in (" +
		"select pa.idAlumne.idUsuari from PareAlumne pa " +
		"where pa.idPare.idUsuari = '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "' " +
		"and pa.idEstat.idEstat = " + CodisMestre.CODI_ESTAT_ALTA + ")";
		return select;
	}

	/**
	 * Esborrar matrcula
	 * @param p_matricula
	 * @throws Throwable
	 */
	public void esborrar(Matricula p_matricula) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_matricula);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}

	/**
	 * Assignacio automtica de les aules
	 * @return List<Missatge>
	 * @throws Throwable
	 */
	public List<Missatge> assignacioAutomaticaAules() throws Throwable {

		List<Missatge> alErrors = new ArrayList<Missatge>();
		HashMap parametres = new HashMap();
		parametres.put(Matricula.CRITERI_MATRICULA_SENSE_AULA, Matricula.CRITERI_MATRICULA_SENSE_AULA);
		parametres.put(Matricula.CRITERI_MATRICULA_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
		Matricula[] matricules = findMatricula(parametres);

		//Cache de la relacio d'aules per cada curs
		HashMap<Integer, Aula[]> cache = new HashMap<Integer, Aula[]>();


		//Cache del numero de matricules existents en una aula
		HashMap<String, Integer> cacheAula = new HashMap<String, Integer>();

		if (Helper.validarArray(matricules)) {
			for (int i = 0; i < matricules.length; i++) {
				Matricula matricula = matricules[i];

				Curs cursActual = matricula.getIdCurs();
				AnyAcademic anyAcademic = matricula.getIdAnyAcademic();

				//Recuperem les aules del curs actual
				cache = recuperaAules(cursActual, cache);

				if (Helper.validarContingutMapa(cache, cursActual.getIdCurs())) {
					Aula[] aules = cache.get(cursActual.getIdCurs());


					//Per cada aula revisem si hi han places lliures
					for (int j = 0; j<aules.length;j++) {
						Aula aula = aules[j];

						cacheAula = recuperaNumPlacesOcupades(cacheAula, anyAcademic, aula);

						String key = anyAcademic.getIdAnyAcademic() + "#" +
						aula.getIdAula();
						if (Helper.validarContingutMapa(cacheAula, key)) {
							Integer numPlacesOcupades = cacheAula.get(key);

							//Si hi ha places lliures assignem l'aula
							if (aula.getAforament() == null ||
									numPlacesOcupades < aula.getAforament().intValue()) {
								matricula.setIdAula(aula);
								matricula = gravar(matricula);

								int iNumPlaces = numPlacesOcupades.intValue() + 1;

								cacheAula.put(key, new Integer(iNumPlaces));
								break;
							}
						}

					}

				}

				if (matricula.getIdAula() == null) {
					alErrors.add(new Missatge("L'alumne " + matricula.getIdUsuari().getNomComplert() +
					" no se li ha pogut assignar una aula"));
				}
			}

		}
		if (alErrors.isEmpty())
			alErrors.add(new Missatge("Procs finalitzat correctament"));
		return alErrors;

	}



	/**
	 * Recuperem les aules en el cas que no estiguin en cache
	 * @param cursActual
	 * @param cache
	 * @return HashMap<Integer, Aula[]>
	 * @throws Throwable
	 */
	private HashMap<Integer, Aula[]> recuperaAules(Curs cursActual,
			HashMap<Integer, Aula[]> cache) throws Throwable {
		if (!Helper.validarContingutMapa(cache, cursActual.getIdCurs())) {
			AulaDAO aulaDAO = new AulaDAO();

			HashMap parametres = new HashMap();
			parametres.put(Aula.CRITERI_AULA_ID_CURS, cursActual.getIdCurs());
			parametres.put(Aula.CRITERI_AULA_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
			Aula aules[] = aulaDAO.findAula(parametres);

			cache.put(cursActual.getIdCurs(), aules);
		}

		return cache;
	}


	/**
	 * Recuperem el nmero de places ocupades en un any acadmic i aula
	 * @param anyAcademic
	 * @param aula
	 * @return HashMap<String, Integer>
	 * @throws Throwable
	 */
	private HashMap<String, Integer> recuperaNumPlacesOcupades(HashMap<String, Integer> cacheAula,
			AnyAcademic anyAcademic, Aula aula) throws Throwable {
		String key = anyAcademic.getIdAnyAcademicString() + "#" + aula.getIdAulaString();

		if (!Helper.validarContingutMapa(cacheAula, key)) {
			HashMap parametres = new HashMap();
			parametres.put(Matricula.CRITERI_MATRICULA_ID_ANY_ACADEMIC_USUARI, anyAcademic.getIdAnyAcademic());
			parametres.put(Matricula.CRITERI_MATRICULA_ID_AULA, aula.getIdAula());
			Matricula matricules[] = findMatricula(parametres);

			cacheAula.put(key, new Integer(matricules.length));
		}

		return cacheAula;

	}

}