package com.uoc.tfc.LN.dao.notificacio;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.manteniment.AssignaturaAulaProfessor;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.notificacio.Notificacio;
import com.uoc.tfc.Base.notificacio.NotificacioAssignatura;
import com.uoc.tfc.Base.notificacio.NotificacioAula;
import com.uoc.tfc.Base.notificacio.NotificacioCurs;
import com.uoc.tfc.Base.notificacio.NotificacioEscola;
import com.uoc.tfc.Base.usuari.Usuari;
import com.uoc.tfc.Base.usuari.UsuariAdministrador;
import com.uoc.tfc.Base.usuari.UsuariAdministratiu;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.Base.usuari.UsuariPare;
import com.uoc.tfc.Base.usuari.UsuariProfessor;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * DAO del bean Notificacio
 * @author ojaen
 *
 */
public class NotificacioDAO extends BaseDAO implements INotificacio {
	private static final String AND = " and ";
	private static final String OR = " or ";
	private static final String WHERE = " where ";

	/**
	 * Gravar notificacio
	 * @param p_notificacio
	 * @return Notificacio
	 * @throws Throwable
	 */
	public Notificacio gravar(Notificacio p_notificacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_notificacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return p_notificacio;
	}

	/**
	 * Cerca notificacio passant-li parmetres de criteri de cerca
	 * @param p_usuari
	 * @param p_parametres
	 * @return Notificacio[]
	 * @throws Throwable
	 */
	public Notificacio[] findNotificacio(Usuari p_usuari, HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Notificacio[] array = null;
		try {
			array = find(s, p_usuari, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca notificacio passant-li parmetres de criteri de cerca en una sessio
	 * @param s
	 * @param p_usuari
	 * @param p_parametres
	 * @return Notificacio[]
	 * @throws Throwable
	 */
	private Notificacio[] find(Session s, Usuari p_usuari, HashMap p_parametres) throws Exception {

		String select = "select f  from Notificacio as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_ID_NOTIFICACIO)) {
			select += sand + " f.idNotificacio = :" + Notificacio.CRITERI_NOTIFICACIO_ID_NOTIFICACIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_TITOL)) {
			select += sand + " f.titol like :" + Notificacio.CRITERI_NOTIFICACIO_TITOL;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_ID_TIPUS_NOTIFICACIO)) {
			select += sand + " f.idTipusNotificacio = :" + Notificacio.CRITERI_NOTIFICACIO_ID_TIPUS_NOTIFICACIO;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_DATA_FI)){
			select += sand + " ( f.dataInici <= :" + Notificacio.CRITERI_NOTIFICACIO_DATA_FI + " or f.dataInici is null ) ";
			sand = AND;
		}
		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_DATA_INICI)){
			select += sand + " ( f.dataFi >= :" + Notificacio.CRITERI_NOTIFICACIO_DATA_INICI + " or f.dataFi is null )";
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + Notificacio.CRITERI_NOTIFICACIO_ID_ESTAT;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Notificacio.CRITERI_NOTIFICACIO_ID_USUARI)) {
			select += sand + " f.idUsuari.idUsuari = :" + Notificacio.CRITERI_NOTIFICACIO_ID_USUARI;
			sand = AND;
		}
		
		select += " order by f.dataInici desc ";

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Notificacio notificacio = (Notificacio) it.next();
			if (validaSegonsUsuari(s, p_usuari, notificacio))
				al.add(notificacio);
		}
		Notificacio[] array = new Notificacio[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Notificacio) al.get(i);
		return array;

	}

	/**
	 * Esborrar notificaci
	 * @param p_notificacio
	 * @throws Throwable
	 */
	public void esborrar(Notificacio p_notificacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_notificacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}

	/**
	 * Validar notificacions segons usuari
	 * @param s
	 * @param p_usuari
	 * @param notificacio
	 * @return boolean
	 */
	private boolean validaSegonsUsuari(Session s, Usuari p_usuari, Notificacio notificacio) {
		if (p_usuari instanceof UsuariAdministrador || 
				p_usuari instanceof UsuariAdministratiu ||
				(notificacio instanceof NotificacioEscola)  || 
				notificacio.getIdUsuari().getIdUsuari().equals(p_usuari.getIdUsuari()))
			return true;
		else if (p_usuari instanceof UsuariProfessor)
			return validaSegonsProfessor(s, p_usuari, notificacio);
		else if (p_usuari instanceof UsuariAlumne)
			return validaSegonsAlumne(s, p_usuari, notificacio);
		else if (p_usuari instanceof UsuariPare)
			return validaSegonsPare(s, p_usuari, notificacio);
		return true;
	}

	/**
	 * Validar notificacions segons professor
	 * @param s
	 * @param pUsuari
	 * @param notificacio
	 * @return boolean
	 */
	private boolean validaSegonsProfessor(Session s, Usuari pUsuari,
			Notificacio notificacio) {

		//Cercar per Aula o Assignatura o Curs per taula AssignaturaAulaProfessor//
		String select = "select f  from AssignaturaAulaProfessor as f ";
		if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)) {
			select += ", CursAssignatura as ca ";
		}
		
		String sand = WHERE;


		select += sand + " f.idProfessor.idUsuari = :param0";
		sand = AND;
		if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)) {
			select += sand + " f.idAula.idCurs = ca.idCurs and f.idAssignatura = ca.idAssignatura ";
			select += sand + " ca.idCursAssignatura = :param1 ";
			sand = AND;
		}
		else if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_AULA)) {
			select += sand + " f.idAula <> null AND f.idAula.idAula = :param1";
			sand = AND;
		}
		else if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_CURS)) {
			select += sand + " f.idAula <> null AND f.idAula.idCurs.idCurs = :param1";
			sand = AND;
		}	
		

		Query query = null;
		Iterator it = null;

		query = s.createQuery(select);

		query.setParameter("param0", pUsuari.getIdUsuari());
		if (notificacio instanceof NotificacioAssignatura) {
			query.setParameter("param1", ((NotificacioAssignatura) notificacio).getIdCursAssignatura().getIdCursAssignatura());
		}
		else if (notificacio  instanceof NotificacioAula) {
			query.setParameter("param1", ((NotificacioAula) notificacio).getIdAula().getIdAula());
		}
		else if (notificacio instanceof NotificacioCurs) {
			query.setParameter("param1", ((NotificacioCurs) notificacio).getIdCurs().getIdCurs());
		}



		it = executaConsulta(query);

		while (it.hasNext()) {
			AssignaturaAulaProfessor aap = (AssignaturaAulaProfessor) it.next();
			return true;
		}

		return false;
	}

	/**
	 * Validar segons alumne
	 * @param s
	 * @param pUsuari
	 * @param notificacio
	 * @return boolean
	 */
	private boolean validaSegonsAlumne(Session s, Usuari pUsuari,
			Notificacio notificacio) {

		//Cercar per Aula o Curs o Assignatura de curs per Matricula//
		//Cercar per Aula o Assignatura o Curs per taula AssignaturaAulaProfessor//
		String select = "select f  from Matricula as f ";

		if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)) {
			select += ", CursAssignatura as ca ";
		}

		String sand = WHERE;


		select += sand + " f.idUsuari.idUsuari = :param0";
		sand = AND;
		select += sand + " f.idAnyAcademic.dataInici <= :param1";
		sand = AND;
		select += sand + " f.idAnyAcademic.dataFi >= :param2";
		sand = AND;

		if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)) {
			select += sand + " f.idCurs = ca.idCurs and ca.idCursAssignatura = :param3";
			sand = AND;
		}
		else if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_AULA)) {
			select += sand + " f.idAula.idAula = :param3";
			sand = AND;
		}
		else if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_CURS)) {
			select += sand + " f.idCurs.idCurs = :param3";
			sand = AND;
		}

		Query query = null;
		Iterator it = null;

		query = s.createQuery(select);

		query.setParameter("param0", pUsuari.getIdUsuari());
		query.setParameter("param1", notificacio.getDataFi());
		query.setParameter("param2", notificacio.getDataInici());

		if (notificacio instanceof NotificacioAssignatura) {
			query.setParameter("param3", ((NotificacioAssignatura) notificacio).getIdCursAssignatura().getIdCursAssignatura());
		}
		else if (notificacio instanceof NotificacioAula) {
			query.setParameter("param3", ((NotificacioAula) notificacio).getIdAula().getIdAula());
		}
		else if (notificacio instanceof NotificacioCurs) {
			query.setParameter("param3", ((NotificacioCurs) notificacio).getIdCurs().getIdCurs());
		}


		it = executaConsulta(query);

		while (it.hasNext()) {
			Matricula matricula = (Matricula) it.next();
			return true;
		}


		return false;
	}

	/**
	 * Validar notificacio segons pare/tutor
	 * @param s
	 * @param pUsuari
	 * @param notificacio
	 * @return boolean
	 */
	private boolean validaSegonsPare(Session s, Usuari pUsuari,
			Notificacio notificacio) {

		//Cercar per Aula o Curs o Assignatura de curs per Matricula//
		//Cercar per Aula o Assignatura o Curs per taula AssignaturaAulaProfessor//
		String select = "select f  from PareAlumne f, Matricula as m ";

		if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)) {
			select += ", CursAssignatura as ca ";
		}

		String sand = WHERE;

		select += sand + " f.idPare.idUsuari = :param0";
		sand = AND;

		select += sand + " f.idAlumne.idUsuari = m.idUsuari";
		sand = AND;

		select += sand + " m.idAnyAcademic.dataInici <= :param1";
		sand = AND;
		select += sand + " m.idAnyAcademic.dataFi >= :param2";
		sand = AND;

		if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)) {
			select += sand + " m.idCurs = ca.idCurs and ca.idCursAssignatura = :param3";
			sand = AND;
		}
		else if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_AULA)) {
			select += sand + " m.idAula.idAula = :param3";
			sand = AND;
		}
		else if (notificacio.getIdTipusNotificacio().equals(CodisMestre.CODI_TIPUS_NOTIFICACIO_CURS)) {
			select += sand + " m.idCurs.idCurs = :param3";
			sand = AND;
		}

		Query query = null;
		Iterator it = null;

		query = s.createQuery(select);

		query.setParameter("param0", pUsuari.getIdUsuari());
		query.setParameter("param1", notificacio.getDataFi());
		query.setParameter("param2", notificacio.getDataInici());

		if (notificacio instanceof NotificacioAssignatura) {
			query.setParameter("param3", ((NotificacioAssignatura) notificacio).getIdCursAssignatura().getIdCursAssignatura());
		}
		else if (notificacio instanceof NotificacioAula) {
			query.setParameter("param3", ((NotificacioAula) notificacio).getIdAula().getIdAula());
		}
		else if (notificacio instanceof NotificacioCurs) {
			query.setParameter("param3", ((NotificacioCurs) notificacio).getIdCurs().getIdCurs());
		}


		it = executaConsulta(query);

		while (it.hasNext()) {			
			return true;
		}


		return false;	
	}

}
