package com.uoc.tfc.LN.dao.qualificacio;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.ConfiguracioUsuari;
import com.uoc.tfc.Base.qualificacio.Qualificacio;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.Base.usuari.UsuariPare;
import com.uoc.tfc.Base.usuari.UsuariProfessor;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean Qualificacio
 * 
 */
public class QualificacioDAO extends BaseDAO implements IQualificacioDAO{

	private static final String AND = " and ";
	private static final String OR = " or ";
	private static final String WHERE = " where ";

	/**
	 * Gravar qualificaci
	 * @param p_qualificacio
	 * @return Qualificacio
	 * @throws Throwable
	 */
	public Qualificacio gravar(Qualificacio p_qualificacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Update(s, p_qualificacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return p_qualificacio;
	}

	/**
	 * Cerca qualificaci passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return Qualificacio[]
	 * @throws Throwable
	 */
	public Qualificacio[] findQualificacio(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Qualificacio[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {			
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca qualificaci passant-li parmetres de criteri de cerca en una sessi
	 * @param s
	 * @param p_parametres
	 * @return Qualificacio[]
	 * @throws Throwable
	 */
	private Qualificacio[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from Qualificacio as f ";
		String sand = WHERE;

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_QUALIFICACIO)) {
			select += sand + " f.idQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_QUALIFICACIO;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_USUARI)) {
			select += sand + " f.idAlumne.idUsuari = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_NOM_USUARI)) {
			select += sand + " f.idAlumne.nom like :" + Qualificacio.CRITERI_QUALIFICACIO_NOM_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_COGNOM1_USUARI)) {
			select += sand + " f.idAlumne.cognom1 like :" + Qualificacio.CRITERI_QUALIFICACIO_COGNOM1_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_COGNOM2_USUARI)) {
			select += sand + " f.idAlumne.cognom2 like :" + Qualificacio.CRITERI_QUALIFICACIO_COGNOM2_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_NUM_DOCUMENT_USUARI)) {
			select += sand + " f.idAlumne.numDocument = :" + Qualificacio.CRITERI_QUALIFICACIO_NUM_DOCUMENT_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_DOCUMENT_USUARI) && 
				HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_NUM_DOCUMENT_USUARI)) {
			select += sand + " f.idAlumne.idTipusDocument.idTipusDocument = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_DOCUMENT_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_ESTAT_USUARI)) {
			select += sand + " f.idAlumne.idEstat.idEstat = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_ESTAT_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_ANY_ACADEMIC_USUARI)) {
			select += sand + " f.idAnyAcademic.idAnyAcademic = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_ANY_ACADEMIC_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_AULA)) {
			select += sand + " f.idAsigAulaProf.idAula.idAula = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_AULA;
			sand = AND;
			select += sand + " f.idAsigAulaProf.idAula.idAula > 0 ";
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_ASSIGNATURA)) {
			select += sand + " f.idAsigAulaProf.idAssignatura.idAssignatura = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_ASSIGNATURA;
			sand = AND;
			select += sand + " f.idAsigAulaProf.idAssignatura.idAssignatura > 0 ";
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO)) {
			select += sand + "(f.qualif1.idTipusQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO;
			select += OR + " f.qualif2.idTipusQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO;
			select += OR + " f.qualif3.idTipusQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO;
			select += OR + " f.repesca1.idTipusQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO;
			select += OR + " f.repesca2.idTipusQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO;
			select += OR + " f.qualifFinal.idTipusQualificacio = :" + Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO;
			select += ")";
		}

		if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariProfessor) {
			select += sand + filtreProfessor();
			sand = AND;
		}
		else if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariAlumne) {
			select += sand + filtreAlumne();
			sand = AND;
		}
		else if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariPare) {
			select += sand + filtrePare();
			sand = AND;
		}
		
		select += " order by f.idAlumne.cognom1, f.idAlumne.cognom2, f.idAlumne.nom, f.idAsigAulaProf.idAssignatura.descripcio ";

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Qualificacio usuari = (Qualificacio) it.next();
			al.add(usuari);
		}
		Qualificacio[] array = new Qualificacio[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Qualificacio) al.get(i);
		return array;

	}

	/**
	 * Filtrar qualificacions per professor
	 * @return String
	 */
	private String filtreProfessor() {
		String select = " f.idAsigAulaProf.idProfessor.idUsuari = '" +   
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "'";
		return select;
	}

	/**
	 * Filtrar qualificacions per alumne
	 * @return String
	 */
	private String filtreAlumne() {
		String select = " f.idAlumne.idUsuari = '" +   
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "'";
		return select;
	}

	/**
	 * Filtrar qualificacions per pare/tutor
	 * @return String
	 */
	private String filtrePare() {
		String select = " f.idAlumne.idUsuari in (" +
		"select pa.idAlumne.idUsuari from PareAlumne pa " +
		"where pa.idPare.idUsuari = '" + 		
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "' " + 
		"and pa.idEstat.idEstat = " + CodisMestre.CODI_ESTAT_ALTA + ")"; 
		return select;
	}


	/**
	 * Esborrar qualificaci
	 * @param p_qualificacio
	 * @throws Throwable
	 */
	public void esborrar(Qualificacio p_qualificacio) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();		
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_qualificacio);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}

}