package com.uoc.tfc.LN.dao.usuari;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.Transaction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.ConfiguracioUsuari;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.common.Internacionalitzacio;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.usuari.Usuari;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.Base.usuari.UsuariPare;
import com.uoc.tfc.Base.usuari.UsuariProfessor;
import com.uoc.tfc.LN.dao.BaseDAO;
import com.uoc.tfc.LN.dao.manteniment.AnyAcademicDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.LN.db.HibernateUtil;

/**
 * @author ojaen
 * DAO del bean Usuari
 *
 */
public class UsuariDAO extends BaseDAO implements IUsuariDAO{

	private static final String AND = " and ";
	private static final String WHERE = " where ";
	private static final String OR = " or ";

	/**
	 * Cercar usuari amb login i contrassenya
	 * @param login
	 * @param contrasenya
	 * @return Usuari[]
	 * @throws Throwable
	 */
	public Usuari[] findUsuari(String login, String contrasenya) throws Throwable{
		String select;

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		select = "select f  from Usuari as f ";
		select += " where ";
		select += " f.login = :param0";
		//select += " and f.paswd = :param1";

		Query query = null;
		Iterator it = null;
		try {
			query = s.createQuery(select);

			if (login != null)
				query.setParameter("param0", login);

			//			if (login != null)
			//			    query.setParameter("param1", contrasenya);

			it = executaConsulta(query);
		} catch (Exception ex) {
			throw new Throwable(ex);
		} finally {
			s.close();
		}


		ArrayList al = new ArrayList();
		String p_hashPwd = UtilUsuari.EncriptarPassword(contrasenya);

		while (it.hasNext()) {
			Usuari usuari = (Usuari) it.next();
			if (usuari.getPaswd() == null ||
					usuari.getPaswd().equals("") ||
					usuari.getPaswd().equals(p_hashPwd))
				al.add(usuari);
		}

		Usuari[] array = new Usuari[al.size()];

		for (int i = 0; i < array.length; i++)
			array[i] = (Usuari) al.get(i);

		return array;

	}

	/**
	 * Gravar usuari
	 * @param p_usuari
	 * @return Usuari
	 * @throws Throwable
	 */
	public Usuari gravar(Usuari p_usuari) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			//Nou usuari
			if (p_usuari.getLogin() == null || p_usuari.getLogin().equals("")) {
				p_usuari.setLogin(generaLogin(s, p_usuari.getCognom1(), p_usuari.getNom()).toUpperCase());
			}
			//Si es alumne generem la primera matrcula automticament si no en te cap.
			if (p_usuari instanceof UsuariAlumne &&
				!Helper.validarColleccio( ((UsuariAlumne) p_usuari).getSetOfMatricula())) {
				crearPrimeraMatricula(s, (UsuariAlumne) p_usuari);
			}

			if (p_usuari instanceof UsuariAlumne)
				Update(s, ((UsuariAlumne) p_usuari).getIdDadesBancaries());
			Update(s, p_usuari);
			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return p_usuari;
	}

	/**
	 * Crear primera matricula al crear un usuari
	 * @param s
	 * @param p_usuari
	 * @throws Throwable
	 */
	private void crearPrimeraMatricula(Session s, UsuariAlumne p_usuari) throws Throwable {
		Matricula matricula = new Matricula();
		matricula.setIdAnyAcademic(getUltimAnyAcademic(s));
		matricula.setIdCurs(p_usuari.getIdCursInici());
		matricula.setIdEstat(p_usuari.getIdEstat());
		matricula.setIdUsuari(p_usuari);
		p_usuari.addMatricula(matricula);
	}

	private AnyAcademic getUltimAnyAcademic(Session s) throws Throwable {

		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();
		AnyAcademic[] anysAcademics = null;
		//Primer cerquem per periode de matriculacio
		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI_MATRICULA, Internacionalitzacio.nomesData(new Date()));
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI_MATRICULA, Internacionalitzacio.nomesData(new Date()));

		anysAcademics = anyAcademicDAO.find(s, parametres);
		if (anysAcademics != null && anysAcademics.length > 0)
			return anysAcademics[0];

		//Si no estem en el periode de matriculaci cerquem l'any acadmic vigent
		else {
			parametres.clear();
			parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
			parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI, Internacionalitzacio.nomesData(new Date()));
			parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI, Internacionalitzacio.nomesData(new Date()));

			anysAcademics = anyAcademicDAO.find(s, parametres);
			if (anysAcademics != null && anysAcademics.length > 0)
				return anysAcademics[0];

		}

		return null;

	}

	/**
	 * Generara el login amb 4 primers caracters (o menys del primer cognom) + "_" +
	 * 3 digits del nom. Si el login existeix l'ultima lletra es substituira per un numero
	 * @param s
	 * @param cognom1
	 * @param nom
	 * @return
	 */
	private String generaLogin(Session s, String cognom1, String nom) throws Throwable{
		return generaLogin(s, cognom1, nom, 0);
	}

	/**
	 * Crear login automticament
	 * @param s
	 * @param cognom1
	 * @param nom
	 * @param contador
	 * @return String
	 * @throws Throwable
	 */
	private String generaLogin(Session s, String cognom1, String nom, int contador) throws Throwable{
		String login = "";

		//Treiem espais buits
		cognom1 = cognom1.replace(" ", "");
		nom = nom.replace(" ", "");

		//Cognom
		if (cognom1.length() >= 4)
			login = cognom1.substring(0, 4);
		else
			login = cognom1;
		login += "_";

		//Nom
		if (nom.length() >= 3)
			login += nom.substring(0, 3);
		else
			login += nom;

		String loginFinal = login ;
		if (contador > 0) {
			String sContador = new Integer(contador).toString();

			loginFinal = login.substring(0, login.length() - 1 - (contador / 10));
			loginFinal += sContador;
		}

		HashMap parametres = new HashMap();
		parametres.put(Usuari.CRITERI_USUARI_LOGIN, loginFinal);

		Usuari[] usuaris = find(s, parametres);

		if (!Helper.validarArray(usuaris)) {
			return loginFinal;
		}
		else {
			contador++;
			return generaLogin(s, cognom1, nom, contador);
		}

	}

	/**
	 * Cerca usuari passant-li parmetres de criteri de cerca
	 * @param p_parametres
	 * @return Usuari[]
	 * @throws Throwable
	 */
	public Usuari[] findUsuari(HashMap p_parametres) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;

		/* get session of the current thread */
		s = hu.getSession();

		Usuari[] array = null;
		try {
			array = find(s, p_parametres);
		} catch (Exception ex) {
			throw new Throwable(ex);
		} finally {
			s.close();
		}

		return array;


	}

	/**
	 * Cerca usuari passant-li parmetres de criteri de cerca en una sessio
	 * @param s
	 * @param p_parametres
	 * @return Usuari[]
	 * @throws Throwable
	 */
	private Usuari[] find(Session s, HashMap p_parametres) throws Exception {

		String select = "select f  from Usuari as f ";
		String sand = WHERE;


		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_ID_USUARI) &&
				HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_NOT_EQUAL_ID)) {
			select += sand + " f.idUsuari <> :" + Usuari.CRITERI_USUARI_ID_USUARI;
			sand = AND;
		}
		else if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_ID_USUARI)) {
			select += sand + " f.idUsuari = :" + Usuari.CRITERI_USUARI_ID_USUARI;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_LOGIN)) {
			select += sand + " f.login = :" + Usuari.CRITERI_USUARI_LOGIN;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_NOM)) {
			select += sand + " f.nom like :" + Usuari.CRITERI_USUARI_NOM;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_COGNOM1)) {
			select += sand + " f.cognom1 like :" + Usuari.CRITERI_USUARI_COGNOM1;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_COGNOM2)) {
			select += sand + " f.cognom2 like :" + Usuari.CRITERI_USUARI_COGNOM2;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_NUM_DOCUMENT)) {
			select += sand + " f.numDocument = :" + Usuari.CRITERI_USUARI_NUM_DOCUMENT;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_ID_TIPUS_DOCUMENT) &&
				HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_NUM_DOCUMENT)) {
			select += sand + " f.idTipusDocument.idTipusDocument = :" + Usuari.CRITERI_USUARI_ID_TIPUS_DOCUMENT;
			sand = AND;
		}

		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_ID_TIPUS_USUARI)) {
			select += sand + " f.idTipusUsuari.idTipusUsuari = :" + Usuari.CRITERI_USUARI_ID_TIPUS_USUARI;
			sand = AND;
		}


		if (HelperDB.validarContingutMapa(p_parametres, Usuari.CRITERI_USUARI_ID_ESTAT)) {
			select += sand + " f.idEstat.idEstat = :" + Usuari.CRITERI_USUARI_ID_ESTAT;
			sand = AND;
		}

		if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariProfessor) {
			select += sand + filtreProfessor();
			sand = AND;
		}
		else if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariAlumne) {
			select += sand + filtreAlumne();
			sand = AND;
		}
		else if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariPare) {
			select += sand + filtrePare();
			sand = AND;
		}

		select += " order by f.cognom1, cognom2, nom ";

		Query query = s.createQuery(select);
		query = HelperDB.assignarParametres(query, p_parametres);
		ArrayList al = new ArrayList();

		// Muntem l'array amb els resultats
		Iterator it = executaConsulta(query);
		while (it.hasNext()) {
			Usuari usuari = (Usuari) it.next();
			al.add(usuari);
		}
		Usuari[] array = new Usuari[al.size()];
		for (int i = 0; i < array.length; i++)
			array[i] = (Usuari) al.get(i);
		return array;

	}

	/**
	 * Esborrar usuari
	 * @param p_usuari
	 * @throws Throwable
	 */
	public void esborrar(Usuari p_usuari) throws Throwable{

		HibernateUtil hu = HibernateUtil.getInstance();
		/* a Hibernate session */
		Session s = null;
		/* we always need a transaction */
		Transaction tx = null;

		/* get session of the current thread */
		s = hu.getSession();

		tx = s.beginTransaction();

		try {
			Delete(s, p_usuari);
			if (p_usuari instanceof UsuariAlumne)
				Delete(s, ((UsuariAlumne) p_usuari).getIdDadesBancaries());

			tx.commit();
		} catch (Exception ex) {
			tx.rollback();
			throw new Throwable(ex);
		} finally {
			s.close();
		}

	}

	/**
	 * Filtrar qualificacions per professor
	 * @return String
	 */
	private String filtreProfessor() {
		String select = " (f.idUsuari.idUsuari in (" +
		" select m.idUsuari.idUsuari from Matricula m, AssignaturaAulaProfessor aap " +
		" where m.idAula = aap.idAula and " +
		" aap.idProfessor.idUsuari =  '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "')";
		select += OR;
		select += " f.idUsuari.idUsuari = '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "')";

		return select;
	}

	/**
	 * Filtrar qualificacions per alumne
	 * @return String
	 */
	private String filtreAlumne() {
		String select = " f.idUsuari.idUsuari = '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "'";
		return select;
	}

	/**
	 * Filtrar qualificacions per pare/tutor
	 * @return String
	 */
	private String filtrePare() {
		String select = " (f.idUsuari.idUsuari in (" +
		"select pa.idAlumne.idUsuari from PareAlumne pa " +
		"where pa.idPare.idUsuari = '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "' " +
		"and pa.idEstat.idEstat = " + CodisMestre.CODI_ESTAT_ALTA + ")";
		select += OR;
		select += " f.idUsuari.idUsuari = '" +
		ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari() + "')";
		return select;
	}

}