package com.uoc.tfc.LN.dao.usuari;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *  @author ojaen
 *
 * Mtodes d'utilitat per Usuari
 *
 **/
public class UtilUsuari {

	private static final String LLETRES_NIF = "TRWAGMYFPDXBNJZSQVHLCKE";

	/**
	 * Encriptar la contrasenya
	 * @param p_pwd
	 * @return String
	 * @throws Throwable
	 */
	public static String EncriptarPassword(String p_pwd)  throws Throwable{

		String hashPwd = null;

		EncriptacioMD5 encriptacio = new EncriptacioMD5();
		hashPwd = encriptacio.encriptar(p_pwd);

		return hashPwd;
	}

	/**
	 * Rutina per validar si una adreca de correu-e s vlida o no i, a partir d'una expressi regular,
	 * valida a ms si l'adreca conte un domini vlid.
	 * @param p_aEmailAddress mail a validar
	 * @return boolean: Cert si es el mail s vlid
	 */
	public static boolean esValidEmail(String p_aEmailAddress){
		if (p_aEmailAddress == null) return false;

		p_aEmailAddress = p_aEmailAddress.toLowerCase();

		try {
			Pattern email = Pattern.compile(
					"^[_A-Za-z0-9-]+(\\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+"+
					"(\\.[A-Za-z0-9-]+)*(\\.(?:[A-Za-z]{2}|"+
					//Extret de http://www.iana.org/domains/root/db/#
			"aero|arpa|asia|biz|cat|com|coop|edu|gov|info|int|jobs|mil|mobi|museum|name|net|org|pro|tel|travel))$");
			Matcher fit = email.matcher(p_aEmailAddress);
			return fit.matches();
		} catch (Exception e) {

			return false;
		}
	}

	/**
	 * Valida si un NIF s correcte. Valida la lletra.
	 * @param p_dni
	 * @param p_lletra
	 * @return boolean
	 */
	public static boolean isValidNIF(int p_dni, String p_lletra) {
		int i = p_dni % 23;
		String arrayLetras = LLETRES_NIF;
		return arrayLetras.substring(i,i+1).equalsIgnoreCase(p_lletra);
	}


	/**
	 *  Rutina de validaci nmeros de comptes bancries.
	 *  @param     p_ccc 20 digits del numero de compte
	 *  @return    boolean: Retorna cert si el codi es valid
	 */
	public static boolean isValidCCC(String p_ccc)
	{
		p_ccc = p_ccc.replaceAll("[-.]*", "");
		return isValidCCC(p_ccc.substring(0, 4), p_ccc.substring(4, 8),
				p_ccc.substring(8, 10), p_ccc.substring(10, 20));
	}

	/**
	 *  Rutina de validaci numeros de comptes bancaries.
	 *  @param     p_banc 4 digits del banc
	 *  @param     p_agencia 4 digits agencia
	 *  @param     p_dc 2 digits de control
	 *  @param	  p_compte 10 digits numero compte
	 *  @return    boolean: Retorna cert si el codi es valid
	 */
	public static boolean isValidCCC(String p_banc, String p_agencia, String p_dc, String p_compte)
	{
		long d = calcularDigitControlCCC(Integer.parseInt(p_banc) * 10000 + Integer.parseInt(p_agencia));
		d = d * 10;
		d += calcularDigitControlCCC(Long.parseLong(p_compte));
		return (d == Integer.parseInt(p_dc));
	}

	/**
	 * Calcular el dgit control a partir d'un nmero de compte
	 * @param p_compte
	 * @return int
	 */
	private static int calcularDigitControlCCC(long p_compte) {
		int[] pesos={6,3,7,9,10,5,8,4,2,1};
		int comptador = 0;
		long s=0, d;
		while (p_compte != 0)
		{
			d = p_compte % 10;
			p_compte = p_compte / 10;
			s += d * pesos[comptador];
			comptador++;
		}

		int resultat= (int)(11-(s % 11));
		if (resultat==10) resultat=1;
		else if (resultat==11) resultat=0;

		return resultat;
	}
}