package com.uoc.tfc.LN.db;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.hibernate.Query;
import org.hibernate.Session;

import com.uoc.tfc.Base.common.Helper;

/**
 * Classe que es fan servir en les operacions que es realitzen en la capa de la lgica de negoci
 * @author ojaen
 *
 */
public class HelperDB extends Helper{

	private static final String NAMED_PARAMETER = "MULTI";
	private static final int BLOCS = 1000;


	/*******************************************************************************************************************
	 *
	 * Aquest mtode permet assignar els parametres a una query d'una forma dinamica
	 *
	 * @param p_query
	 * @param p_parametres
	 * @return Query
	 *
	 */
	public static Query assignarParametres(Query p_query, Map p_parametres) {

		String[] parametres = p_query.getNamedParameters();
		if (validarArray(parametres)) {
			for (int i = 0; i < parametres.length; i++) {
				// Nomes recuperarem els namedParameters, no ens interesen aquells elements del mapa
				// que NO s'han assignat a la select.
				String clau = parametres[i];
				if (validarContingutMapa(p_parametres, clau)) {
					Object parametre = p_parametres.get(clau);
					if (parametre instanceof Collection) {
						p_query.setParameterList(clau, (Collection) parametre);
					} else if (parametre instanceof Object[]) {
						p_query.setParameterList(clau, (Object[]) parametre);
					} else if (parametre instanceof String) {
						String cadena = (String) parametre;
						cadena = cadena.replace('*', '%');
						p_query.setParameter(clau, cadena);
					} else if (parametre instanceof Map) {
						// DO NOTHING
					} else if (parametre instanceof Date) {
						// FIXME serr_ain Asegurarmos de que la data va sense hora, minut, segon.
						GregorianCalendar g = new GregorianCalendar();
						g.setTime((Date) parametre);
						g.set(GregorianCalendar.HOUR, 0);
						g.set(GregorianCalendar.MINUTE, 0);
						g.set(GregorianCalendar.SECOND, 0);
						g.set(GregorianCalendar.MILLISECOND, 0);
						p_query.setParameter(clau, g.getTime());
					} else {
						p_query.setParameter(clau, parametre);
					}
				}
			}
		}
		return p_query;
	}

	/**
	 *
	 * Fa la conversi d'un Object a Integer. El parmetre d'entrada pot ser Integer, Double o String
	 *
	 * @param p_obj
	 * @return Integer
	 *
	 */
	public static Integer convertirAInteger(Object p_obj) {

		if (p_obj == null)
			return null;
		Integer valor = null;
		if (p_obj instanceof Integer) {
			valor = (Integer) p_obj;
		} else if (p_obj instanceof Double) {
			valor = new Integer(((Double) p_obj).intValue());
		} else if (p_obj instanceof String) {
			valor = Integer.valueOf((String) p_obj);
		} else if (p_obj instanceof Boolean) {
			if (((Boolean) p_obj).booleanValue()) {
				valor = new Integer(1);
			} else {
				valor = new Integer(0);
			}
		}
		return valor;
	}

	/**
	 *
	 * Mtode per validacions de Integers dins de mapes.NOMES PER US EN DAO. El boole p_zeroEsValid, permet indicar si
	 * el valor 0 es un valor valid Si es valid, retornara true si hi ha un valor. Si p_zeroEsValid = false, retornar
	 * false si l'integer te un valor de 0.
	 *
	 * @param p_map
	 * @param p_clau
	 * @param p_zeroEsValid
	 * @return boolean
	 *
	 */
	public static boolean validarContingutInteger(Map p_map, Object p_clau, boolean p_zeroEsValid) {

		if (validarContingutMapa(p_map, p_clau)) {
			Integer val = convertirAInteger(p_map.get(p_clau));
			p_map.put(p_clau, val);
			if (val == null)
				return false;
			if (!p_zeroEsValid) {
				if (val.intValue() == 0)
					return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * Mtode que retorna un Double passant-li un Object: Integer, Long, Double, String
	 * @param p_obj
	 * @return Double
	 */
	public static Double convertirADouble(Object p_obj) {

		if (p_obj == null)
			return null;
		Double valor = null;
		if (p_obj instanceof Integer)
			valor = (Double) p_obj;
		else if (p_obj instanceof Long)
			valor = (Double) p_obj;
		else if (p_obj instanceof Double)
			valor = (Double) p_obj;
		else if (p_obj instanceof String)
			valor = Double.valueOf((String) p_obj);
		return valor;
	}

	/**
	 * Permet valida si la clau es un Double
	 * @param p_map
	 * @param p_clau
	 * @param p_zeroEsValid
	 * @return boolean
	 */
	public static boolean validarContingutDouble(Map p_map, Object p_clau, boolean p_zeroEsValid) {

		if (validarContingutMapa(p_map, p_clau)) {
			Double val = convertirADouble(p_map.get(p_clau));
			p_map.put(p_clau, val);
			if (val == null)
				return false;
			if (!p_zeroEsValid) {
				if (val.intValue() == 0)
					return false;
			}
			return true;
		}
		return false;
	}

	/**
	 *
	 * Mtode per validacions de Longs dins de mapes.NOMES PER US EN DAO.
	 *
	 * @param p_map
	 * @param p_clau
	 * @return boolean
	 *
	 */
	public static boolean validarContingutLong(Map p_map, Object p_clau) {

		if (validarContingutMapa(p_map, p_clau)) {
			Object element = p_map.get(p_clau);
			if (element instanceof Long)
				return true;
		}
		return false;
	}

	/*******************************************************************************************************************
	 *
	 * Mtode que, valida que el contingut d'un map per una clau sigui un array o coleccio de longitud major a 1. Si el
	 * valor no es array o coleccio, retornara true.
	 *
	 * @param p_map
	 * @param p_clau
	 * @return boolean
	 *
	 */
	public static boolean validarContingutColleccioArrayMapa(Map p_map, Object p_clau) {

		if (validarContingutMapa(p_map, p_clau)) {
			Object obj = (Object) p_map.get(p_clau);
			if (obj instanceof Object[]) {
				Object[] array = (Object[]) obj;
				return array.length > 0;
			} else if (obj instanceof Collection) {
				Collection col = (Collection) obj;
				return col.size() > 0;
			}
			return true;
		}
		return false;
	}

	/**
	 * Eliminar nulls d'una col.leccio
	 * @param p_llista
	 * @return Collection
	 */
	public static Collection buidarNulls(Collection p_llista) {

		Collection retorn = new ArrayList();
		if (p_llista != null) {
			java.util.Iterator it = p_llista.iterator();
			while (it.hasNext()) {
				Object elem = it.next();
				if (elem != null) {
					retorn.add(elem);
				}
			}
		}
		return retorn;
	}

	/***
	 *
	 * Mtode que permet partir les listes de mes de mil elements per fer querys.
	 * MOLT IMPORTANT si es fa servir per querys on l'unic parametre
	 * de la query no es la llista d'elements, cal assegurarse que despres
	 * s'assingen correctament a la query la resta de paramentres!!!
	 *
	 * NOTA ADICIONAL. Cal que els in o not in a les select estiguin en minuscules.
	 *
	 * select i from insercio as i where i.IdInsercio in (:llistaParametres) correcte
	 * select i from insercio as i where i.IdInsercio IN (:llistaParametres) incorrecte
	 *
	 *@param p_values la llista d'elements
	 *@param p_query la query a traduir
	 *@param p_param el nom del parametre associat a la llista
	 *@param p_session la sessio de treball
	 *@return Query
	 *
	 */
	public static Query traduirIn(Collection p_values, Query p_query, String p_param, Session p_session) {

		// mida de la coleccio
		int size = p_values.size();
		// mod per saber quants elements van a l'ultima in
		int mod = (size % BLOCS);
		if (size <= BLOCS) {
			return p_query;
		}
		// Quants in es creen.
		int numberOfIn = (size / BLOCS) + (mod == 0 ? 0 : 1);
		// cache de nous parametres
		List llistaParametres = new ArrayList();
		Query newQuery = partComuna(p_query, p_session, llistaParametres, p_param, numberOfIn);
		// recuperem els valors associats al paramatres
		List copy = new ArrayList(p_values);
		int ref = BLOCS;
		for (int count = 0, from = 0, to = ref; count < numberOfIn; count++) {
			String nameParam = (String) llistaParametres.get(count);
			// informem els nous named parameters amb una sublista dels valors originals, amb un maxim de mil elements.
			newQuery.setParameterList(nameParam, copy.subList(from, to));
			from = to;
			if (count + 2 < numberOfIn) {
				to += BLOCS;
			} else if (count + 2 == numberOfIn) {
				to += (mod == 0) ? BLOCS : mod;
			}
		}
		return newQuery;
	}



	/**
	 *
	 * Dividir correctament totes les llistes amb mes de mil elements.
	 * Fer servir nomes si es posible que les llistes superin els mil elements.
	 * Si es fa servir aquest mtode conjuntament amb el mtode setParameter o setParameterList de la query,
	 * cal que aquests es facin servir despres d'aquest mtode.
	 *
	 *  NOTA ADICIONAL. Cal que els in o not in a les select estiguin en minuscules.
	 *  select i from insercio as i where i.IdInsercio in (:llistaParametres) correcte
	 * 	select i from insercio as i where i.IdInsercio IN (:llistaParametres) incorrecte
	 *
	 * @param p_query
	 * @param p_parametres
	 * @param p_sessio
	 * @return Query
	 *
	 */
	public static Query assignarParametresDividintIn(Query p_query, Map p_parametres, Session p_sessio) {

		String[] parametres = p_query.getNamedParameters();
		Map mapAux = new HashMap();
		if (validarArray(parametres)) {
			for (int i = 0; i < parametres.length; i++) {
				// Nomes recuperarem els namedParameters, no ens interesen aquells elements del mapa
				// que NO s'han assignat a la select.
				String clau = parametres[i];
				if (validarContingutMapa(p_parametres, clau)) {
					Object parametre = p_parametres.get(clau);
					if (parametre instanceof Collection) {
						if (((Collection) parametre).size() > BLOCS) {
							mapAux.put(clau, parametre);
						}
					} else if (parametre instanceof Object[]) {
						Object[] objectes = (Object[]) parametre;
						if (objectes.length > BLOCS) {
							mapAux.put(clau, Arrays.asList(objectes));
						}
					}
				}
			}
		}
		Query querySortida = p_query;
		if (HelperDB.validarMapa(mapAux)) {
			Iterator itera = mapAux.keySet().iterator();
			while (itera.hasNext()) {
				String clau = (String) itera.next();
				Collection valors = (Collection) mapAux.get(clau);
				querySortida = dividirInPerAssignarParametres(valors, clau, p_parametres, querySortida, p_sessio);
			}
		}
		querySortida = assignarParametres(querySortida, p_parametres);
		return querySortida;
	}

	/**
	 * Dividir la query per assignar els parmetres
	 * @param p_values
	 * @param p_param
	 * @param p_parametres
	 * @param p_query
	 * @param p_session
	 * @return Query
	 */
	private static Query dividirInPerAssignarParametres(Collection p_values, String p_param, Map p_parametres,
			Query p_query, Session p_session) {

		// mida de la coleccio
		int size = p_values.size();
		// mod per saber quants elements van a l'ultima in
		int mod = (size % BLOCS);
		if (size <= BLOCS) {
			return p_query;
		}
		// Quants in es creen.
		int numberOfIn = (size / BLOCS) + (mod == 0 ? 0 : 1);
		// cache de nous parametres
		List llistaParametres = new ArrayList();
		Query newQuery = partComuna(p_query, p_session, llistaParametres, p_param, numberOfIn);
		// recuperem els valors associats al paramatres
		List copy = new ArrayList(p_values);
		int ref = BLOCS;
		for (int count = 0, from = 0, to = ref; count < numberOfIn; count++) {
			final String nameParam = (String) llistaParametres.get(count);
			//informem els nous named parameters amb una sublista dels valors originals, amb un maxim de mil elements.
			p_parametres.put(nameParam, copy.subList(from, to));
			from = to;
			if (count + 2 < numberOfIn) {
				to += BLOCS;
			} else if (count + 2 == numberOfIn) {
				to += (mod == 0) ? BLOCS : mod;
			}
		}
		p_parametres.remove(p_param);
		return newQuery;
	}

	/**
	 *
	 * @param p_query
	 * @param p_session
	 * @param p_llistaparams
	 * @param p_param
	 * @param p_numberOfIn
	 * @return Query
	 */
	private static Query partComuna(Query p_query, Session p_session, List p_llistaparams, String p_param,
			int p_numberOfIn) {

		/*
		 * Expressio regular pel patrons  obj.propietat.id in (items) o  obj.property.id not in (items)
		 */
		String regex = "([\\w\\.]+)\\s+(not\\s+)?in\\s*\\(\\s*:" + p_param + "\\s*\\)";
		String in = new String(" $2 (");
		for (int i = 0; i < p_numberOfIn; i++) {
			//	creem nous named parameter
			String newNameParam = new String();
			newNameParam += (p_param);
			newNameParam += (NAMED_PARAMETER);

			//Obtenim el nom del parametre. S'ha vist que quan es segmentava en mes de
			//10 parametres es perdien registres. Al formatejar el format de la numeracio
			//aquest problema ha desaparegut.
			DecimalFormat formatNumberOfIn = new DecimalFormat("#########0");

			double logBase10 = Math.log(Double.valueOf(String.valueOf(p_numberOfIn)).doubleValue()) /
			Math.log(10) ;

			int formatDecimal = Integer.valueOf(formatNumberOfIn.format(logBase10 ) ) .intValue() + 1;

			String sFormatDecimal = "";
			for (int j= 0; j<formatDecimal;j++) {
				sFormatDecimal += "0";
			}
			newNameParam += new DecimalFormat(sFormatDecimal).format(i);

			in += ("$1 in (:");
			in += (newNameParam.toString());
			in += (")");
			p_llistaparams.add(newNameParam.toString());
			if (i + 1 < p_numberOfIn) {
				in += (" or ");
			}
		}
		in += (") ");// tanquem el nou bloc d'in.
		//			 remplacem l'expressio antiga per la nova expressio.
		String newQueryString = p_query.getQueryString().replaceAll(regex, in.toString());
		// Creem una nova query
		Query newQuery = p_session.createQuery(newQueryString);
		return newQuery;
	}

	/**
	 * En el cas d'error en LN retornar un error
	 * @param p_e
	 * @param error
	 * @return ActionErrors
	 */
	public static ActionErrors tractarError(Throwable p_e, String error){
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		errors.add(error,new ActionError("error.excepcio.basedades"));
		return errors;
	}

}
