package com.uoc.tfc.LN.db;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.cfg.Configuration;

/**
 * @author ojaen
 * Configura i proporciona acces a les sessions d'Hibernate.
 */
public class HibernateSessionFactory {

	/** 
	 * Localitzacio del fitxer de configuracio d'hiberante hibernate.cfg.xml.
	 */
	private static String CONFIG_FILE_LOCATION = "/hibernate.cfg.xml";

	/** Instancia de Session */
	private static final ThreadLocal<Session> threadLocal = new ThreadLocal<Session>();

	/** Instancia of hibernate configuration */
	private static final Configuration cfg = new Configuration();

	/** Instancia de hibernate SessionFactory */
	private static org.hibernate.SessionFactory sessionFactory;

	/**
	 * Retorna ThreadLocal Session instance.
	 *
	 *  @return Session
	 *  @throws HibernateException
	 */
	public static Session currentSession() throws HibernateException {
		Session session = (Session) threadLocal.get();

		if (session != null && ! session.isConnected())
			session = null;
		if (session == null) {
			if (sessionFactory == null) {
				try {
					cfg.configure(CONFIG_FILE_LOCATION);
					sessionFactory = cfg.buildSessionFactory();
				}
				catch (Exception e) {
					System.err.println("%%%% Error Creating SessionFactory %%%%");
					e.printStackTrace();
				}
			}
			session = sessionFactory.openSession();
			threadLocal.set(session);
		}

		return session;
	}

	/**
	 *  Tanca la sessi .
	 *
	 *  @throws HibernateException
	 */
	public static void closeSession() throws HibernateException {
		Session session = (Session) threadLocal.get();
		threadLocal.set(null);

		if (session != null) {
			session.close();
		}
	}

	/**
	 * Constructor.
	 */
	private HibernateSessionFactory() {
	}

}
