package com.uoc.tfc.LN.db;

import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.hibernate.Session;
import org.hibernate.SessionFactory;

/**
 * Classe per la configuraci d'Hibernate
 * @author ojaen
 *
 */
public class HibernateUtil {
	/**
	 * Classe encarregada d'alliberar HibernateUtil quan es pari l'aplicacio.<br/>
	 * Detecta que l'aplicaci s'ha aturat perque s'alliberen els objectes que hi ha a JNDI<br/>
	 * Mecanisme: Al parar l'aplicaci, s'allibera el contingut de JNDI i automticament
	 * aquesta classe allibera HibernateUtil.<br/>
	 *
	 * @author ojaen
	 */
	private final class AlliberadorViaJNDI {
		private boolean actiu=false;
		/**
		 * Registra aquest object a JNDI perque quan l'arbre JNDI es destrueixi, aquest sigui alliberat.
		 */
		public void registrar(){
			try {
				new InitialContext().bind("java:comp/gestioacademica", this);
				actiu=true;
			} catch (NamingException ex) {
			}
		}
	}


	/** Instncia nica al SessionFactory */
	private static org.hibernate.SessionFactory sessionFactory;

	private static HibernateUtil instancia = null;

	//Funcionament inicial HibernateUtil...
	// Create the initial SessionFactory from the default configuration files
	private HibernateUtil()
	{
		new AlliberadorViaJNDI().registrar();
	}

	protected void finalize() throws Throwable {
		super.finalize();
	}



	/**
	 * Recupera la configuracio d'hibernate d'arquitectura
	 * @return HibernateUtil
	 */
	public static HibernateUtil getInstance(){
		if (instancia == null)
			instancia = new HibernateUtil();

		return instancia;
	}


	/**
	 * Getter de la sessi
	 * @return Session
	 */
	public Session getSession()  {
		return HibernateSessionFactory.currentSession();
	}

	/**
	 * Tanca la configuracio d'hibernate.
	 *
	 * No es possible tornar a obrir-la.
	 */
	public void close(){
		if (sessionFactory!=null)
			setSessionFactory(null);
	}


	/**
	 * Assigna la nova sessionFactory, alliberant l'antiga si n'hi ha.
	 * @param p_sf
	 */
	private static void setSessionFactory(SessionFactory p_sf){
		sessionFactory.close();
	}

}
