package com.uoc.tfc.UI.struts.action.common;

import java.util.HashMap;
import java.util.Map;


/**
 * Classe que controla el Context: permet definir variables d'entrada i sortida 
 * i accions.
 * @author ojaen
 *  
 */
public class Context {

	private HashMap variablesEntrada = new HashMap();
	private HashMap variablesSortida = new HashMap();
	private HashMap accions = new HashMap();

	/**
	 * Constructor
	 */
	public Context(){

	}

	/**
	 * Constructor
	 * @param context
	 */
	public Context(Context context){
		super();
		this.addTotesVariablesEntrada(context.variablesEntrada);		
	}

	/**
	 * Afegim variable d'entrada
	 * @param identificador
	 * @param variable
	 */
	public void addVariableEntrada(String identificador, Object variable){
		variablesEntrada.put(identificador, variable);
	}

	/**
	 * Getter d'una variable d'entrada
	 * @param identificador
	 * @return Object
	 */
	public Object getVariableEntrada(String identificador){
		return variablesEntrada.get(identificador);
	}

	/**
	 * Esborrem variable d'entrada
	 * @param identificador
	 */
	public void removeVariableEntrada(String identificador){
		variablesEntrada.remove(identificador);
	}

	/**
	 * Afegim totes les variables d'entrada de cop
	 * @param mapaVariables
	 */
	protected void addTotesVariablesEntrada(Map mapaVariables){
		variablesEntrada.putAll(mapaVariables);
	}

	/**
	 * Afegim variable de sortida
	 * @param identificador
	 * @param variable
	 */
	public void addVariableSortida(String identificador, Object variable){
		variablesSortida.put(identificador, variable);
	}

	/**
	 * Esborrem variable de sortida
	 * @param identificador
	 */
	public void removeVariableSortida(String identificador){
		variablesSortida.remove(identificador);
	}

	/**
	 * Getter d'una variable de sortida
	 * @param identificador
	 * @return Object
	 */
	public Object getVariableSortida(String identificador){
		return variablesSortida.get(identificador);
	}

	/**
	 * Afegim un accio
	 * @param identificador
	 * @param accio
	 */
	public void addAccio(String identificador, Object accio){
		accions.put(identificador, accio);
	}

	/**
	 * Esborrem una accio
	 * @param identificador
	 */
	public void removeAccio(String identificador){
		accions.remove(identificador);
	}

	/**
	 * Getter d'una accio
	 * @param identificador
	 * @return Accio
	 */
	public Accio getAcccio(String identificador){
		return (Accio) accions.get(identificador);
	}

	/**
	 * Getter variables de sortida
	 * @return HashMap
	 */
	public HashMap getVariablesSortida() {
		return variablesSortida;
	}

	/**
	 * Setter de variables de sortida
	 * @param variablesSortida
	 */
	public void setVariablesSortida(HashMap variablesSortida) {
		this.variablesSortida = variablesSortida;
	}

	/**
	 * Getter de les variables d'entrada
	 * @return HashMap
	 */
	public HashMap getVariablesEntrada() {
		return variablesEntrada;
	}

	/**
	 * Setter variables d'entrada
	 * @param variablesEntrada
	 */
	public void setVariablesEntrada(HashMap variablesEntrada) {
		this.variablesEntrada = variablesEntrada;
	}
}
