package com.uoc.tfc.UI.struts.action.manteniment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.qualificacio.Qualificacio;
import com.uoc.tfc.LN.dao.manteniment.AnyAcademicDAO;
import com.uoc.tfc.LN.dao.matricula.MatriculaDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.dao.qualificacio.QualificacioDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.manteniment.EditAnyAcademicForm;

/**
 * Manteniment dels anys acadmics
 * @author ojaen
 *
 */
public class EditAnyAcademicAction extends DispatchAction {

	/**
	 * Preparem el formulari per editar  l'any acadmic
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {

		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));


		EditAnyAcademicForm anyAcademicEditForm = (EditAnyAcademicForm) form;

		//Recuperem el id de l'any acadmic seleccionat
		Integer id = Integer.valueOf(request.getParameter("id"));
		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC, id);

		AnyAcademic[] anysAcademics = null;
		Estat[] arrEstats = null;
		try {

			//Cerquem l'any acadmic seleccionat
			anysAcademics = anyAcademicDAO.findAnyAcademic(parametres);

			//Inicialitzem els estats
			inicialitzarEstats(request, anyAcademicEditForm, anysAcademics);


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAnyAcademicError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		anyAcademicEditForm.setAnyAcademic(anysAcademics[0]);

		return mapping.findForward("editAnyAcademic");
	}

	/**
	 * Preparem el formulari per afegir un nou any acadmic
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {

		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		//Recuperem els estats
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));

		//set the collection in the request
		request.setAttribute("estats", estats);

		return mapping.findForward("editAnyAcademic");

	}


	/**
	 * Gravem un any acadmic
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveAnyAcademic(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAnyAcademicForm anyAcademicEditForm = (EditAnyAcademicForm) form;

		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();
		MestreDAO mestreDAO = new MestreDAO();

		//Cerquem l'Estat seleccionat i l'assignem al tipus de qualificaci
		Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, anyAcademicEditForm.getSelectedEstat());
		anyAcademicEditForm.getAnyAcademic().setIdEstat((Estat) o[0]);

		//Inicialitzem els estats
		inicialitzarEstats(request, anyAcademicEditForm, null);

		//Validem formulari
		ActionErrors errors = validar(request, anyAcademicEditForm);

		if (errors != null && errors.size() > 0) {
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		try {
			anyAcademicDAO.gravar(anyAcademicEditForm.getAnyAcademic());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAnyAcademicError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaAnyAcademic");
	}

	/**
	 * Esborrem un any acadmic
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteAnyAcademic(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAnyAcademicForm anyAcademicEditForm = (EditAnyAcademicForm) form;

		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC, anyAcademicEditForm.getAnyAcademic().getIdAnyAcademic());

		AnyAcademic[] anysAcademics = null;

		try {
			anysAcademics = anyAcademicDAO.findAnyAcademic(parametres);

			inicialitzarEstats(request, anyAcademicEditForm, null);

			ActionErrors errors = validarElimina(request, anysAcademics[0]);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward("failureCerca"));
			}

			anyAcademicDAO.esborrar(anysAcademics[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "anyAcademicError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("cercaAnyAcademic");
	}


	/**
	 * Consultem un any acadmic
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaAnyAcademic(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAnyAcademicForm anyAcademicEditForm = (EditAnyAcademicForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC, id);

		AnyAcademic[] anysAcademics = null;

		try {
			anysAcademics = anyAcademicDAO.findAnyAcademic(parametres);
			anyAcademicEditForm.setAnyAcademic(anysAcademics[0]);

			Estat[] arrEstats = HelperUI.recuperaEstats();
			//define a dummy collection
			ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
			anyAcademicEditForm.setEstats(estats);
			anyAcademicEditForm.setSelectedEstat(anysAcademics[0].getIdEstat().getIdEstat());

			//set the collection in the request
			request.setAttribute("estats", estats);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAnyAcademicError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editAnyAcademic");
	}

	/**
	 * Validaci de la pantalla d'anys academics
	 * @param request
	 * @param anyAcademicEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditAnyAcademicForm anyAcademicEditForm) {
		boolean bValid = true;
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Valida
		if (anyAcademicEditForm.getAnyAcademic().getCodi() == null ||
				anyAcademicEditForm.getAnyAcademic().getCodi().equals("")) {
			// crear ActionError i desar in the request
			errors.add("codi",new ActionError("error.codi.required"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (anyAcademicEditForm.getAnyAcademic().getDescripcio() == null ||
				anyAcademicEditForm.getAnyAcademic().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);
			bValid = false;

		}

		if (anyAcademicEditForm.getAnyAcademic().getDataInici() == null) {
			// crear ActionError i desar in the request
			errors.add("dataInici",new ActionError("error.data.required"));
			saveErrors(request,errors);
			bValid = false;
		}

		if (anyAcademicEditForm.getAnyAcademic().getDataFi() == null) {
			errors.add("dataFi",new ActionError("error.data.required"));
			saveErrors(request,errors);
			bValid = false;
		}

		if (anyAcademicEditForm.getAnyAcademic().getDataIniciMatricula() == null) {
			errors.add("dataIniciMatricula",new ActionError("error.data.required"));
			saveErrors(request,errors);
			bValid = false;
		}

		if (anyAcademicEditForm.getAnyAcademic().getDataFiMatricula() == null) {
			errors.add("dataFiMatricula",new ActionError("error.data.required"));
			saveErrors(request,errors);
			bValid = false;

		}
		//Validem dates
		if (anyAcademicEditForm.getAnyAcademic().getDataInici() != null &&
				anyAcademicEditForm.getAnyAcademic().getDataFi() != null &&
				anyAcademicEditForm.getAnyAcademic().getDataInici().after(
						anyAcademicEditForm.getAnyAcademic().getDataFi())) {
			errors.add("dataInici", new ActionError("error.rangDates.dates"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (anyAcademicEditForm.getAnyAcademic().getDataIniciMatricula() != null &&
				anyAcademicEditForm.getAnyAcademic().getDataFiMatricula() != null &&
				anyAcademicEditForm.getAnyAcademic().getDataIniciMatricula().after(
						anyAcademicEditForm.getAnyAcademic().getDataFiMatricula())) {
			errors.add("dataIniciMatricula", new ActionError("error.rangDates.dates"));
			saveErrors(request,errors);
			bValid = false;
		}

		//Validem que no hi hagi un altre any acadmic que es solapi amb l'actual
		if (bValid) {
			AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();

			//Parametres
			HashMap parametres = new HashMap();
			if (anyAcademicEditForm.getAnyAcademic().getDataInici() != null)
				parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI,
						anyAcademicEditForm.getAnyAcademic().getDataInici());

			if (anyAcademicEditForm.getAnyAcademic().getDataFi() != null)
				parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI,
						anyAcademicEditForm.getAnyAcademic().getDataFi());
			if (anyAcademicEditForm.getAnyAcademic().getIdAnyAcademic() != null) {
				parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC,
						anyAcademicEditForm.getAnyAcademic().getIdAnyAcademic());
				parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_NOT_EQUAL_ID,
						AnyAcademic.CRITERI_ANY_ACADEMIC_NOT_EQUAL_ID);
			}

			parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

			AnyAcademic[] anysAcademics = null;
			Estat[] arrEstats = null;
			try {

				//Cerquem l'any acadmic seleccionat
				anysAcademics = anyAcademicDAO.findAnyAcademic(parametres);

				if (Helper.validarArray(anysAcademics)) {
					errors.add("editAnyAcademicError", new ActionError("error.solapament.dates"));
					saveErrors(request,errors);
				}
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "editAnyAcademicError"));
			}
		}

		return errors;
	}

	/**
	 * Validaci de l'eliminaci d'un any acadmic. No ha d'existir cap matrcula associada.
	 * @param request
	 * @param anyAcademic
	 * @return ActionsErrors
	 */
	private ActionErrors validarElimina(HttpServletRequest request, AnyAcademic anyAcademic) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		MatriculaDAO matriculaDAO = new MatriculaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Matricula.CRITERI_MATRICULA_ID_ANY_ACADEMIC_USUARI,
				anyAcademic.getIdAnyAcademic());
		Matricula[] matricules = null;
		try {

			//Cerquem l'any acadmic seleccionat
			matricules = matriculaDAO.findMatricula(parametres);

			if (Helper.validarArray(matricules)) {
				errors.add("anyAcademicError", new ActionError("error.elimina.anyacademic"));
				saveErrors(request,errors);
			}
			else {
				return validarEliminaQualificacions(request, anyAcademic);
			}

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "anyAcademicError"));
		}

		return errors;
	}

	/**
	 * Validaci de l'eliminaci d'un any acadmic. No ha d'existir cap qualificaci associada.
	 * @param request
	 * @param anyAcademic
	 * @return ActionsErrors
	 */
	private ActionErrors validarEliminaQualificacions(HttpServletRequest request, AnyAcademic anyAcademic) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();

		Qualificacio[] qualificacions = null;
		QualificacioDAO qualificacioDAO = new QualificacioDAO();
		HashMap parametres = new HashMap();
		parametres.put(Qualificacio.CRITERI_QUALIFICACIO_ID_ANY_ACADEMIC_USUARI,
				anyAcademic.getIdAnyAcademic());

		try {
			//Cerquem l'any acadmic seleccionat
			qualificacions = qualificacioDAO.findQualificacio(parametres);

			if (Helper.validarArray(qualificacions)) {
				errors.add("anyAcademicError", new ActionError("error.elimina.anyacademic"));
				saveErrors(request,errors);
			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "anyAcademicError"));
		}


		return errors;
	}




	/**
	 * Inicialitzaci dels estats
	 * @param request
	 * @param anyAcademicEditForm
	 * @param anysAcademics
	 */
	private void inicialitzarEstats(HttpServletRequest request,
			EditAnyAcademicForm anyAcademicEditForm, AnyAcademic[] anysAcademics) {
		Estat[] arrEstats;
		//Recuperem els estats
		arrEstats = HelperUI.recuperaEstats();
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		anyAcademicEditForm.setEstats(estats);
		if (anysAcademics != null)
			anyAcademicEditForm.setSelectedEstat(anysAcademics[0].getIdEstat().getIdEstat());

		//set the collection in the request
		request.setAttribute("estats", estats);
	}




}
