package com.uoc.tfc.UI.struts.action.manteniment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.Assignatura;
import com.uoc.tfc.Base.manteniment.AssignaturaAulaProfessor;
import com.uoc.tfc.Base.manteniment.CursAssignatura;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.LN.dao.manteniment.AssignaturaDAO;
import com.uoc.tfc.LN.dao.manteniment.CursAssignaturaDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.manteniment.EditAssignaturaForm;

/**
 * Manteniment de les assignatures
 * @author ojaen
 *
 */
public class EditAssignaturaAction extends DispatchAction {

	/**
	 * Prepararar l'edici de l'assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAssignaturaForm assignaturaEditForm = (EditAssignaturaForm) form;

		//Recuperem el id de l'assignatura seleccionada
		Integer id = Integer.valueOf(request.getParameter("id"));
		AssignaturaDAO assignaturaDAO = new AssignaturaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA, id);

		Assignatura[] assignatures = null;
		Estat[] arrEstats = null;

		try {
			//Cerquem l'assignatura seleccionada
			assignatures = assignaturaDAO.findAssignatura(parametres);

			//Inicialitzem els estats
			inicialitzarEstats(request, assignaturaEditForm, assignatures);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAssignaturaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		assignaturaEditForm.setAssignatura(assignatures[0]);

		return mapping.findForward("editAssignatura");
	}



	/**
	 * Preparem el formulari per afegir una nova assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		//Recuperem els estats
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));

		//set the collection in the request
		request.setAttribute("estats", estats);


		return mapping.findForward("editAssignatura");

	}

	/**
	 * Gravem una assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAssignaturaForm assignaturaEditForm = (EditAssignaturaForm) form;

		AssignaturaDAO assignaturaDAO = new AssignaturaDAO();
		MestreDAO mestreDAO = new MestreDAO();

		//Cerquem l'Estat seleccionat i l'assignem al tipus de qualificaci
		Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, assignaturaEditForm.getSelectedEstat());
		assignaturaEditForm.getAssignatura().setIdEstat((Estat) o[0]);

		//Inicialitzem els estats
		inicialitzarEstats(request, assignaturaEditForm, null);

		//Validem formulari
		ActionErrors errors = validar(request, assignaturaEditForm);

		if (errors != null && errors.size() > 0) {
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		try {
			assignaturaDAO.gravar(assignaturaEditForm.getAssignatura());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAssignaturaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaAssignatura");
	}

	/**
	 * Esborrem una assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAssignaturaForm assignaturaEditForm = (EditAssignaturaForm) form;

		AssignaturaDAO assignaturaDAO = new AssignaturaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA, assignaturaEditForm.getAssignatura().getIdAssignatura());


		Assignatura[] assignatures = null;

		try {
			assignatures = assignaturaDAO.findAssignatura(parametres);

			inicialitzarEstats(request, assignaturaEditForm, null);

			ActionErrors errors = validarElimina(request, assignatures[0]);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward("failureCerca"));
			}

			assignaturaDAO.esborrar(assignatures[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "assignaturaError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("cercaAssignatura");
	}


	/**
	 * Consultem una assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAssignaturaForm assignaturaEditForm = (EditAssignaturaForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		AssignaturaDAO assignaturaDAO = new AssignaturaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA, id);

		Assignatura[] assignatures = null;

		try {
			assignatures = assignaturaDAO.findAssignatura(parametres);
			assignaturaEditForm.setAssignatura(assignatures[0]);

			Estat[] arrEstats = HelperUI.recuperaEstats();
			//define a dummy collection
			ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
			assignaturaEditForm.setEstats(estats);
			assignaturaEditForm.setSelectedEstat(assignatures[0].getIdEstat().getIdEstat());

			//set the collection in the request
			request.setAttribute("estats", estats);


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAssignaturaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editAssignatura");
	}

	/**
	 * Validaci de la pantalla d'assignatures
	 * @param request
	 * @param assignaturaEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditAssignaturaForm assignaturaEditForm) {
		boolean bValid = true;
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Valida
		if (assignaturaEditForm.getAssignatura().getCodiAssignatura() == null ||
				assignaturaEditForm.getAssignatura().getCodiAssignatura().equals("")) {
			// crear ActionError i desar in the request
			errors.add("codi",new ActionError("error.codi.required"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (assignaturaEditForm.getAssignatura().getDescripcio() == null ||
				assignaturaEditForm.getAssignatura().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);
			bValid = false;

		}


		//Validem que no hi hagi un altre assignatura amb el mateix codi
		if (bValid) {
			AssignaturaDAO assignaturaDAO = new AssignaturaDAO();

			//Parametres
			HashMap parametres = new HashMap();
			if (assignaturaEditForm.getAssignatura().getCodiAssignatura() != null)
				parametres.put(Assignatura.CRITERI_ASSIGNATURA_CODI,
						assignaturaEditForm.getAssignatura().getCodiAssignatura());

			if (assignaturaEditForm.getAssignatura().getIdAssignatura() != null) {
				parametres.put(Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA,
						assignaturaEditForm.getAssignatura().getIdAssignatura());
				parametres.put(Assignatura.CRITERI_ASSIGNATURA_NOT_EQUAL_ID,
						Assignatura.CRITERI_ASSIGNATURA_NOT_EQUAL_ID);
			}

			parametres.put(Assignatura.CRITERI_ASIGNATURA_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

			Assignatura[] assignatures = null;
			Estat[] arrEstats = null;
			try {

				//Cerquem l'assignatura seleccionat
				assignatures = assignaturaDAO.findAssignatura(parametres);

				if (Helper.validarArray(assignatures)) {
					errors.add("editAssignaturaError", new ActionError("error.assignatura.repetit"));
					saveErrors(request,errors);
				}
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "editAssignaturaError"));
			}
		}




		return errors;
	}


	/**
	 * Validaci de la eliminaci d'un assignatura. No ha d'existir cap curs associat.
	 * @param request
	 * @param assignatura
	 * @return ActionsErrors
	 */
	private ActionErrors validarElimina(HttpServletRequest request, Assignatura assignatura) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		CursAssignaturaDAO cursAssignaturaDAO = new CursAssignaturaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(CursAssignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA,
				assignatura.getIdAssignatura());
		CursAssignatura[] cursAssignatures = null;
		try {

			//Cerquem el cursAssignatura seleccionat
			cursAssignatures = cursAssignaturaDAO.findCursAssignatura(parametres);

			if (Helper.validarArray(cursAssignatures)) {
				errors.add("assignaturaError", new ActionError("error.elimina.assignatura"));
				saveErrors(request,errors);
			}
			else
				return validaEliminaAssignaturaAulaProfessor(request, assignatura);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "assignaturaError"));
		}

		return errors;
	}

	/**
	 * Validaci de la eliminaci d'un assignatura. No ha d'existir cap assignatura associada a una aula/professor.
	 * @param request
	 * @param assignatura
	 * @return ActionsErrors
	 */
	private ActionErrors validaEliminaAssignaturaAulaProfessor(HttpServletRequest request, Assignatura assignatura) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		AssignaturaDAO assignaturaDAO = new AssignaturaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(CursAssignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA,
				assignatura.getIdAssignatura());
		AssignaturaAulaProfessor[] assignaturaAulaProfessors = null;
		try {

			//Cerquem el cursAssignatura seleccionat
			assignaturaAulaProfessors = assignaturaDAO.findAssignaturaAulaProfessor(parametres);

			if (Helper.validarArray(assignaturaAulaProfessors)) {
				errors.add("assignaturaError", new ActionError("error.elimina.assignatura"));
				saveErrors(request,errors);
			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "assignaturaError"));
		}

		return errors;
	}



	/**
	 * Inicialitzacio dels estats
	 * @param request
	 * @param anyAcademicEditForm
	 * @param anysAcademics
	 */
	private void inicialitzarEstats(HttpServletRequest request,
			EditAssignaturaForm assignaturaEditForm, Assignatura[] assignatures) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		assignaturaEditForm.setEstats(estats);
		if (assignatures != null)
			assignaturaEditForm.setSelectedEstat(assignatures[0].getIdEstat().getIdEstat());

		//set the collection in the request
		request.setAttribute("estats", estats);
	}


}
