package com.uoc.tfc.UI.struts.action.manteniment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.AssignaturaAulaProfessor;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.manteniment.CursAssignatura;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.mestres.TipusDocument;
import com.uoc.tfc.Base.mestres.TipusUsuari;
import com.uoc.tfc.Base.usuari.ProfessorAssignatura;
import com.uoc.tfc.Base.usuari.Usuari;
import com.uoc.tfc.Base.usuari.UsuariProfessor;
import com.uoc.tfc.LN.dao.manteniment.AulaDAO;
import com.uoc.tfc.LN.dao.matricula.MatriculaDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.dao.usuari.UsuariDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.manteniment.EditAulaForm;

/**
 * Accio per l'edici de les aules
 * @author ojaen
 *
 */
public class EditAulaAction extends DispatchAction {

	/**
	 * Prepara l'edici de l'aula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;

		//Recuperem el id de l'aula seleccionada
		Integer id = Integer.valueOf(request.getParameter("id"));
		AulaDAO aulaDAO = new AulaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Aula.CRITERI_AULA_ID_AULA, id);

		Aula[] aules = null;
		Estat[] arrEstats = null;

		try {
			//Cerquem l'aula seleccionada
			aules = aulaDAO.findAula(parametres);
			inicialitzaEstats(request, aulaEditForm, aules);

			inicialitzaCursos(request, aulaEditForm, aules);

			aulaEditForm.setAula(aules[0]);

			//Inicialitzem la relacio entre aules, assignatures i professors
			inicialitzaAssignaturaAulaProfessors(aulaEditForm, aules[0].getSetOfAssignaturaAulaProfessor());




		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAulaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("editAula");
	}



	/**
	 * Preparem el formulari per afegir una nova aula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;
		Aula[] aules = null;

		try {
			//Inicialitzem estats
			inicialitzaEstats(request, aulaEditForm, aules);

			//Inicialitzem cursos
			inicialitzaCursos(request, aulaEditForm, aules);

			//Inicialitzem la relacio entre aules, assignatures i professors
			inicialitzaAssignaturaAulaProfessors(aulaEditForm, null);

			//inicialitzem la resta de valors del form
			inicialitzaForm(aulaEditForm);

			//Inicialitzem la relacio entre aules, assignatures i professors
			inicialitzaAssignaturaAulaProfessors(aulaEditForm, null);



		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "aulaError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("editAula");

	}

	/**
	 * Gravem una aula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveAula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;

		AulaDAO aulaDAO = new AulaDAO();
		MestreDAO mestreDAO = new MestreDAO();

		try {
			//Cerquem l'Estat seleccionat i l'assignem al aula
			Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, aulaEditForm.getSelectedEstat());
			aulaEditForm.getAula().setIdEstat((Estat) o[0]);

			//Cerquem el tipus de titulaci seleccionat i l'assignem al aula
			HashMap parametres = new HashMap();
			parametres.put(Curs.CRITERI_CURS_ID_CURS, aulaEditForm.getSelectedCurs());
			Curs[] arrCursos = HelperUI.recuperaCursos(parametres);
			aulaEditForm.getAula().setIdCurs((Curs) arrCursos[0]);

			inicialitzaEstats(request, aulaEditForm, null);
			inicialitzaCursos(request, aulaEditForm, null);

			ActionErrors errors = validar(request, aulaEditForm);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward(CodisMestre.FAILURE));
			}


			aulaDAO.gravar(aulaEditForm.getAula());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAulaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaAula");
	}

	/**
	 * Esborrem una aula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteAula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;

		AulaDAO aulaDAO = new AulaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Aula.CRITERI_AULA_ID_AULA, aulaEditForm.getAula().getIdAula());


		Aula[] aules = null;

		try {
			aules = aulaDAO.findAula(parametres);

			inicialitzaEstats(request, aulaEditForm, null);
			inicialitzaCursos(request, aulaEditForm, null);

			ActionErrors errors = validarElimina(request, aules[0]);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward("failureCerca"));
			}

			aulaDAO.esborrar(aules[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "aulaError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("cercaAula");
	}


	/**
	 * Consulta d'una aula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaAula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		AulaDAO aulaDAO = new AulaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Aula.CRITERI_AULA_ID_AULA, id);

		Aula[] aules = null;

		try {
			aules = aulaDAO.findAula(parametres);
			aulaEditForm.setAula(aules[0]);

			inicialitzaEstats(request, aulaEditForm, aules);
			inicialitzaCursos(request, aulaEditForm, aules);

			//Inicialitzem la relacio entre aules, assignatures i professors
			inicialitzaAssignaturaAulaProfessors(aulaEditForm, aules[0].getSetOfAssignaturaAulaProfessor());



		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAulaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editAula");
	}

	/**
	 * Inicialitzar formulari
	 * @param aulaEditForm
	 */
	private void inicialitzaForm(EditAulaForm aulaEditForm) {
		aulaEditForm.setId(null);
		aulaEditForm.setDescripcio("");
		//aulaEditForm.setAssignaturaAulaProfessors(new HashSet<AssignaturaAulaProfessor>());
		aulaEditForm.setAula(new Aula());
		aulaEditForm.setSelectedEstat(new Integer(0));
		aulaEditForm.setAforament(null);
	}

	/**
	 * Validaci de la pantalla d'aules
	 * @param request
	 * @param aulaEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditAulaForm aulaEditForm) {

		boolean bValid = true;

		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Valida
		if (aulaEditForm.getAula().getCodiAula() == null ||
				aulaEditForm.getAula().getCodiAula().equals("")) {
			// crear ActionError i desar in the request
			errors.add("codi",new ActionError("error.codi.required"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (aulaEditForm.getAula().getDescripcio() == null ||
				aulaEditForm.getAula().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);
			bValid = false;

		}


		//Validem que no hi hagi un altre curs amb el mateix codi
		if (bValid) {
			AulaDAO aulaDAO = new AulaDAO();

			//Parametres
			HashMap parametres = new HashMap();
			if (aulaEditForm.getAula().getIdCurs() != null)
				parametres.put(Aula.CRITERI_AULA_ID_CURS,
						aulaEditForm.getAula().getIdCurs().getIdCurs());

			if (aulaEditForm.getAula().getCodiAula() != null)
				parametres.put(Aula.CRITERI_AULA_CODI,
						aulaEditForm.getAula().getCodiAula());

			if (aulaEditForm.getAula().getIdAula() != null) {
				parametres.put(Aula.CRITERI_AULA_ID_AULA,
						aulaEditForm.getAula().getIdAula());
				parametres.put(Aula.CRITERI_AULA_NOT_EQUAL_ID,
						Aula.CRITERI_AULA_NOT_EQUAL_ID);
			}

			parametres.put(Aula.CRITERI_AULA_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

			Aula[] aules = null;
			Estat[] arrEstats = null;
			try {

				//Cerquem l'aula seleccionat
				aules = aulaDAO.findAula(parametres);

				if (Helper.validarArray(aules)) {
					errors.add("editAulaError", new ActionError("error.aula.repetit"));
					saveErrors(request,errors);
				}
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "editAulaError"));
			}
		}


		return errors;
	}

	/**
	 * Validaci de la eliminaci d'un curs. No ha d'existir cap aula associada.
	 * @param request
	 * @param aula
	 * @return ActionsErrors
	 */
	private ActionErrors validarElimina(HttpServletRequest request, Aula aula) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		MatriculaDAO matriculaDAO = new MatriculaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Matricula.CRITERI_MATRICULA_ID_AULA,
				aula.getIdAula());
		Matricula[] matricules = null;
		try {

			//Cerquem la matrcula seleccionat
			matricules = matriculaDAO.findMatricula(parametres);

			if (Helper.validarArray(matricules)) {
				errors.add("aulaError", new ActionError("error.elimina.aula"));
				saveErrors(request,errors);
			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "aulaError"));
		}

		return errors;
	}


	/**
	 * Inicialitzar la relacio assignatura-aula-professor
	 * @param form
	 * @param assignaturaAulaProfessor
	 * @throws Throwable
	 */
	private void inicialitzaAssignaturaAulaProfessors(EditAulaForm form,
			Set<AssignaturaAulaProfessor> assignaturaAulaProfessor) throws Throwable {
		Set<CursAssignatura> cursAssignatura = recuperaCursAssignatura(form.getSelectedCurs());
		informaAssignaturaAulaProfessor(form, cursAssignatura, assignaturaAulaProfessor);
	}

	/**
	 * Recuperacio de les assignatures del curs associat a l'aula
	 * @param selectedCurs
	 * @return Set de CursAssignatura
	 * @throws Throwable
	 */
	private Set<CursAssignatura> recuperaCursAssignatura(Integer selectedCurs) throws Throwable {
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_CURS, selectedCurs);

		Curs[] cursos = HelperUI.recuperaCursos(parametres);
		if (cursos != null && cursos.length > 0)
			return cursos[0].getSetOfCursAssignatura();
		return new HashSet<CursAssignatura>();

	}

	/**
	 * Actualitzar la relacio assignatura-aula-professor
	 * @param form
	 * @param asigAulaProf
	 */
	private void actualitzaAssignaturaAulaProfessors(EditAulaForm form, AssignaturaAulaProfessor asigAulaProf) {
		form.addAssignaturaAulaProfessor(asigAulaProf);
	}

	/**
	 * Comparem totes les assignatures del curs respecte a les existentes a l'aula. Si no existeix
	 * l'assignatura la crearem (sense professor assignat)
	 * @param form
	 * @param cursAssignatura
	 * @param assignaturaAulaProfessor
	 * @return
	 */
	private void informaAssignaturaAulaProfessor(
			EditAulaForm form,
			Set<CursAssignatura> cursAssignatura, Set<AssignaturaAulaProfessor> assignaturaAulaProfessor) {
		if (assignaturaAulaProfessor == null)
			assignaturaAulaProfessor = new HashSet<AssignaturaAulaProfessor>();

		Iterator<CursAssignatura> iterCurs = cursAssignatura.iterator();

		while (iterCurs.hasNext()) {
			CursAssignatura ca = iterCurs.next();
			boolean bTrobat = false;
			for (Iterator<AssignaturaAulaProfessor> iterAula = assignaturaAulaProfessor.iterator();
			iterAula.hasNext();) {
				AssignaturaAulaProfessor aap = iterAula.next();

				if (aap.getIdAssignatura().getIdAssignatura().equals(ca.getIdAssignatura().getIdAssignatura())) {
					bTrobat = true;
					actualitzaAssignaturaAulaProfessors(form, aap);
					break;
				}

			}
			if (!bTrobat)
				crearNouRegistreAssignaturaAulaProfessor(form, ca);


		}

	}

	/**
	 * Crear una nova relacio assignatura-aula-professor
	 * @param form
	 * @param ca
	 */
	private void crearNouRegistreAssignaturaAulaProfessor(EditAulaForm form,
			CursAssignatura ca) {
		if (ca.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA) &&
			ca.getIdCurs().getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA) &&
			ca.getIdAssignatura().getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
			AssignaturaAulaProfessor aap = new AssignaturaAulaProfessor();
			aap.setIdAssignatura(ca.getIdAssignatura());
			aap.setIdAula(form.getAula());

			actualitzaAssignaturaAulaProfessors(form, aap);
		}

	}


	/**
	 * Crida al selector d'usuaris (professors)
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 * @throws Throwable
	 */
	public ActionForward selectorUsuari(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) throws Throwable {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;

		//Parametres
		String id = request.getParameter("codiAssignatura");
		marcaAssignaturaSeleccionada(aulaEditForm, id);

		inicialitzaEstats(request, aulaEditForm, null);

		inicialitzarTipusUsuari(request);

		inicialitzarTipusDocument(request);

		return mapping.findForward("selectorUsuari");


	}


	/**
	 * Inicialitzar tipus document
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzarTipusDocument(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus de documents
		TipusDocument[] arrTipusDocuments = HelperUI.recuperaTipusDocuments(new HashMap());

		ArrayList tipusDocuments = new ArrayList(Arrays.asList(arrTipusDocuments));
		//set the collection in the request
		request.setAttribute("tipusDocuments", tipusDocuments);
	}


	/**
	 * Inicialitzar tipus usuari
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzarTipusUsuari(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus d'usuaris
		TipusUsuari[] arrTipusUsuaris = HelperUI.recuperaTipusUsuaris(new HashMap());

		ArrayList tipusUsuaris = new ArrayList(Arrays.asList(arrTipusUsuaris));
		//set the collection in the request
		request.setAttribute("tipusUsuaris", tipusUsuaris);

		//set the collection in the request
		request.setAttribute("tipusUsuari", CodisMestre.CODI_USUARI_PROFESSOR);

	}

	/**
	 * Retornar del selector d'usuari
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward retornSelectorUsuari(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;
		cercaAssignaturaSeleccionada(aulaEditForm);

		try {
			inicialitzaEditAulaForm(request, aulaEditForm);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAulaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("editAula");
	}

	/**
	 * Inicialitzar formulari
	 * @param request
	 * @param aulaEditForm
	 * @throws Throwable
	 */
	private void inicialitzaEditAulaForm(HttpServletRequest request,
			EditAulaForm aulaEditForm) throws Throwable {

		Aula[] aules = new Aula[1];
		aules[0] = aulaEditForm.getAula();

		inicialitzaEstats(request, aulaEditForm, aules);
		inicialitzaCursos(request, aulaEditForm, aules);
	}

	/**
	 * Afegir professor en la relacio aula-assignatura-professor
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward afegeixSelectorUsuari(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditAulaForm aulaEditForm = (EditAulaForm) form;

		//Parametres
		Integer id = Integer.valueOf(request.getParameter("idProfessor"));

		//Afegim la nova assignatura-professor en el form
		try {

			inicialitzaEditAulaForm(request, aulaEditForm);
			AssignaturaAulaProfessor aap = cercaAssignaturaSeleccionada(aulaEditForm);
			UsuariProfessor professor = (UsuariProfessor) cercaUsuari(id);

			if (!validaProfessorAssignatura(aap, professor)) {
				ActionErrors errors = new ActionErrors();
				ActionError error = new ActionError("error.professor.assignatura");
				errors.add("editAulaError",error);
				saveErrors(request,errors);
				return (mapping.findForward(CodisMestre.FAILURE));
			}
			assignarProfessor(aulaEditForm, professor, aap);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editAulaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("editAula");
	}


	/**
	 * Valirdar professor-assignatura
	 * @param aap
	 * @param professor
	 * @return boolean
	 */
	private boolean validaProfessorAssignatura(AssignaturaAulaProfessor aap,
			UsuariProfessor professor) {
		for (Iterator<ProfessorAssignatura> iter=professor.getSetOfProfessorAssignatura().iterator();
		iter.hasNext();) {
			ProfessorAssignatura pa = iter.next();
			if (pa.getIdAssignatura().getIdAssignatura().equals(aap.getIdAssignatura().getIdAssignatura()) &&
					pa.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA))
				return true;
		}
		return false;
	}

	/**
	 * Cercar usuari (professor)
	 * @param id
	 * @return Usuari
	 * @throws Throwable
	 */
	private Usuari cercaUsuari(Integer id) throws Throwable {
		UsuariDAO usuariDAO = new UsuariDAO();
		HashMap parametres = new HashMap();
		parametres.put(Usuari.CRITERI_USUARI_ID_USUARI, id);
		Usuari[] usuaris = usuariDAO.findUsuari(parametres);
		if (usuaris!= null && usuaris.length > 0)
			return usuaris[0];
		return null;
	}

	/**
	 * Assignar professor a la relacio aula-assignatura-professor
	 * @param aulaEditForm
	 * @param professor
	 * @param aap
	 * @throws Throwable
	 */
	private void assignarProfessor(EditAulaForm aulaEditForm, Usuari professor, AssignaturaAulaProfessor aap) throws Throwable {

		if (professor != null && aap != null) {
			aap.setIdProfessor(professor);
			actualitzaAssignaturaAulaProfessors(aulaEditForm, aap);
		}

	}

	/**
	 * Cerca relacio assignatura-aula-professor
	 * @param aulaEditForm
	 * @return AssignaturaAulaProfessor
	 */
	private AssignaturaAulaProfessor cercaAssignaturaSeleccionada(EditAulaForm aulaEditForm) {
		for (Iterator<AssignaturaAulaProfessor> iterAula = aulaEditForm.getAssignaturaAulaProfessors().iterator();
		iterAula.hasNext();) {
			AssignaturaAulaProfessor aap = iterAula.next();
			if (aap.getIsSeleccionada()) {
				aap.setIsSeleccionada(new Boolean(false));
				return aap;
			}
		}
		return null;

	}

	/**
	 * Marca la relacio assignatura-aula-professor seleccionat
	 * @param aulaEditForm
	 * @param idAssignatura
	 * @return AssignaturaAulaProfessor
	 */
	private AssignaturaAulaProfessor marcaAssignaturaSeleccionada(EditAulaForm aulaEditForm, String idAssignatura) {
		for (Iterator<AssignaturaAulaProfessor> iterAula = aulaEditForm.getAssignaturaAulaProfessors().iterator();
		iterAula.hasNext();) {
			AssignaturaAulaProfessor aap = iterAula.next();
			if (aap.getIdAssignatura().getCodiAssignatura().equals(idAssignatura)) {
				aap.setIsSeleccionada(new Boolean(true));
				return aap;
			}
		}
		return null;

	}


	/**
	 * Inicialitzar cursos
	 * @param request
	 * @param aulaEditForm
	 * @param aules
	 * @throws Throwable
	 */
	private void inicialitzaCursos(HttpServletRequest request,
			EditAulaForm aulaEditForm, Aula[] aules)
	throws Throwable {
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
		Curs[] arrCursos = HelperUI.recuperaCursos(parametres);
		//define a dummy collection
		ArrayList cursos = new ArrayList(Arrays.asList(arrCursos));
		aulaEditForm.setCursos(cursos);

		if (aules != null && aules.length > 0 && aules[0].getIdCurs() != null)
			aulaEditForm.setSelectedCurs(aules[0].getIdCurs().getIdCurs());
		else
			aulaEditForm.setSelectedCurs(new Integer(0));

		//set the collection in the request
		request.setAttribute("cursos", cursos);
	}

	/*Inicialitzar Estats*/
	private void inicialitzaEstats(HttpServletRequest request,
			EditAulaForm aulaEditForm, Aula[] aules) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		aulaEditForm.setEstats(estats);

		if (aules != null && aules.length > 0 && aules[0].getIdEstat() != null)
			aulaEditForm.setSelectedEstat(aules[0].getIdEstat().getIdEstat());
		else if (aulaEditForm.getSelectedEstat() == null)
			aulaEditForm.setSelectedEstat(CodisMestre.CODI_ESTAT_ALTA);

		//set the collection in the request
		request.setAttribute("estats", estats);
	}



}
