package com.uoc.tfc.UI.struts.action.manteniment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.Assignatura;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.manteniment.CursAssignatura;
import com.uoc.tfc.Base.manteniment.TipusTitulacio;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.LN.dao.manteniment.AulaDAO;
import com.uoc.tfc.LN.dao.manteniment.CursDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.manteniment.EditCursForm;

/**
 * Accio per l'edici dels cursos
 * @author ojaen
 *
 */
public class EditCursAction extends DispatchAction {

	/**
	 * Preparar l'edici del curs
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		//Recuperem el id del curs seleccionat
		Integer id = Integer.valueOf(request.getParameter("id"));
		CursDAO cursDAO = new CursDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_CURS, id);
		parametres.put(Curs.CRITERI_CURS_INIT, Curs.CRITERI_CURS_INIT);

		Curs[] cursos = null;
		Estat[] arrEstats = null;

		try {
			//Cerquem el curs seleccionat
			cursos = cursDAO.findCurs(parametres);

			inicialitzaEstats(request, cursEditForm, cursos);
			inicialitzaTipusTitulacions(request, cursEditForm, cursos);
			inicialitzaCursosPrevis(request, cursEditForm, cursos);

			cursEditForm.setCurs(cursos[0]);

			//Inicialitzem la relaci entre cursos i assignatures
			inicialitzaCursAssignatures(cursEditForm, cursos[0].getSetOfCursAssignatura());



		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("editCurs");
	}

	/**
	 * Preparem el formulari per afegir un nou curs
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;
		Curs[] cursos = null;
		try {

			//Inicialitzem estats
			inicialitzaEstats(request, cursEditForm, cursos);

			//Inicialitzem els tipus de titulacions
			inicialitzaTipusTitulacions(request, cursEditForm, cursos);

			//Inicialitzem els cursos previs
			inicialitzaCursosPrevis(request, cursEditForm, cursos);

			//inicialitzem la resta de valors del form
			inicialitzaForm(cursEditForm);


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "cursError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("editCurs");

	}

	/**
	 * Inicialitzar formulari
	 * @param cursEditForm
	 */
	private void inicialitzaForm(EditCursForm cursEditForm) {
		cursEditForm.setId(null);
		cursEditForm.setCodiCurs("");
		cursEditForm.setDescripcio("");
		cursEditForm.setCursAssignatures(new HashSet<CursAssignatura>());
		cursEditForm.setCurs(new Curs());
		cursEditForm.setSelectedEstat(new Integer(0));
		cursEditForm.setSelectedTipusTitulacio(new Integer(0));
		cursEditForm.setSelectedCursPrevi(new Integer(0));

	}

	/**
	 * Gravem un curs
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveCurs(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		CursDAO cursDAO = new CursDAO();
		MestreDAO mestreDAO = new MestreDAO();

		try {
			//Cerquem l'Estat seleccionat i l'assignem al curs
			Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, cursEditForm.getSelectedEstat());
			cursEditForm.getCurs().setIdEstat((Estat) o[0]);

			//Cerquem el tipus de titulaci seleccionat i l'assignem al curs
			HashMap parametres = new HashMap();
			parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO, cursEditForm.getSelectedTipusTitulacio());
			TipusTitulacio[] arrTipusTitulacions = HelperUI.recuperaTipusTitulacions(parametres);
			cursEditForm.getCurs().setIdTipusTitulacio((TipusTitulacio) arrTipusTitulacions[0]);

			//Cerquem el curs previ seleccionat i l'assignem al curs
			if (cursEditForm.getSelectedCursPrevi() != null &&
					cursEditForm.getSelectedCursPrevi().intValue() > 0) {
				parametres = new HashMap();
				parametres.put(Curs.CRITERI_CURS_ID_CURS, cursEditForm.getSelectedCursPrevi());
				Curs[] arrCursos = recuperaCursos(parametres, false);
				cursEditForm.getCurs().setIdCursPrevi((Curs) arrCursos[0]);
			}
			else
				cursEditForm.getCurs().setIdCursPrevi(null);


			inicialitzaEstats(request, cursEditForm, null);
			inicialitzaTipusTitulacions(request, cursEditForm, null);
			inicialitzaCursosPrevis(request, cursEditForm, null);


			ActionErrors errors = validar(request, cursEditForm);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward(CodisMestre.FAILURE));
			}


			cursDAO.gravar(cursEditForm.getCurs());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaCurs");
	}

	/**
	 * Esborrem un curs
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteCurs(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		CursDAO cursDAO = new CursDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_CURS, cursEditForm.getCurs().getIdCurs());
		parametres.put(Curs.CRITERI_CURS_INIT, Curs.CRITERI_CURS_INIT);


		Curs[] cursos = null;

		try {
			cursos = cursDAO.findCurs(parametres);

			inicialitzaEstats(request, cursEditForm, null);
			inicialitzaTipusTitulacions(request, cursEditForm, null);

			ActionErrors errors = validarElimina(request, cursos[0]);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward("failureCerca"));
			}


			cursDAO.esborrar(cursos[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "cursError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("cercaCurs");
	}


	/**
	 * Consulta d'un curs
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaCurs(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		CursDAO cursDAO = new CursDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_CURS, id);
		parametres.put(Curs.CRITERI_CURS_INIT, Curs.CRITERI_CURS_INIT);

		Curs[] cursos = null;

		try {
			cursos = cursDAO.findCurs(parametres);
			cursEditForm.setCurs(cursos[0]);

			inicialitzaEstats(request, cursEditForm, cursos);
			inicialitzaTipusTitulacions(request, cursEditForm, cursos);
			inicialitzaCursosPrevis(request, cursEditForm, cursos);


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editCurs");
	}

	/**
	 * Validaci de la pantalla de cursos
	 * @param request
	 * @param cursEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditCursForm cursEditForm) {

		boolean bValid = true;

		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Valida
		if (cursEditForm.getCurs().getCodiCurs() == null ||
				cursEditForm.getCurs().getCodiCurs().equals("")) {
			// crear ActionError i desar in the request
			errors.add("codi",new ActionError("error.codi.required"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (cursEditForm.getCurs().getDescripcio() == null ||
				cursEditForm.getCurs().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);
			bValid = false;

		}


		//Validem que no hi hagi un altre curs amb el mateix codi
		if (bValid) {
			CursDAO cursDAO = new CursDAO();

			//Parametres
			HashMap parametres = new HashMap();

			if (cursEditForm.getCurs().getIdTipusTitulacio() != null)
				parametres.put(Curs.CRITERI_CURS_ID_TIPUS_TITULACIO,
						cursEditForm.getCurs().getIdTipusTitulacio().getIdTipusTitulacio());

			if (cursEditForm.getCurs().getCodiCurs() != null)
				parametres.put(Curs.CRITERI_CURS_CODI,
						cursEditForm.getCurs().getCodiCurs());

			if (cursEditForm.getCurs().getIdCurs() != null) {
				parametres.put(Curs.CRITERI_CURS_ID_CURS,
						cursEditForm.getCurs().getIdCurs());
				parametres.put(Curs.CRITERI_CURS_NOT_EQUAL_ID,
						Curs.CRITERI_CURS_NOT_EQUAL_ID);
			}

			parametres.put(Curs.CRITERI_CURS_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

			Curs[] cursos = null;
			Estat[] arrEstats = null;
			try {

				//Cerquem l'curs seleccionat
				cursos = cursDAO.findCurs(parametres);

				if (Helper.validarArray(cursos)) {
					errors.add("editCursError", new ActionError("error.curs.repetit"));
					saveErrors(request,errors);
				}
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			}
		}


		return errors;
	}

	/**
	 * Validaci de la eliminaci d'un curs. No ha d'existir cap aula associada.
	 * @param request
	 * @param curs
	 * @return ActionsErrors
	 */
	private ActionErrors validarElimina(HttpServletRequest request, Curs curs) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		AulaDAO aulaDAO = new AulaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Aula.CRITERI_AULA_ID_CURS,
				curs.getIdCurs());
		Aula[] aules = null;
		try {

			//Cerquem el aula seleccionat
			aules = aulaDAO.findAula(parametres);

			if (Helper.validarArray(aules)) {
				errors.add("cursError", new ActionError("error.elimina.curs"));
				saveErrors(request,errors);
			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "cursError"));
		}

		return errors;
	}



	/**
	 * Crida al selector d'assignatures
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward selectorAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		//Recuperem els estats
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		//set the collection in the request
		request.setAttribute("estats", estats);

		return mapping.findForward("selectorAssignatura");


	}

	/**
	 * Retorn del selector d'assignatures
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward retornSelectorAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		try {
			inicialitzaEditCursForm(request, cursEditForm);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("editCurs");
	}

	/**
	 * Afegir nova assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward afegeixAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		//Parametres
		Integer id = Integer.valueOf(request.getParameter("idAssignatura"));

		//Afegim la nova assignatura en el form
		try {
			inicialitzaEditCursForm(request, cursEditForm);
			if (validaAfegeixAssignatura(cursEditForm, id)) {
				afegirNovaAssignatura(mapping, request, cursEditForm, id);
			}
			else {

				ActionErrors errors = new ActionErrors();
				ActionError error = new ActionError("error.selectorAssignatura.jaExisteix");
				errors.add("editCursError",error);
				saveErrors(request,errors);
				return (mapping.findForward(CodisMestre.FAILURE));

			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}
		return mapping.findForward("editCurs");
	}

	/**
	 * Inicialitzar formulari
	 * @param request
	 * @param cursEditForm
	 * @throws Throwable
	 */
	private void inicialitzaEditCursForm(HttpServletRequest request,
			EditCursForm cursEditForm) throws Throwable {
		Curs[] cursos = new Curs[1];
		cursos[0] = cursEditForm.getCurs();

		inicialitzaEstats(request, cursEditForm, cursos);
		inicialitzaTipusTitulacions(request, cursEditForm, cursos);
		inicialitzaCursosPrevis(request, cursEditForm, cursos);
	}


	/**
	 * Validem si l'assignatura que es desitja afegir al curs ja existeix
	 * @param cursEditForm
	 * @param id
	 * @return
	 */
	private boolean validaAfegeixAssignatura(EditCursForm cursEditForm,
			Integer id) {
		Set<CursAssignatura> cursAssignatures = cursEditForm.getCursAssignatures();
		Iterator<CursAssignatura> iter = cursAssignatures.iterator();
		while (iter.hasNext()) {
			CursAssignatura cursAssignatura = iter.next();
			if (cursAssignatura.getIdAssignatura().getIdAssignatura().equals(id)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Esborrar assignatura (donar de baixa)
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward esborraAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditCursForm cursEditForm = (EditCursForm) form;

		//Parametres
		String codiAssignatura = request.getParameter("codiAssignatura");

		//Validem l'assignatura per si alguna persona te alguna qualificacio en aquesta assignatura
		try {
			inicialitzaEditCursForm(request, cursEditForm);
			actualitzaEstatAssignatura(cursEditForm, codiAssignatura);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editCursError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("editCurs");
	}

	/**
	 * Actualitzem l'assignatura selecciona el seu estat a 'Baixa'/'Alta'
	 * @param cursEditForm
	 * @param id
	 */
	private void actualitzaEstatAssignatura(EditCursForm cursEditForm, String codiAssignatura) {
		Set<CursAssignatura> cursAssignatures = cursEditForm.getCursAssignatures();
		Iterator<CursAssignatura> iter = cursAssignatures.iterator();
		while (iter.hasNext()) {
			CursAssignatura cursAssignatura = iter.next();
			if (cursAssignatura.getCodiAssignatura().equals(codiAssignatura)) {

				MestreDAO mestreDAO = new MestreDAO();
				Object[] o = null;
				if (cursAssignatura.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, CodisMestre.CODI_ESTAT_BAIXA);
				}
				else {
					o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
				}
				cursAssignatura.setIdEstat((Estat) o[0]);
			}

		}

	}

	/**
	 * Afegir nova assignatura
	 * @param mapping
	 * @param request
	 * @param cursEditForm
	 * @param id
	 * @throws Throwable
	 */
	private void afegirNovaAssignatura(ActionMapping mapping,
			HttpServletRequest request, EditCursForm cursEditForm, Integer id) throws Throwable {
		HashMap parametres = new HashMap();
		parametres.put(Assignatura.CRITERI_ASSIGNATURA_ID_ASSIGNATURA, id);
		Assignatura[] arrAssignatures = HelperUI.recuperaAssignatura(parametres);
		if (arrAssignatures != null) {

			CursAssignatura cursAssignatura = new CursAssignatura();
			cursAssignatura.setIdAssignatura(arrAssignatures[0]);
			cursAssignatura.setIdCurs(cursEditForm.getCurs());
			//Cerquem l'Estat seleccionat i l'assignem al curs
			MestreDAO mestreDAO = new MestreDAO();
			Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);
			cursAssignatura.setIdEstat((Estat) o[0]);
			afegeixCursAssignatura(cursEditForm, cursAssignatura);
		}

	}

	/**
	 * Inicialitzar relaci curs-assignatures
	 * @param form
	 * @param cursAssignatures
	 */
	private void inicialitzaCursAssignatures(EditCursForm form, Set<CursAssignatura> cursAssignatures) {
		form.setCursAssignatures(cursAssignatures);
	}

	/**
	 * Afegir relaci curs-assignatura
	 * @param form
	 * @param cursAssignatura
	 */
	private void afegeixCursAssignatura(EditCursForm form, CursAssignatura cursAssignatura) {
		form.addCursAssignatura(cursAssignatura);
	}

	/**
	 * Inicialitzar tipus de titulacions
	 * @param request
	 * @param cursEditForm
	 * @param curs
	 * @throws Throwable
	 */
	private void inicialitzaTipusTitulacions(HttpServletRequest request,
			EditCursForm cursEditForm, Curs[] curs)
	throws Throwable {
		TipusTitulacio[] arrTipusTitulacio;
		arrTipusTitulacio = HelperUI.recuperaTipusTitulacions(new HashMap());
		//define a dummy collection
		ArrayList tipusTitulacions = new ArrayList(Arrays.asList(arrTipusTitulacio));

		if (curs != null && curs.length > 0 && curs[0].getIdTipusTitulacio() != null)
			cursEditForm.setSelectedTipusTitulacio(curs[0].getIdTipusTitulacio().getIdTipusTitulacio());
		else
			cursEditForm.setSelectedTipusTitulacio(new Integer(0));

		//set the collection in the request
		request.setAttribute("tipusTitulacions", tipusTitulacions);
	}

	/**
	 * Inicialitzar estats
	 * @param request
	 * @param cursEditForm
	 * @param cursos
	 */
	private void inicialitzaEstats(HttpServletRequest request,
			EditCursForm cursEditForm, Curs[] cursos) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();
		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		cursEditForm.setEstats(estats);
		if (cursos != null && cursos.length > 0 && cursos[0].getIdEstat() != null)
			cursEditForm.setSelectedEstat(cursos[0].getIdEstat().getIdEstat());
		else if (cursEditForm.getSelectedEstat() == null)
			cursEditForm.setSelectedEstat(CodisMestre.CODI_ESTAT_ALTA);
		//set the collection in the request
		request.setAttribute("estats", estats);
	}

	/**
	 * Recuperem els cursos de la base de dades
	 * @return Curs[]: Array de cursos
	 */
	private Curs[] recuperaCursos(HashMap p_parametres, boolean creaCapcurs) throws Throwable{

		CursDAO cursDAO = new CursDAO();
		Curs[] o = (Curs[]) cursDAO.findCurs(p_parametres);

		Curs[] cursos = new Curs[o.length+1];

		if (creaCapcurs) {
			//Creem un curs fictici per poder seleccionar 'Cap Curs Previ'
			cursos[0] = new Curs();
			cursos[0].setCodiCurs("0");
			cursos[0].setDescripcio("Cap Curs Previ");
			cursos[0].setIdCurs(new Integer(0));

			for (int i = 0; i < o.length; i++)
				cursos[i+1] = o[i];
			return cursos;
		}
		else
			return o;

	}

	/**
	 * Inicialitzar cursos previs
	 * @param request
	 * @param cursEditForm
	 * @param cursos
	 * @throws Throwable
	 */
	private void inicialitzaCursosPrevis(HttpServletRequest request,
			EditCursForm cursEditForm, Curs[] cursos)
	throws Throwable {
		Curs[] arrCurs;
		arrCurs = recuperaCursos(new HashMap(), true);
		//define a dummy collection
		ArrayList cursosPrevis = new ArrayList(Arrays.asList(arrCurs));
		cursEditForm.setCursosPrevis(cursosPrevis);
		if (cursos != null && cursos.length > 0 && cursos[0].getIdCursPrevi() != null)
			cursEditForm.setSelectedCursPrevi(cursos[0].getIdCursPrevi().getIdCurs());
		else
			cursEditForm.setSelectedCursPrevi(new Integer(0));

		//set the collection in the request
		request.setAttribute("cursosPrevis", cursosPrevis);
	}





}
