package com.uoc.tfc.UI.struts.action.manteniment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.TipusQualificacio;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.qualificacio.Qualificacio;
import com.uoc.tfc.LN.dao.manteniment.TipusQualificacioDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.dao.qualificacio.QualificacioDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.manteniment.EditTipusQualificacioForm;

/**
 * Manteniment dels tipus de qualificacions
 * @author ojaen
 *
 */
public class EditTipusQualificacioAction extends DispatchAction {

	/**
	 * Preparar l'edici del tipus de qualificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusQualificacioForm tipusQualificacioEditForm = (EditTipusQualificacioForm) form;

		//Recuperem el id del tipus de qualificaci seleccionat
		Integer id = Integer.valueOf(request.getParameter("id"));
		TipusQualificacioDAO tipusQualificacioDAO = new TipusQualificacioDAO();

		//Cerquem el tipus de qualificacio
		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO, id);

		TipusQualificacio[] tipusQualificacions = null;
		Estat[] arrEstats = null;
		try {
			//Cercar el tipus de qualificacio seleccionat
			tipusQualificacions = tipusQualificacioDAO.findTipusQualificacio(parametres);

			//Inicialitzem els estats
			inicialitzarEstats(request, tipusQualificacioEditForm,
					tipusQualificacions);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editTipusQualificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		tipusQualificacioEditForm.setTipusQualificacio(tipusQualificacions[0]);

		return mapping.findForward("editTipusQualificacio");
	}

	/**
	 * Preparem el formulari per afegir un nou tipus de qualificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		//Recuperem els estats
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));

		//set the collection in the request
		request.setAttribute("estats", estats);

		return mapping.findForward("editTipusQualificacio");

	}

	/**
	 * Gravem un tipus de qualificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveTipusQualificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusQualificacioForm tipusQualificacioEditForm = (EditTipusQualificacioForm) form;


		TipusQualificacioDAO tipusQualificacioDAO = new TipusQualificacioDAO();
		MestreDAO mestreDAO = new MestreDAO();

		//Cerquem l'Estat seleccionat i l'assignem al tipus de qualificaci
		Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, tipusQualificacioEditForm.getSelectedEstat());
		tipusQualificacioEditForm.getTipusQualificacio().setIdEstat((Estat) o[0]);

		//Inicialitzem els estats
		inicialitzarEstats(request, tipusQualificacioEditForm, null);

		//Validem formulari
		ActionErrors errors = validar(request, tipusQualificacioEditForm);

		if (errors != null && errors.size() > 0) {
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		try {
			tipusQualificacioDAO.gravar(tipusQualificacioEditForm.getTipusQualificacio());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editTipusQualificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaTipusQualificacio");
	}

	/**
	 * Esborra tipus de qualificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteTipusQualificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusQualificacioForm tipusQualificacioEditForm = (EditTipusQualificacioForm) form;

		TipusQualificacioDAO tipusQualificacioDAO = new TipusQualificacioDAO();

		//Cerquem al tipus de qualificaci a eliminar
		HashMap parametres = new HashMap();
		parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO, tipusQualificacioEditForm.getTipusQualificacio().getIdTipusQualificacio());

		TipusQualificacio[] tipusQualificacions = null;

		try {
			tipusQualificacions = tipusQualificacioDAO.findTipusQualificacio(parametres);

			inicialitzarEstats(request, tipusQualificacioEditForm, null);

			ActionErrors errors = validarElimina(request, tipusQualificacions[0]);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward("failureCerca"));
			}


			tipusQualificacioDAO.esborrar(tipusQualificacions[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "tipusQualificacioError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("cercaTipusQualificacio");
	}


	/**
	 * Consultem tipus de qualificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaTipusQualificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusQualificacioForm tipusQualificacioEditForm = (EditTipusQualificacioForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		TipusQualificacioDAO tipusQualificacioDAO = new TipusQualificacioDAO();

		//Cerquem al tipus de qualificaci a eliminar
		HashMap parametres = new HashMap();
		parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO, id);

		TipusQualificacio[] tipusQualificacions = null;

		try {

			tipusQualificacions = tipusQualificacioDAO.findTipusQualificacio(parametres);
			tipusQualificacioEditForm.setTipusQualificacio(tipusQualificacions[0]);

			Estat[] arrEstats = HelperUI.recuperaEstats();
			//define a dummy collection
			ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
			tipusQualificacioEditForm.setEstats(estats);
			tipusQualificacioEditForm.setSelectedEstat(tipusQualificacions[0].getIdEstat().getIdEstat());

			//set the collection in the request
			request.setAttribute("estats", estats);


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editTipusQualificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editTipusQualificacio");
	}

	/**
	 * Validaci de la pantalla de tipus de qualificacions
	 * @param request
	 * @param tipusQualificacioEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditTipusQualificacioForm tipusQualificacioEditForm) {
		boolean bValid = true;
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Valida
		if (tipusQualificacioEditForm.getTipusQualificacio().getCodiTipusQualificacio() == null ||
				tipusQualificacioEditForm.getTipusQualificacio().getCodiTipusQualificacio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("codi",new ActionError("error.codi.required"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (tipusQualificacioEditForm.getTipusQualificacio().getDescripcio() == null ||
				tipusQualificacioEditForm.getTipusQualificacio().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);
			bValid = false;

		}


		//Validem que no hi hagi un altre tipusQualificacio amb el mateix codi
		if (bValid) {
			TipusQualificacioDAO tipusQualificacioDAO = new TipusQualificacioDAO();

			//Parametres
			HashMap parametres = new HashMap();
			if (tipusQualificacioEditForm.getTipusQualificacio().getCodiTipusQualificacio() != null)
				parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_CODI,
						tipusQualificacioEditForm.getTipusQualificacio().getCodiTipusQualificacio());

			if (tipusQualificacioEditForm.getTipusQualificacio().getIdTipusQualificacio() != null) {
				parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_ID_TIPUSQUALIFICACIO,
						tipusQualificacioEditForm.getTipusQualificacio().getIdTipusQualificacio());
				parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_NOT_EQUAL_ID,
						TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_NOT_EQUAL_ID);
			}

			parametres.put(TipusQualificacio.CRITERI_TIPUSQUALIFICACIO_CODI_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

			TipusQualificacio[] tipusQualificacions = null;
			Estat[] arrEstats = null;
			try {

				//Cerquem l'tipusQualificacio seleccionat
				tipusQualificacions = tipusQualificacioDAO.findTipusQualificacio(parametres);

				if (Helper.validarArray(tipusQualificacions)) {
					errors.add("editTipusQualificacioError", new ActionError("error.tipusqualificacio.repetit"));
					saveErrors(request,errors);
				}
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "editTipusQualificacioError"));
			}
		}


		return errors;
	}

	/**
	 * Validaci de la eliminaci d'un tipusQualificacio. No ha d'existir cap curs associat.
	 * @param request
	 * @param tipusQualificacio
	 * @return ActionsErrors
	 */
	private ActionErrors validarElimina(HttpServletRequest request, TipusQualificacio tipusQualificacio) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		QualificacioDAO qualificacioDAO = new QualificacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Qualificacio.CRITERI_QUALIFICACIO_ID_TIPUS_QUALIFICACIO,
				tipusQualificacio.getIdTipusQualificacio());
		Qualificacio[] qualificacions = null;
		try {

			//Cerquem el qualificacio seleccionat
			qualificacions = qualificacioDAO.findQualificacio(parametres);

			if (Helper.validarArray(qualificacions)) {
				errors.add("tipusQualificacioError", new ActionError("error.elimina.tipusqualificacio"));
				saveErrors(request,errors);
			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "tipusQualificacioError"));
		}

		return errors;
	}



	/**
	 * Inicialitzacio dels estats
	 * @param request
	 * @param anyAcademicEditForm
	 * @param anysAcademics
	 */
	private void inicialitzarEstats(HttpServletRequest request,
			EditTipusQualificacioForm tipusQualificacioEditForm,
			TipusQualificacio[] tipusQualificacions) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		tipusQualificacioEditForm.setEstats(estats);

		if (tipusQualificacions != null)
			tipusQualificacioEditForm.setSelectedEstat(tipusQualificacions[0].getIdEstat().getIdEstat());

		//set the collection in the request
		request.setAttribute("estats", estats);
	}

}
