package com.uoc.tfc.UI.struts.action.manteniment;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.manteniment.TipusTitulacio;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.LN.dao.manteniment.CursDAO;
import com.uoc.tfc.LN.dao.manteniment.TipusTitulacioDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.manteniment.EditTipusTitulacioForm;

/**
 * Manteniment dels tipus de titulacions
 * @author ojaen
 *
 */
public class EditTipusTitulacioAction extends DispatchAction {

	/**
	 * Preparar l'edici del tipus de titulaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusTitulacioForm tipusTitulacioEditForm = (EditTipusTitulacioForm) form;

		//Recuperem el id del tipus de titulaci seleccionat
		Integer id = Integer.valueOf(request.getParameter("id"));
		TipusTitulacioDAO tipusTitulacioDAO = new TipusTitulacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO, id);

		TipusTitulacio[] tipusTitulacions = null;
		Estat[] arrEstats = null;

		try {
			//Recuperem el tipus de titulaci seleccionat
			tipusTitulacions = tipusTitulacioDAO.findTipusTitulacio(parametres);
			tipusTitulacioEditForm.setTipusTitulacio(tipusTitulacions[0]);

			//Inicialitzar Estats
			inicialitzarEstats(request, tipusTitulacioEditForm,
					tipusTitulacions);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editTipusTitulacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("editTipusTitulacio");
	}

	/**
	 * Preparem el formulari per afegir un nou tipus de titulaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		//Recuperem els estats
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));

		//set the collection in the request
		request.setAttribute("estats", estats);

		return mapping.findForward("editTipusTitulacio");

	}

	/**
	 * Gravem un tipus de titulaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveTipusTitulacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusTitulacioForm tipusTitulacioEditForm = (EditTipusTitulacioForm) form;

		TipusTitulacioDAO tipusTitulacioDAO = new TipusTitulacioDAO();
		MestreDAO mestreDAO = new MestreDAO();

		//Cerquem l'Estat seleccionat i l'assignem al tipus de qualificaci
		Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, tipusTitulacioEditForm.getSelectedEstat());
		tipusTitulacioEditForm.getTipusTitulacio().setIdEstat((Estat) o[0]);

		//Inicialitzem els estats
		inicialitzarEstats(request, tipusTitulacioEditForm, null);

		//Validem formulari
		ActionErrors errors = validar(request, tipusTitulacioEditForm);

		if (errors != null && errors.size() > 0) {
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		try {
			tipusTitulacioDAO.gravar(tipusTitulacioEditForm.getTipusTitulacio());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editTipusTitulacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("cercaTipusTitulacio");
	}

	/**
	 * Esborrar tipus de titulaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteTipusTitulacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusTitulacioForm tipusTitulacioEditForm = (EditTipusTitulacioForm) form;

		TipusTitulacioDAO tipusTitulacioDAO = new TipusTitulacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO, tipusTitulacioEditForm.getTipusTitulacio().getIdTipusTitulacio());

		TipusTitulacio[] tipusTitulacions = null;

		try {
			tipusTitulacions =	tipusTitulacioDAO.findTipusTitulacio(parametres);

			inicialitzarEstats(request, tipusTitulacioEditForm, null);

			ActionErrors errors = validarElimina(request, tipusTitulacions[0]);

			if (errors != null && errors.size() > 0) {
				return (mapping.findForward("failureCerca"));
			}

			tipusTitulacioDAO.esborrar(tipusTitulacions[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "tipusTitulacioError"));
			return (mapping.findForward("failureCerca"));
		}

		return mapping.findForward("cercaTipusTitulacio");
	}


	/**
	 * Consultem tipus de titulaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaTipusTitulacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditTipusTitulacioForm tipusTitulacioEditForm = (EditTipusTitulacioForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		TipusTitulacioDAO tipusTitulacioDAO = new TipusTitulacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO, id);

		TipusTitulacio[] tipusTitulacions = null;

		try {
			tipusTitulacions = tipusTitulacioDAO.findTipusTitulacio(parametres);
			tipusTitulacioEditForm.setTipusTitulacio(tipusTitulacions[0]);

			Estat[] arrEstats = HelperUI.recuperaEstats();
			//define a dummy collection
			ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
			tipusTitulacioEditForm.setEstats(estats);
			tipusTitulacioEditForm.setSelectedEstat(tipusTitulacions[0].getIdEstat().getIdEstat());

			//set the collection in the request
			request.setAttribute("estats", estats);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editTipusTitulacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}
		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editTipusTitulacio");
	}

	/**
	 * Validaci de la pantalla de tipus de titulacions
	 * @param request
	 * @param tipusTitulacioEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditTipusTitulacioForm tipusTitulacioEditForm) {
		boolean bValid = true;
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Valida
		if (tipusTitulacioEditForm.getTipusTitulacio().getCodiTipusTitulacio() == null ||
				tipusTitulacioEditForm.getTipusTitulacio().getCodiTipusTitulacio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("codi",new ActionError("error.codi.required"));
			saveErrors(request,errors);
			bValid = false;
		}
		if (tipusTitulacioEditForm.getTipusTitulacio().getDescripcio() == null ||
				tipusTitulacioEditForm.getTipusTitulacio().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);
			bValid = false;

		}

		//Validem que no hi hagi un altre tipusTitulacio amb el mateix codi
		if (bValid) {
			TipusTitulacioDAO tipusTitulacioDAO = new TipusTitulacioDAO();

			//Parametres
			HashMap parametres = new HashMap();
			if (tipusTitulacioEditForm.getTipusTitulacio().getCodiTipusTitulacio() != null)
				parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_CODI,
						tipusTitulacioEditForm.getTipusTitulacio().getCodiTipusTitulacio());

			if (tipusTitulacioEditForm.getTipusTitulacio().getIdTipusTitulacio() != null) {
				parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_ID_TIPUSTITULACIO,
						tipusTitulacioEditForm.getTipusTitulacio().getIdTipusTitulacio());
				parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_NOT_EQUAL_ID,
						TipusTitulacio.CRITERI_TIPUSTITULACIO_NOT_EQUAL_ID);
			}

			parametres.put(TipusTitulacio.CRITERI_TIPUSTITULACIO_CODI_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

			TipusTitulacio[] tipusTitulacions = null;
			Estat[] arrEstats = null;
			try {

				//Cerquem l'tipusTitulacio seleccionat
				tipusTitulacions = tipusTitulacioDAO.findTipusTitulacio(parametres);

				if (Helper.validarArray(tipusTitulacions)) {
					errors.add("editTipusTitulacioError", new ActionError("error.tipustitulacio.repetit"));
					saveErrors(request,errors);
				}
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "editTipusTitulacioError"));
			}
		}


		return errors;
	}


	/**
	 * Validaci de la eliminaci d'un tipusTitulacio. No ha d'existir cap curs associat.
	 * @param request
	 * @param tipusTitulacio
	 * @return ActionsErrors
	 */
	private ActionErrors validarElimina(HttpServletRequest request, TipusTitulacio tipusTitulacio) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();


		CursDAO cursDAO = new CursDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_TIPUS_TITULACIO,
				tipusTitulacio.getIdTipusTitulacio());
		Curs[] cursos = null;
		try {

			//Cerquem el curs seleccionat
			cursos = cursDAO.findCurs(parametres);

			if (Helper.validarArray(cursos)) {
				errors.add("tipusTitulacioError", new ActionError("error.elimina.tipustitulacio"));
				saveErrors(request,errors);
			}
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "tipusTitulacioError"));
		}

		return errors;
	}


	/**
	 * Inicialitzacio dels estats
	 * @param request
	 * @param anyAcademicEditForm
	 * @param anysAcademics
	 */
	private void inicialitzarEstats(HttpServletRequest request,
			EditTipusTitulacioForm tipusTitulacioEditForm,
			TipusTitulacio[] tipusTitulacions) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		tipusTitulacioEditForm.setEstats(estats);

		if (tipusTitulacions != null)
			tipusTitulacioEditForm.setSelectedEstat(tipusTitulacions[0].getIdEstat().getIdEstat());

		//set the collection in the request
		request.setAttribute("estats", estats);
	}


}
