package com.uoc.tfc.UI.struts.action.matricula;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.common.Internacionalitzacio;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.mestres.TipusDocument;
import com.uoc.tfc.LN.dao.manteniment.AnyAcademicDAO;
import com.uoc.tfc.LN.dao.matricula.MatriculaDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.matricula.CercaMatriculaForm;

/**
 * Accions per la cerca de les matrcules
 * @author ojaen
 *
 */
public class CercaMatriculaAction extends Action {

	/**
	 * Execuci de l'acci de cerca
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward execute(
			ActionMapping mapping,
			ActionForm form,
			HttpServletRequest request,
			HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));


		CercaMatriculaForm cercaMatriculaForm = (CercaMatriculaForm) form;
		MatriculaDAO matriculaDAO = new MatriculaDAO();
		Integer estatDefecte = CodisMestre.CODI_ESTAT_ALTA;
		Integer anyAcademicDefecte = new Integer(0);
		Integer tipusDocumentDefecte = CodisMestre.CODI_TIPUS_DOCUMENT_NIF;

		String accio = request.getParameter(CodisMestre.DO);

		if (accio == null || (!accio.equals(CodisMestre.PREPARE_CERCA) && !accio.equals(CodisMestre.RETORN_CERCA))) {
			HashMap parametres = new HashMap();

			if (Helper.validarCadena(cercaMatriculaForm.getCodiUsuari()))
				parametres.put(Matricula.CRITERI_MATRICULA_ID_USUARI, Integer.parseInt(cercaMatriculaForm.getCodiUsuari()));
			parametres.put(Matricula.CRITERI_MATRICULA_NOM_USUARI,
					cercaMatriculaForm.getNom() != null ? cercaMatriculaForm.getNom().toUpperCase() : "");
			parametres.put(Matricula.CRITERI_MATRICULA_COGNOM1_USUARI,
					cercaMatriculaForm.getCognom1() != null ? cercaMatriculaForm.getCognom1().toUpperCase() : "");
			parametres.put(Matricula.CRITERI_MATRICULA_COGNOM2_USUARI,
					cercaMatriculaForm.getCognom2() != null ? cercaMatriculaForm.getCognom2().toUpperCase() : "");
			parametres.put(Matricula.CRITERI_MATRICULA_NUM_DOCUMENT_USUARI, cercaMatriculaForm.getNumDocument());
			parametres.put(Matricula.CRITERI_MATRICULA_ID_TIPUS_DOCUMENT_USUARI, cercaMatriculaForm.getSelectedTipusDocument());
			parametres.put(Matricula.CRITERI_MATRICULA_ID_ESTAT_USUARI, cercaMatriculaForm.getSelectedEstat());
			parametres.put(Matricula.CRITERI_MATRICULA_ID_ANY_ACADEMIC_USUARI, cercaMatriculaForm.getSelectedAnyAcademic());

			estatDefecte = cercaMatriculaForm.getSelectedEstat();
			if (cercaMatriculaForm.getSelectedAnyAcademic() != null)
				anyAcademicDefecte = cercaMatriculaForm.getSelectedAnyAcademic();

			if (cercaMatriculaForm.getSelectedTipusDocument() != null)
				tipusDocumentDefecte = cercaMatriculaForm.getSelectedTipusDocument();


			try {
				cercaMatriculaForm.setMatricules(matriculaDAO.findMatricula(parametres));
			} catch (Throwable e) {
				saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
				return (mapping.findForward(CodisMestre.FAILURE));
			}
		}
		//No fem res
		else if (accio != null && accio.equals(CodisMestre.RETORN_CERCA)) {

		}
		//Reiniciem el formulari
		else {
			resetFormulari(cercaMatriculaForm);
		}


		try {

			inicialitzaAnysAcademics(request, cercaMatriculaForm,
					anyAcademicDefecte);

			inicialitzaTipusDocuments(request, cercaMatriculaForm,
					tipusDocumentDefecte);

			inicialitzaEstats(request, cercaMatriculaForm, estatDefecte);

			avisMatriculaOberta(request);


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("cercaMatricula");
	}

	/**
	 * Inicialitzar estats
	 * @param request
	 * @param cercaMatriculaForm
	 * @param estatDefecte
	 */
	private void inicialitzaEstats(HttpServletRequest request,
			CercaMatriculaForm cercaMatriculaForm, Integer estatDefecte) {
		//Carrega opcions d'estat
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		cercaMatriculaForm.setEstats(estats);

		if (cercaMatriculaForm.getSelectedEstat() == null)
			cercaMatriculaForm.setSelectedEstat(estatDefecte);

		//set the collection in the request
		request.setAttribute("estats", estats);
	}

	/**
	 * Inicialitzar tipus de documents
	 * @param request
	 * @param cercaMatriculaForm
	 * @param tipusDocumentDefecte
	 * @throws Throwable
	 */
	private void inicialitzaTipusDocuments(HttpServletRequest request,
			CercaMatriculaForm cercaMatriculaForm, Integer tipusDocumentDefecte)
	throws Throwable {
		//Carrega opcions de tipus de documents
		TipusDocument[] arrTipusDocuments = HelperUI.recuperaTipusDocuments(new HashMap());

		ArrayList tipusDocuments = new ArrayList(Arrays.asList(arrTipusDocuments));
		cercaMatriculaForm.setAnyAcademic(tipusDocuments);

		if (cercaMatriculaForm.getSelectedTipusDocument() == null)
			cercaMatriculaForm.setSelectedTipusDocument(tipusDocumentDefecte);

		//set the collection in the request
		request.setAttribute("tipusDocuments", tipusDocuments);
	}

	/**
	 * Inicialitzar anys academics
	 * @param request
	 * @param cercaMatriculaForm
	 * @param anyAcademicDefecte
	 * @throws Throwable
	 */
	private void inicialitzaAnysAcademics(HttpServletRequest request,
			CercaMatriculaForm cercaMatriculaForm, Integer anyAcademicDefecte)
	throws Throwable {
		//Carrega opcions de tipus d'usuaris
		AnyAcademic[] arrAnyAcademics = HelperUI.recuperaAnysAcademics(new HashMap());

		ArrayList anyAcademics = new ArrayList(Arrays.asList(arrAnyAcademics));
		cercaMatriculaForm.setAnyAcademic(anyAcademics);
		if (cercaMatriculaForm.getSelectedAnyAcademic() == null)
			cercaMatriculaForm.setSelectedAnyAcademic(anyAcademicDefecte);

		//set the collection in the request
		request.setAttribute("anysAcademics", anyAcademics);
	}


	/**
	 * Avs matrcula oberta
	 * @param request
	 * @throws Throwable
	 */
	private void avisMatriculaOberta(HttpServletRequest request) throws Throwable {
		if (!isObertaPeriodeMatriculacio()) {
			ActionErrors errors = new ActionErrors();
			errors.add("matriculaError", new ActionError("error.periode.matricula"));
			saveErrors(request, errors);
		}
		else {
			ActionErrors errors = new ActionErrors();
			errors.add("matriculaAvis", new ActionError("avis.periode.matricula"));
			saveErrors(request, errors);

		}

	}

	/**
	 * Verifiquem si el periode de matriculacio esta obert
	 * @return boolean
	 * @throws Throwable
	 */
	private boolean isObertaPeriodeMatriculacio() throws Throwable {

		AnyAcademic[] arrAnyAcademics;
		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();

		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI_MATRICULA, Internacionalitzacio.nomesData(new Date()));
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI_MATRICULA, Internacionalitzacio.nomesData(new Date()));
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

		Object[] o = (Object[]) anyAcademicDAO.findAnyAcademic(parametres);

		if (o != null && o.length > 0)
			return true;
		return false;
	}

	/**
	 * Reiniciar valors del formulari
	 * @param cercaMatriculaForm
	 */
	private void resetFormulari(CercaMatriculaForm cercaMatriculaForm) {

		cercaMatriculaForm.setCodiUsuari("");
		cercaMatriculaForm.setCognom1("");
		cercaMatriculaForm.setCognom2("");
		cercaMatriculaForm.setNom("");
		cercaMatriculaForm.setNumDocument("");
		cercaMatriculaForm.setMatricules(new Matricula[0]);

	}

}

