package com.uoc.tfc.UI.struts.action.matricula;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.ConfiguracioUsuari;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.common.Internacionalitzacio;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.manteniment.Assignatura;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.manteniment.TipusQualificacio;
import com.uoc.tfc.Base.matricula.Matricula;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.mestres.TipusDocument;
import com.uoc.tfc.Base.qualificacio.Qualificacio;
import com.uoc.tfc.Base.usuari.Usuari;
import com.uoc.tfc.Base.usuari.UsuariAdministrador;
import com.uoc.tfc.Base.usuari.UsuariAdministratiu;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.LN.dao.manteniment.AnyAcademicDAO;
import com.uoc.tfc.LN.dao.manteniment.AssignaturaDAO;
import com.uoc.tfc.LN.dao.matricula.MatriculaDAO;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.dao.usuari.UsuariDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.matricula.EditMatriculaForm;


/**
 * Acci per l'edici de les matrcules
 * @author ojaen
 * */
public class EditMatriculaAction extends DispatchAction {

	/**
	 * Prepara l'edici de la matrcula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditMatriculaForm matriculaEditForm = (EditMatriculaForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		MatriculaDAO matriculaDAO = new MatriculaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Matricula.CRITERI_MATRICULA_ID_MATRICULA, id);

		Matricula[] matricules = null;

		try {
			matricules = matriculaDAO.findMatricula(parametres);

			if (!isPeriodeObertMatricula(matricules[0])) {

				ActionErrors errors = new ActionErrors();
				errors.add("matriculaError", new ActionError("error.periode.matricula"));
				saveErrors(request, errors);

				inicialitzaDadesInicialsCercaMatricula(request);

				return (mapping.findForward("failureCerca"));
			}
			matriculaEditForm.setMatricula(matricules[0]);

			inicialitzaDades(request, matriculaEditForm, matricules, matricules[0].getIdUsuari());

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
			try {
				inicialitzaDadesInicialsCercaMatricula(request);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "editMatriculaError"));
				return (mapping.findForward(CodisMestre.FAILURE));
			}
			return (mapping.findForward("failureCerca"));
		}

		return mapping.findForward("editMatricula");
	}


	/**
	 * Inicialitzaci de les llistes al tornar a CercaMatricula
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaDadesInicialsCercaMatricula (
			HttpServletRequest request) throws Throwable {
		inicialitzaAnysAcademicsCerca(request);
		inicialitzaTipusDocumentsCerca(request);
		inicialitzaEstatsCerca(request);
	}



	/**
	 * Hem d'estar en el periode d'inscripci a la matrcula
	 * @param matricula
	 * @return
	 */
	private boolean isPeriodeObertMatricula(Matricula matricula) {
		Date dataActual = Internacionalitzacio.nomesData(new Date());

		//Si s administrador o personal administratiu podr crear/modificar matrcules
		//fora del perode de matriculaci
		if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariAdministrador ||
			ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariAdministratiu) {
			return true;
		}

		if (dataActual.before(matricula.getIdAnyAcademic().getDataIniciMatricula()) ||
			dataActual.after(matricula.getIdAnyAcademic().getDataFiMatricula())	)
			return false;
		return true;
	}


	/**
	 * No tenim cap matrcula, agafem l'ultim any acadmic i comparem si estem dintre
	 * del periode d'inscripcio a la matrcula
	 * @param
	 * @return
	 * @throws Throwable
	 */
	private boolean isPeriodeObertMatricula(ActionForm form) throws Throwable {
		AnyAcademic[] anysAcademics = cercaAnyAcademicAMatricular();

		if (anysAcademics == null || anysAcademics.length == 0)
			return false;
		else {
			EditMatriculaForm matriculaEditForm = (EditMatriculaForm) form;
			matriculaEditForm.setSelectedAnyAcademic(anysAcademics[0].getIdAnyAcademic());
		}
		return true;
	}


	/**
	 * Cerca any acadmic a matricular
	 * @return AnyAcademic[]
	 * @throws Throwable
	 */
	private AnyAcademic[] cercaAnyAcademicAMatricular() throws Throwable {
		Date dataActual = Internacionalitzacio.nomesData(new Date());

		HashMap parametres = new HashMap();
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_INICI_MATRICULA, dataActual);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_DATA_FI_MATRICULA, dataActual);
		parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ESTAT, CodisMestre.CODI_ESTAT_ALTA);

		AnyAcademic[] anysAcademics = HelperUI.recuperaAnysAcademics(parametres);
		return anysAcademics;
	}

	/**
	 * Validar si l'usuari ja ha estat matriculat en aquest any acadmic
	 *
	 */
	private boolean validaJaMatriculat(ActionForm form, UsuariAlumne p_usuari) throws Throwable {
		AnyAcademic[] anysAcademics = cercaAnyAcademicAMatricular();

		if (anysAcademics != null || anysAcademics.length > 0) {
			AnyAcademic anyAcademic = anysAcademics[0];

			for (Iterator<Matricula> iter=p_usuari.getSetOfMatricula().iterator();iter.hasNext();) {
				Matricula matricula = iter.next();
				if (matricula.getIdAnyAcademic().getIdAnyAcademic().equals(anyAcademic.getIdAnyAcademic()))
					return false;
			}
		}

		return true;

	}



	/**
	 * Afegir nova matrcula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		Integer idUsuari = Integer.valueOf(request.getParameter("codiUsuari"));

		try {

			if (!isPeriodeObertMatricula(form)) {

				ActionErrors errors = new ActionErrors();
				errors.add("matriculaError", new ActionError("error.periode.matricula"));
				saveErrors(request, errors);

				inicialitzaDadesInicialsCercaMatricula(request);
				return (mapping.findForward("failureCerca"));
			}
			UsuariAlumne usuari = (UsuariAlumne) cercaUsuari(idUsuari);

			if (usuari != null && !validaJaMatriculat(form, usuari)) {
				ActionErrors errors = new ActionErrors();
				errors.add("matriculaError", new ActionError("error.ja.matriculat"));
				saveErrors(request, errors);

				inicialitzaDadesInicialsCercaMatricula(request);
				return (mapping.findForward("failureCerca"));

			}


			if (usuari != null) {
				EditMatriculaForm matriculaEditForm = (EditMatriculaForm) form;

				/*Matricula matricula = new Matricula();
				matricula.setIdAnyAcademic(cercaAnyAcademicAMatricular());*/

				inicialitzaDades(request, matriculaEditForm, null, usuari);

				//inicialitzem la resta de valors del form
				inicialitzaForm(matriculaEditForm, usuari);

				//No tenim curs a matricular
				if (matriculaEditForm.getSelectedCurs() == null) {
					ActionErrors errors = new ActionErrors();
					errors.add("matriculaError", new ActionError("error.curs.inexistent"));
					saveErrors(request, errors);
					inicialitzaDadesInicialsCercaMatricula(request);
					return (mapping.findForward("failureCerca"));
				}


			}
			else  {
				ActionErrors errors = new ActionErrors();
				errors.add("matriculaError", new ActionError("error.curs.inexistent"));
				saveErrors(request, errors);
				inicialitzaDadesInicialsCercaMatricula(request);
				return (mapping.findForward("failureCerca"));
			}

		} catch (Throwable e) {
			try {
				inicialitzaDadesInicialsCercaMatricula(request);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
				return (mapping.findForward("failureCerca"));
			}
			saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
			return (mapping.findForward("failureCerca"));
		}

		return mapping.findForward("editMatricula");

	}

	/**
	 * Cerca de l'alumne al que se li fara la matricula
	 * @param idUsuari
	 * @return
	 * @throws Throwable
	 */
	private Usuari cercaUsuari(Integer idUsuari) throws Throwable {
		UsuariDAO usuariDAO = new UsuariDAO();
		HashMap parametres = new HashMap();
		parametres.put(Usuari.CRITERI_USUARI_ID_USUARI, idUsuari);
		Usuari[] usuaris = usuariDAO.findUsuari(parametres);
		if (usuaris != null && usuaris.length > 0)
			return usuaris[0];
		return null;
	}



	/**
	 * Inicialitzar dades
	 * @param request
	 * @param matriculaEditForm
	 * @param matricules
	 * @param p_usuari
	 * @throws Throwable
	 */
	private void inicialitzaDades(HttpServletRequest request,
			EditMatriculaForm matriculaEditForm, Matricula[] matricules, UsuariAlumne p_usuari)
	throws Throwable {
		inicialitzaAnysAcademics(request, matriculaEditForm, matricules);
		inicialitzaEstats(request, matriculaEditForm, matricules);
		inicialitzaCursos(request, matriculaEditForm, matricules, p_usuari);
		inicialitzaAules(request, matriculaEditForm, matricules);
	}

	/**
	 * Inicialitzar formulari
	 * @param matriculaEditForm
	 * @param p_usuari
	 */
	private void inicialitzaForm(EditMatriculaForm matriculaEditForm, UsuariAlumne p_usuari) {
		matriculaEditForm.setMatricula(new Matricula());
		matriculaEditForm.getMatricula().setIdUsuari(p_usuari);
	}

	/**
	 * Gravar matrcula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveMatricula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditMatriculaForm matriculaEditForm = (EditMatriculaForm) form;

		MatriculaDAO matriculaDAO = new MatriculaDAO();
		MestreDAO mestreDAO = new MestreDAO();

		UsuariAlumne usuari = null;

		try {
			//Cerquem l'Estat seleccionat i l'assignem a la matricula
			Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, matriculaEditForm.getSelectedEstat());
			matriculaEditForm.getMatricula().setIdEstat((Estat) o[0]);

			//Cerquem l'any acadmic seleccionat i l'assignem a la matricula
			HashMap parametres = new HashMap();
			parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC, matriculaEditForm.getSelectedAnyAcademic());
			AnyAcademic[] arrAnysAcademics = HelperUI.recuperaAnysAcademics(parametres);
			matriculaEditForm.getMatricula().setIdAnyAcademic((AnyAcademic) arrAnysAcademics[0]);

			//Cerquem el curs seleccionat en la matricula
			parametres = new HashMap();
			parametres.put(Curs.CRITERI_CURS_ID_CURS, matriculaEditForm.getSelectedCurs());
			Curs[] arrCursos = HelperUI.recuperaCursos(parametres);
			matriculaEditForm.getMatricula().setIdCurs((Curs) arrCursos[0]);


			//Cerquem l'aula seleccionada en la matricula
			if (matriculaEditForm.getSelectedAula() > 0) {
				parametres = new HashMap();
				parametres.put(Aula.CRITERI_AULA_ID_AULA, matriculaEditForm.getSelectedAula());
				Aula[] arrAules = recuperaAules(parametres, false);
				matriculaEditForm.getMatricula().setIdAula((Aula) arrAules[0]);
			}
			else
				matriculaEditForm.getMatricula().setIdAula(null);


			//Usuari
			usuari = matriculaEditForm.getMatricula().getIdUsuari();
			matriculaEditForm.getMatricula().setIdUsuari(usuari);
			usuari.addMatricula(matriculaEditForm.getMatricula());

			matriculaDAO.gravar(matriculaEditForm.getMatricula());
			inicialitzaDadesInicialsCercaMatricula(request);

		} catch (Throwable e) {
			try {
				if (usuari != null)
					usuari.removeMatricula(matriculaEditForm.getMatricula());
				inicialitzaDades(request, matriculaEditForm, null, usuari);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "editMatriculaError"));
				return (mapping.findForward(CodisMestre.FAILURE));

			}
			saveErrors(request, HelperDB.tractarError(e, "editMatriculaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaMatricula");
	}

	/**
	 * Esborrar matrcula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteMatricula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditMatriculaForm matriculaEditForm = (EditMatriculaForm) form;

		MatriculaDAO matriculaDAO = new MatriculaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Matricula.CRITERI_MATRICULA_ID_MATRICULA, matriculaEditForm.getMatricula().getIdMatricula());


		Matricula[] matricules = null;

		try {
			matricules = matriculaDAO.findMatricula(parametres);

			ActionErrors errors = validarUsuariMatricula(request,
					matricules[0].getIdCurs(),
					matricules[0].getIdUsuari());

			if (errors != null && errors.size() > 0) {
				inicialitzaDadesInicialsCercaMatricula(request);
				return (mapping.findForward("failureCerca"));
			}

			matriculaDAO.esborrar(matricules[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editMatriculaError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaMatricula");
	}

	/**
	 * Validar matrcula de l'usuari
	 * @param request
	 * @param idCurs
	 * @param idUsuari
	 * @return ActionErrors
	 */
	private ActionErrors validarUsuariMatricula(HttpServletRequest request, Curs idCurs,
			UsuariAlumne idUsuari) {
		ActionErrors errors = new ActionErrors();
		boolean bValida = true;
		for (Iterator<Qualificacio> iterator = idUsuari.getSetOfQualificacio().iterator(); iterator.hasNext();) {
			Qualificacio qualificacio = iterator.next();

			//Si ja tenim alguna qualificacio en aquest curs no es pot esborrar
			if (qualificacio.getIdAsigAulaProf().getIdAula().getIdCurs() != null &&
					idCurs.getIdCurs().equals(qualificacio.getIdAsigAulaProf().getIdAula().getIdCurs().getIdCurs())) {
				bValida = false;
			}
		}


		if (!bValida) {
			ActionError error = new ActionError("error.matricula.delete");
			errors.add("matriculaError",error);
			saveErrors(request,errors);

		}
		return errors;

	}



	/**
	 * Consultar matrcula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaMatricula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditMatriculaForm matriculaEditForm = (EditMatriculaForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		MatriculaDAO matriculaDAO = new MatriculaDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Matricula.CRITERI_MATRICULA_ID_MATRICULA, id);

		Matricula[] matricules = null;

		try {
			matricules = matriculaDAO.findMatricula(parametres);
			matriculaEditForm.setMatricula(matricules[0]);


			inicialitzaEstats(request, matriculaEditForm, matricules);
			inicialitzaAnysAcademics(request, matriculaEditForm, matricules);
			inicialitzaCursos(request, matriculaEditForm, matricules, matricules[0].getIdUsuari());
			inicialitzaAules(request, matriculaEditForm, matricules);


		} catch (Throwable e) {
			try {
				inicialitzaDadesInicialsCercaMatricula(request);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
				return (mapping.findForward("failureCerca"));
			}
			saveErrors(request, HelperDB.tractarError(e, "matriculaError"));
			return (mapping.findForward("failureCerca"));
		}


		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editMatricula");
	}



	/**
	 * Recuperem els tipus de titulacions de la base de dades
	 * @return Curs[]: Array de tipus de titulaci
	 */
	private Aula[] recuperaAules(HashMap p_parametres, boolean crearCapAula) throws Throwable{
		
		Aula[] o = HelperUI.recuperaAules(p_parametres);

		Aula[] aules = new Aula[o.length+1];
		if (crearCapAula) {
			//Creem un curs fictici per poder seleccionar 'Cap Curs Previ'
			aules[0] = new Aula();
			aules[0].setCodiAula("0");
			aules[0].setDescripcio("Sense Assignar Aula");
			aules[0].setIdAula(new Integer(0));

			for (int i = 0; i < o.length; i++)
				aules[i+1] = o[i];
			return aules;
		}
		else
			return o;
	}

	/**
	 * Inicialitzar estats
	 * @param request
	 * @param matriculaEditForm
	 * @param matricules
	 */
	private void inicialitzaEstats(HttpServletRequest request,
			EditMatriculaForm matriculaEditForm, Matricula[] matricules) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();
		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		matriculaEditForm.setEstats(estats);
		if (matricules != null && matricules.length > 0 && matricules[0].getIdEstat() != null)
			matriculaEditForm.setSelectedEstat(matricules[0].getIdEstat().getIdEstat());
		else if (matriculaEditForm.getSelectedEstat() == null)
			matriculaEditForm.setSelectedEstat(CodisMestre.CODI_ESTAT_ALTA);
		//set the collection in the request
		request.setAttribute("estats", estats);
	}

	/**
	 * Inicialitzar anys acadmics
	 * @param request
	 * @param matriculaEditForm
	 * @param matricules
	 * @throws Throwable
	 */
	private void inicialitzaAnysAcademics(HttpServletRequest request,
			EditMatriculaForm matriculaEditForm, Matricula[] matricules)
	throws Throwable {
		AnyAcademic[] arrAnyAcademic;
		arrAnyAcademic = HelperUI.recuperaAnysAcademics(new HashMap());
		//define a dummy collection
		ArrayList<AnyAcademic> anysAcademics = new ArrayList(Arrays.asList(arrAnyAcademic));
		matriculaEditForm.setAnysAcademics(anysAcademics);
		if (matricules != null && matricules.length > 0 && matricules[0].getIdAnyAcademic() != null &&
				(matriculaEditForm.getSelectedAnyAcademic() == null ||
						matriculaEditForm.getSelectedAnyAcademic().intValue() == 0))
			matriculaEditForm.setSelectedAnyAcademic(matricules[0].getIdAnyAcademic().getIdAnyAcademic());
		else if (matriculaEditForm.getSelectedAnyAcademic() == null ||
				matriculaEditForm.getSelectedAnyAcademic().intValue() == 0){
			//Ens arriba ordenat ordenat per data de l'any acadmic
			for (int i=0;i<anysAcademics.size();i++) {
				AnyAcademic aa = anysAcademics.get(i);
				if (aa.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					matriculaEditForm.setSelectedAnyAcademic(anysAcademics.get(0).getIdAnyAcademic());
					break;
				}
			}
		}

		//set the collection in the request
		request.setAttribute("anysAcademics", anysAcademics);

	}

	/**
	 * Inicialitzar cursos
	 * @param request
	 * @param matriculaEditForm
	 * @param matricules
	 * @param p_usuari
	 * @throws Throwable
	 */
	private void inicialitzaCursos(HttpServletRequest request,
			EditMatriculaForm matriculaEditForm, Matricula[] matricules, UsuariAlumne p_usuari)
	throws Throwable {
		Curs[] arrCurs;
		arrCurs = HelperUI.recuperaCursos(new HashMap());
		//define a dummy collection
		ArrayList cursos = new ArrayList(Arrays.asList(arrCurs));
		matriculaEditForm.setCursos(cursos);
		if (matricules != null && matricules.length > 0 && matricules[0].getIdCurs() != null)
			matriculaEditForm.setSelectedCurs(matricules[0].getIdCurs().getIdCurs());
		else  {
			matriculaEditForm.setSelectedCurs(cercaCursAMatricular(p_usuari));
		}

		//set the collection in the request
		request.setAttribute("cursos", cursos);
	}


	/**
	 * Cercarem el curs en el que s'ha de matricular l'alumne
	 * @param p_usuari
	 * @return
	 * @throws Throwable
	 */
	private Integer cercaCursAMatricular(UsuariAlumne p_usuari) throws Throwable {
		Matricula matricula = cercaUltimaMatricula(p_usuari);
		Curs curs = matricula.getIdCurs();
		Set<Qualificacio> qualificacions = cercaUltimesQualificacions(p_usuari, matricula.getIdAnyAcademic());

		int numAssignaturesSuspeses = 0;
		TipusQualificacio[] tipusQualificacio = HelperUI.recuperaTipusQualificacions(new HashMap());
		for (Iterator<Qualificacio> iter=qualificacions.iterator();iter.hasNext();) {
			Qualificacio qualif = iter.next();
			if (!qualif.isAprovat(tipusQualificacio))
				numAssignaturesSuspeses++;
		}

		if (numAssignaturesSuspeses > Matricula.NUM_ASSIGNATURES_SUSPESES_REPETEIX)
			return curs.getIdCurs();
		else {
			return cercaCursSeguent(curs);
		}
	}


	/**
	 * Cercar les ltimes qualificacions d'un any acadmic
	 * @param pUsuari
	 * @param curs
	 * @param idAnyAcademic
	 * @return Set<Qualificacio>
	 */
	private Set<Qualificacio> cercaUltimesQualificacions(UsuariAlumne p_usuari, AnyAcademic idAnyAcademic) {
		Set<Qualificacio> qualificacions = new HashSet<Qualificacio>();
		for (Iterator<Qualificacio> iter = p_usuari.getSetOfQualificacio().iterator();iter.hasNext();) {
			Qualificacio qualificacio = iter.next();
			if (qualificacio.getIdAnyAcademic().getIdAnyAcademic().equals(idAnyAcademic.getIdAnyAcademic()))
				qualificacions.add(qualificacio);
		}
		return qualificacions;
	}

	/**
	 * Cerquem l'ultima matrcula de l'alumne
	 * @param pUsuari
	 * @return Matricula
	 */
	private Matricula cercaUltimaMatricula(UsuariAlumne pUsuari) {
		Date ultimaDataMatricula = null;
		Matricula ultimaMatricula = null;
		for (Iterator<Matricula> iterator=pUsuari.getSetOfMatricula().iterator();iterator.hasNext();) {
			Matricula matricula = iterator.next();
			if (ultimaDataMatricula == null ||
					ultimaDataMatricula.before(matricula.getDataMatricula()) ){
				ultimaDataMatricula = matricula.getDataMatricula();
				ultimaMatricula = matricula;
			}
		}

		return ultimaMatricula;

	}



	/**
	 * Cerquem el proper curs en el que s'ha de matricular l'alumne
	 * @param curs
	 * @return
	 * @throws Throwable
	 */
	private Integer cercaCursSeguent(Curs curs) throws Throwable {
		HashMap parametres = new HashMap();
		parametres.put(Curs.CRITERI_CURS_ID_CURS_PREVI, curs.getIdCurs());

		Curs[] cursos = HelperUI.recuperaCursos(parametres);

		if (cursos != null && cursos.length > 0)
			return cursos[0].getIdCurs();
		return null;
	}


	/**
	 * Inicialitzar aules
	 * @param request
	 * @param matriculaEditForm
	 * @param matricules
	 * @throws Throwable
	 */
	private void inicialitzaAules(HttpServletRequest request,
			EditMatriculaForm matriculaEditForm, Matricula[] matricules)
	throws Throwable {
		Aula[] arrAula;

		HashMap parametres = new HashMap();

		//Nomes carreguem les aules del curs seleccionat
		if (matriculaEditForm.getSelectedCurs() != null &&
				matriculaEditForm.getSelectedCurs().intValue() > 0)
			parametres.put(Aula.CRITERI_AULA_ID_CURS, matriculaEditForm.getSelectedCurs());

		arrAula = recuperaAules(new HashMap(), true);
		//define a dummy collection
		ArrayList aules = new ArrayList(Arrays.asList(arrAula));
		matriculaEditForm.setAules(aules);
		if (matricules != null && matricules.length > 0 && matricules[0].getIdAula() != null)
			matriculaEditForm.setSelectedAula(matricules[0].getIdAula().getIdAula());
		else
			matriculaEditForm.setSelectedAula(new Integer(0));

		//set the collection in the request
		request.setAttribute("aules", aules);
	}

	/**
	 * Inicialitzar estats de cerca
	 * @param request
	 */
	private void inicialitzaEstatsCerca(HttpServletRequest request) {
		//Carrega opcions d'estat
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		//set the collection in the request
		request.setAttribute("estats", estats);
	}

	/**
	 * Inicialitzar tipus de documents de cerca
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaTipusDocumentsCerca(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus de documents
		TipusDocument[] arrTipusDocuments = HelperUI.recuperaTipusDocuments(new HashMap());

		ArrayList tipusDocuments = new ArrayList(Arrays.asList(arrTipusDocuments));
		//set the collection in the request
		request.setAttribute("tipusDocuments", tipusDocuments);
	}

	/**
	 * Inicialitzar anys acadmics de cerca
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaAnysAcademicsCerca(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus d'usuaris
		AnyAcademic[] arrAnyAcademics = recuperaAnyAcademics(new HashMap());

		ArrayList anyAcademics = new ArrayList(Arrays.asList(arrAnyAcademics));
		//set the collection in the request
		request.setAttribute("anysAcademics", anyAcademics);
	}



	/**
	 * Recuperem els diferents anys acadmics de la base de dades
	 * @return AnyAcademic[]: Array d'anys acadmics
	 */
	private AnyAcademic[] recuperaAnyAcademics(HashMap p_parametres) throws Throwable{
		AnyAcademic[] arrAnyAcademics;
		AnyAcademicDAO anyAcademicDAO = new AnyAcademicDAO();
		Object[] o = (Object[]) anyAcademicDAO.findAnyAcademic(p_parametres);
		arrAnyAcademics = new AnyAcademic[o.length];
		for (int i=0;i<o.length;i++) {
			arrAnyAcademics[i] = (AnyAcademic) o[i];
		}
		return arrAnyAcademics;
	}

	/**
	 * Recuperem els assignatures de la base de dades
	 * @return Assignatura[]: Array de assignatures
	 */
	private Assignatura[] recuperaAssignatures(HashMap p_parametres, boolean creaCapassignatura) throws Throwable{

		AssignaturaDAO assignaturaDAO = new AssignaturaDAO();
		Assignatura[] o = HelperUI.recuperaAssignatura(p_parametres);

		Assignatura[] assignatures = new Assignatura[o.length+1];

		if (creaCapassignatura) {
			//Creem un assignatura fictici per poder seleccionar 'Cap Assignatura Previ'
			assignatures[0] = new Assignatura();
			assignatures[0].setCodiAssignatura("0");
			assignatures[0].setDescripcio("Qualsevol Assignatura");
			assignatures[0].setIdAssignatura(new Integer(0));

			for (int i = 0; i < o.length; i++)
				assignatures[i+1] = o[i];
			return assignatures;
		}
		else
			return o;

	}



}
