package com.uoc.tfc.UI.struts.action.notificacio;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.ConfiguracioUsuari;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.Curs;
import com.uoc.tfc.Base.manteniment.CursAssignatura;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.mestres.TipusNotificacio;
import com.uoc.tfc.Base.notificacio.Notificacio;
import com.uoc.tfc.Base.notificacio.NotificacioAssignatura;
import com.uoc.tfc.Base.notificacio.NotificacioAula;
import com.uoc.tfc.Base.notificacio.NotificacioCurs;
import com.uoc.tfc.Base.notificacio.NotificacioEscola;
import com.uoc.tfc.Base.usuari.Usuari;
import com.uoc.tfc.Base.usuari.UsuariAdministrador;
import com.uoc.tfc.Base.usuari.UsuariAdministratiu;
import com.uoc.tfc.Base.usuari.UsuariProfessor;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.dao.notificacio.NotificacioDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.notificacio.EditNotificacioForm;


/**
 * Accio pel manteniment de les notificacions
 * @author ojaen
 *
 */
public class EditNotificacioAction extends DispatchAction {

	/**
	 * Prepara l'edici de la notificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditNotificacioForm notificacioEditForm = (EditNotificacioForm) form;

		//Recuperem el id de la notificaci  seleccionada
		Integer id = Integer.valueOf(request.getParameter("id"));
		NotificacioDAO notificacioDAO = new NotificacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Notificacio.CRITERI_NOTIFICACIO_ID_NOTIFICACIO, id);

		Notificacio[] notificacions = null;

		try {
			notificacions = notificacioDAO.findNotificacio(
					ConfiguracioUsuari.getInstance().getUsuari(),
					parametres);

			inicialitzaEstats(request, notificacioEditForm, notificacions);
			inicialitzaTipusNotificacions(request, notificacioEditForm, notificacions, notificacions[0].getIdTipusNotificacio());
			inicialitzaAules(request, notificacioEditForm, notificacions);
			inicialitzaCursos(request, notificacioEditForm, notificacions);
			inicialitzaCursAssignatures(request, notificacioEditForm, notificacions);

			notificacioEditForm.setNotificacio(notificacions[0]);

			request.setAttribute("TipusNotificacio", notificacions[0].getIdTipusNotificacio());

			//Validem si l'usuari pot editar la notificacio
			ActionErrors errors = validarUsuariNotificacio(request, notificacions);

			if (errors != null && errors.size() > 0) {
				inicialitzaTipusNotificacions(request, notificacioEditForm, null, null);
				inicialitzaEstats(request, notificacioEditForm, null);
				inicialitzaUsuariConnectat(request);

				return (mapping.findForward("failureCerca"));
			}


		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editNotificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}

		return mapping.findForward("editNotificacio");
	}

	/**
	 * Prepara el edit per afegir una notificaci tipus Escola
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAddEscola(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		return prepareAdd(mapping, form, request, response, CodisMestre.CODI_TIPUS_NOTIFICACIO_ESCOLA);
	}

	/**
	 * Prepara el edit per afegir una notificaci tipus Aula
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAddAula(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		return prepareAdd(mapping, form, request, response, CodisMestre.CODI_TIPUS_NOTIFICACIO_AULA);
	}

	/**
	 * Prepara el edit per afegir una notificaci tipus Curs
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAddCurs(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		return prepareAdd(mapping, form, request, response, CodisMestre.CODI_TIPUS_NOTIFICACIO_CURS);
	}

	/**
	 * Prepara el edit per afegir una notificaci tipus Assignatura
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAddAssignatura(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		return prepareAdd(mapping, form, request, response, CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA);
	}


	/**
	 * prepares l'edici per afegir una nova notificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareAdd(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response, Integer p_tipusNotificacio) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditNotificacioForm notificacioEditForm = (EditNotificacioForm) form;
		Notificacio[] notificacions = null;

		try {
			inicialitzaDades(request, p_tipusNotificacio, notificacioEditForm,
					notificacions);

			//inicialitzem la resta de valors del form
			inicialitzaForm(notificacioEditForm, p_tipusNotificacio);

			request.setAttribute("TipusNotificacio", p_tipusNotificacio);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "notificacioError"));
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("editNotificacio");

	}

	/**
	 * Inicialitzar desplegables
	 * @param request
	 * @param p_tipusNotificacio
	 * @param notificacioEditForm
	 * @param notificacions
	 * @throws Throwable
	 */
	private void inicialitzaDades(HttpServletRequest request,
			Integer p_tipusNotificacio,
			EditNotificacioForm notificacioEditForm, Notificacio[] notificacions)
	throws Throwable {
		inicialitzaEstats(request, notificacioEditForm, notificacions);
		inicialitzaTipusNotificacions(request, notificacioEditForm, notificacions, p_tipusNotificacio);
		inicialitzaAules(request, notificacioEditForm, notificacions);
		inicialitzaCursos(request, notificacioEditForm, notificacions);
		inicialitzaCursAssignatures(request, notificacioEditForm, notificacions);
	}

	/**
	 * Inicialitzar formulari
	 * @param notificacioEditForm
	 * @param p_tipusNotificacio
	 */
	private void inicialitzaForm(EditNotificacioForm notificacioEditForm, Integer p_tipusNotificacio) {
		notificacioEditForm.setIdNotificacio(null);
		notificacioEditForm.setTitol("");
		notificacioEditForm.setDescripcio("");
		notificacioEditForm.setDataInici(null);
		notificacioEditForm.setDataFi(null);

		if (p_tipusNotificacio == CodisMestre.CODI_TIPUS_NOTIFICACIO_ESCOLA)
			notificacioEditForm.setNotificacio(new NotificacioEscola());
		else if (p_tipusNotificacio == CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA)
			notificacioEditForm.setNotificacio(new NotificacioAssignatura());
		else if (p_tipusNotificacio == CodisMestre.CODI_TIPUS_NOTIFICACIO_AULA)
			notificacioEditForm.setNotificacio(new NotificacioAula());
		else if (p_tipusNotificacio == CodisMestre.CODI_TIPUS_NOTIFICACIO_CURS)
			notificacioEditForm.setNotificacio(new NotificacioCurs());

		notificacioEditForm.setSelectedEstat(new Integer(CodisMestre.CODI_ESTAT_ALTA));
		notificacioEditForm.setSelectedTipusNotificacio(p_tipusNotificacio);
		notificacioEditForm.setSelectedCursAssignatura(new Integer(0));
		notificacioEditForm.setSelectedCurs(new Integer(0));
		notificacioEditForm.setSelectedAula(new Integer(0));
	}

	/**
	 * Gravar notificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveNotificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditNotificacioForm notificacioEditForm = (EditNotificacioForm) form;

		NotificacioDAO notificacioDAO = new NotificacioDAO();
		MestreDAO mestreDAO = new MestreDAO();

		try {
			//Cerquem l'Estat seleccionat i l'assignem al notificacio
			Object[] o = mestreDAO.findPerCodi("Estat", Estat.ID_ESTAT, notificacioEditForm.getSelectedEstat());
			notificacioEditForm.getNotificacio().setIdEstat((Estat) o[0]);

			//Cerquem el tipus notificacio seleccionat i l'assignem al notificacio
			o = mestreDAO.findPerCodi("TipusNotificacio", TipusNotificacio.ID_TIPUS_NOTIFICACIO, notificacioEditForm.getSelectedTipusNotificacio());
			notificacioEditForm.getNotificacio().setIdTipusNotificacio(notificacioEditForm.getSelectedTipusNotificacio());

			//Cerquem el curs seleccionat en la notificacio
			if (notificacioEditForm.getNotificacio() instanceof NotificacioCurs) {

				HashMap parametres = new HashMap();
				parametres.put(Curs.CRITERI_CURS_ID_CURS, notificacioEditForm.getSelectedCurs());
				Curs[] arrCursos = HelperUI.recuperaCursos(parametres);

				((NotificacioCurs) notificacioEditForm.getNotificacio()).setIdCurs((Curs) arrCursos[0]);
			}


			//Cerquem l'aula seleccionada en la notificacio
			if (notificacioEditForm.getNotificacio() instanceof NotificacioAula) {

				HashMap parametres = new HashMap();
				parametres.put(Aula.CRITERI_AULA_ID_AULA, notificacioEditForm.getSelectedAula());
				Aula[] arrAules = HelperUI.recuperaAules(parametres);
				((NotificacioAula) notificacioEditForm.getNotificacio()).setIdAula((Aula) arrAules[0]);
			}


			//Cerquem l'aula seleccionada en la notificacio
			if (notificacioEditForm.getNotificacio() instanceof NotificacioAssignatura) {

				HashMap parametres = new HashMap();
				parametres.put(CursAssignatura.CRITERI_ASSIGNATURA_ID_CURSASSIGNATURA, notificacioEditForm.getSelectedCursAssignatura());
				CursAssignatura[] arrCursAssignatures = HelperUI.recuperaCursAssignatures(parametres);
				((NotificacioAssignatura) notificacioEditForm.getNotificacio()).setIdCursAssignatura((CursAssignatura) arrCursAssignatures[0]);
			}

			notificacioEditForm.getNotificacio().setIdUsuari(ConfiguracioUsuari.getInstance().getUsuari());

			//Controlem errors
			ActionErrors errors = validar(request, notificacioEditForm);

			if (errors != null && errors.size() > 0) {

				Notificacio[] notificacions = new Notificacio[1];
				notificacions[0] = notificacioEditForm.getNotificacio();
				inicialitzaEstats(request, notificacioEditForm, notificacions);
				inicialitzaTipusNotificacions(request, notificacioEditForm, notificacions, notificacioEditForm.getNotificacio().getIdTipusNotificacio());
				inicialitzaAules(request, notificacioEditForm, notificacions);
				inicialitzaCursos(request, notificacioEditForm, notificacions);
				inicialitzaCursAssignatures(request, notificacioEditForm, notificacions);

				request.setAttribute("TipusNotificacio", notificacioEditForm.getNotificacio().getIdTipusNotificacio());

				return (mapping.findForward(CodisMestre.FAILURE));
			}

			notificacioDAO.gravar(notificacioEditForm.getNotificacio());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editNotificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaNotificacio");
	}

	/**
	 * Esborrar una notificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward deleteNotificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditNotificacioForm notificacioEditForm = (EditNotificacioForm) form;

		NotificacioDAO notificacioDAO = new NotificacioDAO();
		Integer id = Integer.valueOf(request.getParameter("id"));

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Notificacio.CRITERI_NOTIFICACIO_ID_NOTIFICACIO, id);


		Notificacio[] notificacions = null;

		try {
			notificacions = notificacioDAO.findNotificacio(
					ConfiguracioUsuari.getInstance().getUsuari(),
					parametres);

			ActionErrors errors = validarUsuariNotificacio(request, notificacions);

			if (errors != null && errors.size() > 0) {
				inicialitzaDades(request,
						notificacioEditForm.getSelectedTipusNotificacio(),
						notificacioEditForm,
						notificacions);
				inicialitzaUsuariConnectat(request);
				return (mapping.findForward("failureCerca"));
			}

			notificacioDAO.esborrar(notificacions[0]);
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "notificacioError"));
			try {
				inicialitzaDades(request,
						notificacioEditForm.getIdTipusNotificacio(),
						notificacioEditForm,
						notificacions);
				inicialitzaUsuariConnectat(request);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e1, "notificacioError"));
			}
			return (mapping.findForward("failureCerca"));
		}


		return mapping.findForward("cercaNotificacio");
	}

	/**
	 * Validar si un usuari pot esborrar/modificar una notificaci.
	 * Aixo passar amb els professors que podran veure les notificacions i crear alguna notificaci,
	 * pero nomrs podra esborrar les notificacions creades per ell.
	 * @param request
	 * @param notificacions
	 * @return ActionErrors
	 */
	private ActionErrors validarUsuariNotificacio(HttpServletRequest request,
			Notificacio[] notificacions) {
		ActionErrors errors = new ActionErrors();
		//Si es professor nomes pot esborrar o modificar les seves notificacions
		if (ConfiguracioUsuari.getInstance().getUsuari() instanceof UsuariProfessor &&
				!notificacions[0].getIdUsuari().getIdUsuari().equals(
						ConfiguracioUsuari.getInstance().getUsuari().getIdUsuari())) {
			ActionError error = new ActionError("error.permisos.required");
			errors.add("notificacioError",error);
			saveErrors(request,errors);
		}

		return errors;
	}


	/**
	 * Consultar notificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaNotificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditNotificacioForm notificacioEditForm = (EditNotificacioForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		NotificacioDAO notificacioDAO = new NotificacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Notificacio.CRITERI_NOTIFICACIO_ID_NOTIFICACIO, id);

		Notificacio[] notificacions = null;

		try {
			notificacions = notificacioDAO.findNotificacio(
					ConfiguracioUsuari.getInstance().getUsuari(),
					parametres);
			notificacioEditForm.setNotificacio(notificacions[0]);

			//Inicialitzem formulari
			inicialitzaEstats(request, notificacioEditForm, notificacions);
			inicialitzaTipusNotificacions(request, notificacioEditForm, notificacions, notificacions[0].getIdTipusNotificacio());
			inicialitzaAules(request, notificacioEditForm, notificacions);
			inicialitzaCursos(request, notificacioEditForm, notificacions);
			inicialitzaCursAssignatures(request, notificacioEditForm, notificacions);

			request.setAttribute("TipusNotificacio", notificacions[0].getIdTipusNotificacio());
		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "editNotificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editNotificacio");
	}

	/**
	 * Validaci de la pantalla de notificacions
	 * @param request
	 * @param notificacioEditForm
	 * @return ActionsErrors
	 */
	private ActionErrors validar(HttpServletRequest request, EditNotificacioForm notificacioEditForm) {
		// crear ActionError i desar in the request
		ActionErrors errors = new ActionErrors();
		//Validem nom
		if (notificacioEditForm.getNotificacio().getTitol() == null ||
				notificacioEditForm.getNotificacio().getTitol().equals("")) {
			// crear ActionError i desar in the request
			errors.add("titol",new ActionError("error.nom.required"));
			saveErrors(request,errors);
		}

		//Descripcio
		if (notificacioEditForm.getNotificacio().getDescripcio() == null ||
				notificacioEditForm.getNotificacio().getDescripcio().equals("")) {
			// crear ActionError i desar in the request
			errors.add("descripcio",new ActionError("error.descripcio.required"));
			saveErrors(request,errors);

		}

		//Data inici
		if (notificacioEditForm.getNotificacio().getDataInici() == null) {
			// crear ActionError i desar in the request
			errors.add("dataInici",new ActionError("error.data.required"));
			saveErrors(request,errors);

		}

		//Data Fi
		if (notificacioEditForm.getNotificacio().getDataFi() == null) {
			// crear ActionError i desar in the request
			errors.add("dataFi",new ActionError("error.data.required"));
			saveErrors(request,errors);

		}


		//Comparar dates
		if (notificacioEditForm.getNotificacio().getDataInici() != null &&
				notificacioEditForm.getNotificacio().getDataFi() != null &&
				notificacioEditForm.getNotificacio().getDataInici().after(
						notificacioEditForm.getNotificacio().getDataFi())) {
			errors.add("dataInici", new ActionError("error.rangDates.dates"));
			saveErrors(request,errors);
		}



		//Si la notificacio es tipus Curs el curs es obligatoria
		if (notificacioEditForm.getNotificacio().getIdTipusNotificacio().equals(
				CodisMestre.CODI_TIPUS_NOTIFICACIO_CURS) &&
				notificacioEditForm.getNotificacio() instanceof NotificacioCurs &&
				((NotificacioCurs) notificacioEditForm.getNotificacio()).getIdCurs() == null) {
			// crear ActionError i desar in the request
			errors.add("curs",new ActionError("error.curs.required"));
			saveErrors(request,errors);
		}

		//Si la notificacio es tipus Aula l'aula es obligatoria
		if (notificacioEditForm.getNotificacio().getIdTipusNotificacio().equals(
				CodisMestre.CODI_TIPUS_NOTIFICACIO_AULA) &&
				notificacioEditForm.getNotificacio() instanceof  NotificacioAula &&
				((NotificacioAula) notificacioEditForm.getNotificacio()).getIdAula() == null) {
			// crear ActionError i desar in the request
			errors.add("aula",new ActionError("error.aula.required"));
			saveErrors(request,errors);
		}

		//Si la notificacio es tipus Aula assignatura es obligatoria
		if (notificacioEditForm.getNotificacio().getIdTipusNotificacio().equals(
				CodisMestre.CODI_TIPUS_NOTIFICACIO_ASSIGNATURA) &&
				notificacioEditForm.getNotificacio() instanceof NotificacioAssignatura &&
				((NotificacioAssignatura) notificacioEditForm.getNotificacio()).getIdCursAssignatura() == null) {
			// crear ActionError i desar in the request
			errors.add("assignatura",new ActionError("error.assignatura.required"));
			saveErrors(request,errors);
		}



		return errors;
	}


	/**
	 * Inicialitzar formulari
	 * @param request
	 * @param notificacioEditForm
	 * @throws Throwable
	 */
	private void inicialitzaEditNotificacioForm(HttpServletRequest request,
			EditNotificacioForm notificacioEditForm) throws Throwable {

		Notificacio[] notificacions = new Notificacio[1];
		notificacions[0] = notificacioEditForm.getNotificacio();

		inicialitzaEstats(request, notificacioEditForm, notificacions);
		inicialitzaTipusNotificacions(request, notificacioEditForm, notificacions, notificacions[0].getIdTipusNotificacio());
		inicialitzaAules(request, notificacioEditForm, notificacions);
		inicialitzaCursos(request, notificacioEditForm, notificacions);
		inicialitzaCursAssignatures(request, notificacioEditForm, notificacions);

		request.setAttribute("TipusNotificacio", notificacioEditForm.getSelectedTipusNotificacio());
	}

	/**
	 * Recuperem els diferents tipus de notificacions de la base de dades
	 * @return TipusNotificacio[]: Array de tipus de notificacions
	 */
	private TipusNotificacio[] recuperaTipusNotificacions(HashMap p_parametres) throws Throwable{
		TipusNotificacio[] arrTipusNotificacions = HelperUI.recuperaTipusNotificacions(p_parametres);
		
		TipusNotificacio[] tipusNotificacions = new TipusNotificacio[arrTipusNotificacions.length+1];


		//Creem un tipus de notificacio per poder seleccionar 'Qualsevol tipus'
		tipusNotificacions[0] = new TipusNotificacio();
		tipusNotificacions[0].setIdTipusNotificacio(new Integer(-1));
		tipusNotificacions[0].setDescripcio("Qualsevol Tipus Notificaci");

		for (int i=0;i<arrTipusNotificacions.length;i++) {
			tipusNotificacions[i+1] = (TipusNotificacio) arrTipusNotificacions[i];
		}
		return tipusNotificacions;

	}


	/**
	 * Inicialitzar els estats
	 * @param request
	 * @param notificacioEditForm
	 * @param notificacions
	 */
	private void inicialitzaEstats(HttpServletRequest request,
			EditNotificacioForm notificacioEditForm, Notificacio[] notificacions) {
		Estat[] arrEstats;
		arrEstats = HelperUI.recuperaEstats();
		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		notificacioEditForm.setEstats(estats);
		if (notificacions != null && notificacions.length > 0 && notificacions[0].getIdEstat() != null)
			notificacioEditForm.setSelectedEstat(notificacions[0].getIdEstat().getIdEstat());
		else if (notificacioEditForm.getSelectedEstat() == null)
			notificacioEditForm.setSelectedEstat(CodisMestre.CODI_ESTAT_ALTA);
		//set the collection in the request
		request.setAttribute("estats", estats);
	}

	/**
	 * Inicialitzar els tipus de notificacions
	 * @param request
	 * @param notificacioEditForm
	 * @param notificacions
	 * @param p_TipusNotificacio
	 * @throws Throwable
	 */
	private void inicialitzaTipusNotificacions(HttpServletRequest request,
			EditNotificacioForm notificacioEditForm, Notificacio[] notificacions, Integer p_TipusNotificacio)
	throws Throwable {
		TipusNotificacio[] arrTipusNotificacio;
		arrTipusNotificacio = HelperUI.recuperaTipusNotificacions(new HashMap());
		//define a dummy collection
		ArrayList tipusNotificacions = new ArrayList(Arrays.asList(arrTipusNotificacio));
		notificacioEditForm.setTipusNotificacions(tipusNotificacions);
		if (notificacions != null && notificacions.length > 0 && notificacions[0].getIdTipusNotificacio() != null)
			notificacioEditForm.setSelectedTipusNotificacio(notificacions[0].getIdTipusNotificacio());
		else if (notificacioEditForm.getSelectedTipusNotificacio() == null)
			notificacioEditForm.setSelectedTipusNotificacio(p_TipusNotificacio);

		//set the collection in the request
		request.setAttribute("tipusNotificacions", tipusNotificacions);
	}

	/**
	 * Inicialitzar cursos
	 * @param request
	 * @param notificacioEditForm
	 * @param notificacions
	 * @throws Throwable
	 */
	private void inicialitzaCursos(HttpServletRequest request,
			EditNotificacioForm notificacioEditForm, Notificacio[] notificacions)
	throws Throwable {
		Curs[] arrCurs;
		arrCurs = HelperUI.recuperaCursos(new HashMap());
		//define a dummy collection
		ArrayList cursos = new ArrayList(Arrays.asList(arrCurs));
		notificacioEditForm.setCursos(cursos);
		if (notificacions != null && notificacions.length > 0 &&
			notificacions[0] instanceof NotificacioCurs &&
				((NotificacioCurs) notificacions[0]).getIdCurs() != null)
			notificacioEditForm.setSelectedCurs(((NotificacioCurs) notificacions[0]).getIdCurs().getIdCurs());
		else
			notificacioEditForm.setSelectedCurs(null);

		//set the collection in the request
		request.setAttribute("cursos", cursos);
	}

	/**
	 * Inicialitzar aules
	 * @param request
	 * @param notificacioEditForm
	 * @param notificacions
	 * @throws Throwable
	 */
	private void inicialitzaAules(HttpServletRequest request,
			EditNotificacioForm notificacioEditForm, Notificacio[] notificacions)
	throws Throwable {
		Aula[] arrAula;
		arrAula = HelperUI.recuperaAules(new HashMap());
		//define a dummy collection
		ArrayList aules = new ArrayList(Arrays.asList(arrAula));
		notificacioEditForm.setAules(aules);
		if (notificacions != null && notificacions.length > 0 &&
				notificacions[0] instanceof NotificacioAula &&
				((NotificacioAula) notificacions[0]).getIdAula() != null)
			notificacioEditForm.setSelectedAula(((NotificacioAula) notificacions[0]).getIdAula().getIdAula());
		else
			notificacioEditForm.setSelectedAula(null);

		//set the collection in the request
		request.setAttribute("aules", aules);
	}

	/**
	 * Inicialitzar cursAssignatura
	 * @param request
	 * @param notificacioEditForm
	 * @param notificacions
	 * @throws Throwable
	 */
	private void inicialitzaCursAssignatures(HttpServletRequest request,
			EditNotificacioForm notificacioEditForm, Notificacio[] notificacions)
	throws Throwable {
		CursAssignatura[] arrCursAssignatura;
		arrCursAssignatura = HelperUI.recuperaCursAssignatures(new HashMap());
		//define a dummy collection
		ArrayList cursAssignatures = new ArrayList(Arrays.asList(arrCursAssignatura));
		notificacioEditForm.setCursAssignatures(cursAssignatures);
		if (notificacions != null && notificacions.length > 0 &&
				notificacions[0] instanceof NotificacioAssignatura &&
				((NotificacioAssignatura) notificacions[0]).getIdCursAssignatura() != null)
			notificacioEditForm.setSelectedCursAssignatura(((NotificacioAssignatura) notificacions[0]).getIdCursAssignatura().getIdCursAssignatura());
		else
			notificacioEditForm.setSelectedCursAssignatura(null);

		//set the collection in the request
		request.setAttribute("cursAssignatures", cursAssignatures);
	}

	/**
	 * Inicialitzar usuari connectat. Hem de passar els permissos depenent del tipus d'usuari
	 * @param request
	 */
	private void inicialitzaUsuariConnectat(HttpServletRequest request) {
		//Seguretat per notificacio
		Usuari usuariConnectat = ConfiguracioUsuari.getInstance().getUsuari();
		if (usuariConnectat instanceof UsuariProfessor) {
			request.setAttribute("PermisProfessor", "PermisProfessor");
		}
		else if (usuariConnectat instanceof UsuariAdministrador  ||
				usuariConnectat instanceof UsuariAdministratiu) {
			request.setAttribute(CodisMestre.PERMIS, CodisMestre.PERMIS);
		}
	}


}
