package com.uoc.tfc.UI.struts.action.qualificacio;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.actions.DispatchAction;

import com.uoc.tfc.Base.common.CodisMestre;
import com.uoc.tfc.Base.common.Helper;
import com.uoc.tfc.Base.manteniment.AnyAcademic;
import com.uoc.tfc.Base.manteniment.Assignatura;
import com.uoc.tfc.Base.manteniment.Aula;
import com.uoc.tfc.Base.manteniment.TipusQualificacio;
import com.uoc.tfc.Base.mestres.Estat;
import com.uoc.tfc.Base.mestres.TipusDocument;
import com.uoc.tfc.Base.qualificacio.Qualificacio;
import com.uoc.tfc.Base.usuari.UsuariAlumne;
import com.uoc.tfc.LN.dao.mestres.MestreDAO;
import com.uoc.tfc.LN.dao.qualificacio.QualificacioDAO;
import com.uoc.tfc.LN.db.HelperDB;
import com.uoc.tfc.UI.struts.action.common.HelperUI;
import com.uoc.tfc.UI.struts.form.qualificacio.EditQualificacioForm;

/**
 * Edici del formulari de les qualificacions
 * @author ojaen
 *
 */
public class EditQualificacioAction extends DispatchAction {

	/**
	 * Preparaci de l'edici del formulari
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward prepareEdit(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditQualificacioForm qualificacioEditForm = (EditQualificacioForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		QualificacioDAO qualificacioDAO = new QualificacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Qualificacio.CRITERI_QUALIFICACIO_ID_QUALIFICACIO, id);

		Qualificacio[] qualificacions = null;

		try {
			qualificacions = qualificacioDAO.findQualificacio(parametres);
			qualificacioEditForm.setQualificacio(qualificacions[0]);
			inicialitzaDades(request, qualificacioEditForm, qualificacions);

		} catch (Throwable e) {
			saveErrors(request, HelperDB.tractarError(e, "qualificacioError"));
			try {
				inicialitzaDadesInicialsCercaQualificacio(request);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "editQualificacioError"));
				return (mapping.findForward(CodisMestre.FAILURE));
			}
			return (mapping.findForward("failureCerca"));
		}

		return mapping.findForward("editQualificacio");
	}


	/**
	 * Inicialitzaci de les llistes al tornar a CercaQualificacio
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaDadesInicialsCercaQualificacio (
			HttpServletRequest request) throws Throwable {
		inicialitzaAnysAcademicsCerca(request);
		inicialitzaTipusDocumentsCerca(request);
		inicialitzaEstatsCerca(request);
		inicialitzaAulesCerca(request);
		inicialitzaAssignaturesCerca(request);
	}


	/**
	 * Inicialitzar les dades del formulari
	 * @param request
	 * @param qualificacioEditForm
	 * @param qualificacions
	 * @throws Throwable
	 */
	private void inicialitzaDades(HttpServletRequest request,
			EditQualificacioForm qualificacioEditForm, Qualificacio[] qualificacions)
	throws Throwable {
		inicialitzaAnysAcademics(request, qualificacioEditForm, qualificacions);
		inicialitzaAules(request, qualificacioEditForm, qualificacions);
		inicialitzaAssignatures(request, qualificacioEditForm, qualificacions);
		inicialitzaTipusQualificacions(request, qualificacioEditForm, qualificacions);
	}

	/**
	 * Inicialitzar editQualificacioForm
	 * @param qualificacioEditForm
	 * @param p_usuari
	 */
	private void inicialitzaForm(EditQualificacioForm qualificacioEditForm, UsuariAlumne p_usuari) {
		qualificacioEditForm.setQualificacio(new Qualificacio());
		qualificacioEditForm.getQualificacio().setIdAlumne(p_usuari);
	}

	/**
	 * Gravar una qualificaci
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward saveQualificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditQualificacioForm qualificacioEditForm = (EditQualificacioForm) form;

		QualificacioDAO qualificacioDAO = new QualificacioDAO();
		MestreDAO mestreDAO = new MestreDAO();

		UsuariAlumne usuari = null;

		try {
			//Cerquem l'any acadmic seleccionat i l'assignem a la qualificaci
			HashMap parametres = new HashMap();
			parametres.put(AnyAcademic.CRITERI_ANY_ACADEMIC_ID_ANY_ACADEMIC, qualificacioEditForm.getSelectedAnyAcademic());
			TipusQualificacio[] arrTipusQualificacions = recuperaTipusQualificacions(false);


			qualificacioEditForm.getQualificacio().setQualif1(getTipusQualificacio(
					arrTipusQualificacions, qualificacioEditForm.getSelectedQualificacio1()));

			qualificacioEditForm.getQualificacio().setQualif2(getTipusQualificacio(
					arrTipusQualificacions, qualificacioEditForm.getSelectedQualificacio2()));

			qualificacioEditForm.getQualificacio().setQualif3(getTipusQualificacio(
					arrTipusQualificacions, qualificacioEditForm.getSelectedQualificacio3()));

			qualificacioEditForm.getQualificacio().setRepesca1(getTipusQualificacio(
					arrTipusQualificacions, qualificacioEditForm.getSelectedQualificacioRepesca1()));

			qualificacioEditForm.getQualificacio().setRepesca2(getTipusQualificacio(
					arrTipusQualificacions, qualificacioEditForm.getSelectedQualificacioRepesca2()));

			qualificacioEditForm.getQualificacio().setQualifFinal(getTipusQualificacio(
					arrTipusQualificacions, qualificacioEditForm.getSelectedQualificacioFinal()));


			//Usuari
			usuari = qualificacioEditForm.getQualificacio().getIdAlumne();
			usuari.addQualificacio(qualificacioEditForm.getQualificacio());
			qualificacioDAO.gravar(qualificacioEditForm.getQualificacio());
			inicialitzaDadesInicialsCercaQualificacio(request);

		} catch (Throwable e) {
			try {
				if (usuari != null)
					usuari.removeQualificacio(qualificacioEditForm.getQualificacio());
				inicialitzaDades(request, qualificacioEditForm, null);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "editQualificacioError"));
				return (mapping.findForward(CodisMestre.FAILURE));

			}
			saveErrors(request, HelperDB.tractarError(e, "editQualificacioError"));
			return (mapping.findForward(CodisMestre.FAILURE));
		}


		return mapping.findForward("cercaQualificacio");
	}

	/**
	 * Consultar qualificacions
	 *
	 * @param mapping
	 * @param form
	 * @param request
	 * @param response
	 * @return ActionForward
	 */
	public ActionForward consultaQualificacio(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response) {
		if (Helper.isSessioExpirada(request))
			return (mapping.findForward(CodisMestre.SESSIO_EXPIRADA));

		EditQualificacioForm qualificacioEditForm = (EditQualificacioForm) form;

		Integer id = Integer.valueOf(request.getParameter("id"));
		QualificacioDAO qualificacioDAO = new QualificacioDAO();

		//Parametres
		HashMap parametres = new HashMap();
		parametres.put(Qualificacio.CRITERI_QUALIFICACIO_ID_QUALIFICACIO, id);

		Qualificacio[] qualificacions = null;

		try {
			qualificacions = qualificacioDAO.findQualificacio(parametres);
			qualificacioEditForm.setQualificacio(qualificacions[0]);

			inicialitzaAnysAcademics(request, qualificacioEditForm, qualificacions);
			inicialitzaAules(request, qualificacioEditForm, qualificacions);
			inicialitzaAssignatures(request, qualificacioEditForm, qualificacions);
			inicialitzaTipusQualificacions(request, qualificacioEditForm, qualificacions);


		} catch (Throwable e) {
			try {
				inicialitzaDadesInicialsCercaQualificacio(request);
			} catch (Throwable e1) {
				saveErrors(request, HelperDB.tractarError(e, "qualificacioError"));
				return (mapping.findForward("failureCerca"));
			}
			saveErrors(request, HelperDB.tractarError(e, "qualificacioError"));
			return (mapping.findForward("failureCerca"));
		}


		request.setAttribute(CodisMestre.CONSULTA, CodisMestre.CONSULTA);

		return mapping.findForward("editQualificacio");
	}


	/**
	 * Recuperem els diferents tipus de qualificaci de la base de dades
	 * @return TipusQualificacio[]: Array de tipus de qualificacio
	 */
	private TipusQualificacio[] recuperaTipusQualificacions(boolean crearCapQualificacio) {
		TipusQualificacio[] arrTipusQualificacions = HelperUI.recuperaTipusQualificacions(new HashMap());

		TipusQualificacio[] tipusQualificacions = new TipusQualificacio[arrTipusQualificacions.length+1];

		if (crearCapQualificacio) {
			//Creem un curs fictici per poder seleccionar 'Cap Curs Previ'
			tipusQualificacions[0] = new TipusQualificacio();
			tipusQualificacions[0].setCodiTipusQualificacio("0");
			tipusQualificacions[0].setDescripcio("Sense qualificar");
			tipusQualificacions[0].setIdTipusQualificacio(new Integer(0));

			for (int i = 0; i < arrTipusQualificacions.length; i++)
				tipusQualificacions[i+1] = arrTipusQualificacions[i];
			return tipusQualificacions;
		}
		else
			return arrTipusQualificacions;

	}


	/**
	 * Recuperem les aules de la base de dades
	 * @return Aula[]: Array de aules
	 */
	private Aula[] recuperaAules(HashMap p_parametres, boolean creaQualsevolAula) throws Throwable{

		Aula[] o = HelperUI.recuperaAules(p_parametres);

		Aula[] aules = new Aula[o.length+1];

		if (creaQualsevolAula) {
			//Creem un aula fictici per poder seleccionar 'Cap Aula Previ'
			aules[0] = new Aula();
			aules[0].setCodiAula("0");
			aules[0].setDescripcio("Qualsevol Aula");
			aules[0].setIdAula(new Integer(0));

			for (int i = 0; i < o.length; i++)
				aules[i+1] = o[i];
			return aules;
		}
		else
			return o;

	}

	/**
	 * Inicialitzar anys academics
	 * @param request
	 * @param qualificacioEditForm
	 * @param qualificacions
	 * @throws Throwable
	 */
	private void inicialitzaAnysAcademics(HttpServletRequest request,
			EditQualificacioForm qualificacioEditForm, Qualificacio[] qualificacions)
	throws Throwable {
		AnyAcademic[] arrAnyAcademic;
		arrAnyAcademic = HelperUI.recuperaAnysAcademics(new HashMap());
		//define a dummy collection
		ArrayList<AnyAcademic> anysAcademics = new ArrayList(Arrays.asList(arrAnyAcademic));
		qualificacioEditForm.setAnyAcademics(anysAcademics);
		if (qualificacions != null && qualificacions.length > 0 && qualificacions[0].getIdAnyAcademic() != null)
			qualificacioEditForm.setSelectedAnyAcademic(qualificacions[0].getIdAnyAcademic().getIdAnyAcademic());
		else if (qualificacioEditForm.getSelectedAnyAcademic() == null ||
				qualificacioEditForm.getSelectedAnyAcademic().intValue() == 0){
			//Ens arriba ordenat ordenat per data de l'any acadmic
			for (int i=0;i<anysAcademics.size();i++) {
				AnyAcademic aa = anysAcademics.get(i);
				if (aa.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedAnyAcademic(anysAcademics.get(0).getIdAnyAcademic());
					break;
				}
			}
		}

		//set the collection in the request
		request.setAttribute("anysAcademics", anysAcademics);

	}

	/**
	 * Inicialitzar aules
	 * @param request
	 * @param qualificacioEditForm
	 * @param qualificacions
	 * @throws Throwable
	 */
	private void inicialitzaAules(HttpServletRequest request,
			EditQualificacioForm qualificacioEditForm, Qualificacio[] qualificacions)
	throws Throwable {
		Aula[] arrAula;
		arrAula = recuperaAules(new HashMap(), false);
		//define a dummy collection
		ArrayList<Aula> aules = new ArrayList(Arrays.asList(arrAula));
		qualificacioEditForm.setAules(aules);
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getIdAsigAulaProf() != null &&
				qualificacions[0].getIdAsigAulaProf().getIdAula() != null)
			qualificacioEditForm.setSelectedAula(qualificacions[0].getIdAsigAulaProf().getIdAula().getIdAula());
		else if (qualificacioEditForm.getSelectedAula() == null ||
				qualificacioEditForm.getSelectedAula().intValue() == 0){
			//Ens arriba ordenat ordenat per data de l'any acadmic
			for (int i=0;i<aules.size();i++) {
				Aula aa = aules.get(i);
				if (aa.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedAula(aules.get(0).getIdAula());
					break;
				}
			}
		}

		//set the collection in the request
		request.setAttribute("aules", aules);

	}

	/**
	 * Inicialitzar assignatures
	 * @param request
	 * @param qualificacioEditForm
	 * @param qualificacions
	 * @throws Throwable
	 */
	private void inicialitzaAssignatures(HttpServletRequest request,
			EditQualificacioForm qualificacioEditForm, Qualificacio[] qualificacions)
	throws Throwable {
		Assignatura[] arrAssignatura;
		arrAssignatura = recuperaAssignatures(new HashMap(), false);
		//define a dummy collection
		ArrayList<Assignatura> assignatures = new ArrayList(Arrays.asList(arrAssignatura));
		qualificacioEditForm.setAssignatures(assignatures);
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getIdAsigAulaProf() != null)
			qualificacioEditForm.setSelectedAssignatura(
					qualificacions[0].getIdAsigAulaProf().getIdAssignatura().getIdAssignatura());
		else if (qualificacioEditForm.getSelectedAssignatura() == null ||
				qualificacioEditForm.getSelectedAssignatura().intValue() == 0){
			//Ens arriba ordenat ordenat per data de l'any acadmic
			for (int i=0;i<assignatures.size();i++) {
				Assignatura assignatura = assignatures.get(i);
				if (assignatura.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedAssignatura(assignatures.get(0).getIdAssignatura());
					break;
				}
			}
		}

		//set the collection in the request
		request.setAttribute("assignatures", assignatures);

	}

	/**
	 * Inicialitzar tipus de qualificacions
	 * @param request
	 * @param qualificacioEditForm
	 * @param qualificacions
	 * @throws Throwable
	 */
	private void inicialitzaTipusQualificacions(HttpServletRequest request,
			EditQualificacioForm qualificacioEditForm, Qualificacio[] qualificacions)
	throws Throwable {
		TipusQualificacio[] arrTipusQualificacio;
		arrTipusQualificacio = recuperaTipusQualificacions(true);
		//define a dummy collection
		ArrayList<TipusQualificacio> tipusQualificacions = new ArrayList(Arrays.asList(arrTipusQualificacio));
		qualificacioEditForm.setTipusQualificacions(tipusQualificacions);


		//Qualificacio 1
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getQualif1() != null)
			qualificacioEditForm.setSelectedQualificacio1(
					qualificacions[0].getQualif1().getIdTipusQualificacio());
		else {
			for (int i=0;i<tipusQualificacions.size();i++) {
				TipusQualificacio tipusQualificacio = tipusQualificacions.get(i);
				if (tipusQualificacio.getIdEstat() == null ||
						tipusQualificacio.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedQualificacio1(tipusQualificacions.get(0).getIdTipusQualificacio());
					break;
				}
			}
		}

		//Qualificacio 2
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getQualif2() != null)
			qualificacioEditForm.setSelectedQualificacio2(
					qualificacions[0].getQualif2().getIdTipusQualificacio());
		else {
			for (int i=0;i<tipusQualificacions.size();i++) {
				TipusQualificacio tipusQualificacio = tipusQualificacions.get(i);
				if (tipusQualificacio.getIdEstat() == null ||
						tipusQualificacio.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedQualificacio2(tipusQualificacions.get(0).getIdTipusQualificacio());
					break;
				}
			}
		}

		//Qualificacio 3
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getQualif3() != null)
			qualificacioEditForm.setSelectedQualificacio3(
					qualificacions[0].getQualif3().getIdTipusQualificacio());
		else {
			for (int i=0;i<tipusQualificacions.size();i++) {
				TipusQualificacio tipusQualificacio = tipusQualificacions.get(i);
				if (tipusQualificacio.getIdEstat() == null ||
						tipusQualificacio.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedQualificacio3(tipusQualificacions.get(0).getIdTipusQualificacio());
					break;
				}
			}
		}


		//Repesca 1
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getRepesca1() != null )
			qualificacioEditForm.setSelectedQualificacioRepesca1(
					qualificacions[0].getRepesca1().getIdTipusQualificacio());
		else {
			for (int i=0;i<tipusQualificacions.size();i++) {
				TipusQualificacio tipusQualificacio = tipusQualificacions.get(i);
				if (tipusQualificacio.getIdEstat() == null ||
						tipusQualificacio.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedQualificacioRepesca1(tipusQualificacions.get(0).getIdTipusQualificacio());
					break;
				}
			}
		}

		//Repesca 2
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getRepesca2() != null )
			qualificacioEditForm.setSelectedQualificacioRepesca2(
					qualificacions[0].getRepesca2().getIdTipusQualificacio());
		else {
			for (int i=0;i<tipusQualificacions.size();i++) {
				TipusQualificacio tipusQualificacio = tipusQualificacions.get(i);
				if (tipusQualificacio.getIdEstat() == null ||
						tipusQualificacio.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedQualificacioRepesca2(tipusQualificacions.get(0).getIdTipusQualificacio());
					break;
				}
			}
		}


		//Qualificacio final
		if (qualificacions != null && qualificacions.length > 0 &&
				qualificacions[0].getQualifFinal() != null )
			qualificacioEditForm.setSelectedQualificacioFinal(
					qualificacions[0].getQualifFinal().getIdTipusQualificacio());
		else {
			for (int i=0;i<tipusQualificacions.size();i++) {
				TipusQualificacio tipusQualificacio = tipusQualificacions.get(i);
				if (tipusQualificacio.getIdEstat() == null ||
						tipusQualificacio.getIdEstat().getIdEstat().equals(CodisMestre.CODI_ESTAT_ALTA)) {
					qualificacioEditForm.setSelectedQualificacioFinal(tipusQualificacions.get(0).getIdTipusQualificacio());
					break;
				}
			}
		}


		//set the collection in the request
		request.setAttribute("qualificacions", tipusQualificacions);

	}


	/**
	 * Inicialitzar estats
	 * @param request
	 */
	private void inicialitzaEstatsCerca(HttpServletRequest request) {
		//Carrega opcions d'estat
		Estat[] arrEstats = HelperUI.recuperaEstats();

		//define a dummy collection
		ArrayList estats = new ArrayList(Arrays.asList(arrEstats));
		//set the collection in the request
		request.setAttribute("estats", estats);
	}

	/**
	 * Inicialitzar tipus de documents
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaTipusDocumentsCerca(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus de documents
		TipusDocument[] arrTipusDocuments = HelperUI.recuperaTipusDocuments(new HashMap());

		ArrayList tipusDocuments = new ArrayList(Arrays.asList(arrTipusDocuments));
		//set the collection in the request
		request.setAttribute("tipusDocuments", tipusDocuments);
	}

	/**
	 * Inicialitzar anys academics
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaAnysAcademicsCerca(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus d'usuaris
		AnyAcademic[] arrAnyAcademics = HelperUI.recuperaAnysAcademics(new HashMap());

		ArrayList anyAcademics = new ArrayList(Arrays.asList(arrAnyAcademics));
		//set the collection in the request
		request.setAttribute("anysAcademics", anyAcademics);
	}

	/**
	 * Inicialitzar aules
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaAulesCerca(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus de documents
		Aula[] arrAules = recuperaAules(new HashMap(), true);

		ArrayList aules = new ArrayList(Arrays.asList(arrAules));
		//set the collection in the request
		request.setAttribute("aules", aules);
	}

	/**
	 * Inicialitzar assignatures
	 * @param request
	 * @throws Throwable
	 */
	private void inicialitzaAssignaturesCerca(HttpServletRequest request)
	throws Throwable {
		//Carrega opcions de tipus de documents
		Assignatura[] arrAssignatures = recuperaAssignatures(new HashMap(), true);

		ArrayList assignatures = new ArrayList(Arrays.asList(arrAssignatures));
		//set the collection in the request
		request.setAttribute("assignatures", assignatures);
	}


	/**
	 * Recuperem els assignatures de la base de dades
	 * @return Assignatura[]: Array de assignatures
	 */
	private Assignatura[] recuperaAssignatures(HashMap p_parametres, boolean creaCapassignatura) throws Throwable{

		Assignatura[] o = HelperUI.recuperaAssignatura(p_parametres);

		Assignatura[] assignatures = new Assignatura[o.length+1];

		if (creaCapassignatura) {
			//Creem un assignatura fictici per poder seleccionar 'Cap Assignatura Previ'
			assignatures[0] = new Assignatura();
			assignatures[0].setCodiAssignatura("0");
			assignatures[0].setDescripcio("Qualsevol Assignatura");
			assignatures[0].setIdAssignatura(new Integer(0));

			for (int i = 0; i < o.length; i++)
				assignatures[i+1] = o[i];
			return assignatures;
		}
		else
			return o;

	}


	/**
	 * Retorna el TipusQualificacio depenent del id de la qualificacio
	 * @param arrTipusQualificacions
	 * @param selectedQualificacio
	 * @return TipusQualificacio
	 */
	private TipusQualificacio getTipusQualificacio(
			TipusQualificacio[] arrTipusQualificacions,
			Integer selectedQualificacio) {
		if (Helper.validarArray(arrTipusQualificacions) && selectedQualificacio != null &&
				selectedQualificacio.intValue() > 0) {
			for (int i=0;i<arrTipusQualificacions.length;i++) {
				if (arrTipusQualificacions[i].getIdTipusQualificacio().equals(selectedQualificacio))
					return arrTipusQualificacions[i];
			}
		}
		return null;
	}

}
